#import "ApplistTableView.h"
#import "ViewApplist.h"
#import "FileAttributeExtension.h"

@implementation ApplistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

    // Set sort descriptors
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES];
    [applistArrayController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
    [sortDescriptor release];

	// set registerForDraggedTypes
	[self registerForDraggedTypes:[NSArray arrayWithObjects:ApplistPboardType, NSFilenamesPboardType, nil]];

}

//=======================================================================
// bind to tableview column
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
        objectValueForTableColumn:(NSTableColumn *)aTableColumn
        row:(int)rowIndex
{

	// get record from applistArrayController
	id record = [[applistArrayController arrangedObjects] objectAtIndex:rowIndex];

	// image
	if([[aTableColumn identifier] isEqualToString:@"appIcon"]){

		NSString *filePath = [record valueForKey:@"filePath"];
		id iconImage = nil;
		if([self checkFileExist:filePath] != FILE_EXIST_NOT_FOUND){
			iconImage = [[NSWorkspace sharedWorkspace] iconForFile:filePath];
			[iconImage setSize:NSMakeSize(16,16)];
			[iconImage setScalesWhenResized:YES];
		}
		return iconImage;
	}

    return nil;
}

//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[applistArrayController arrangedObjects] count];
}
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// writeRowsWithIndexes
//------------------------------------
- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard {

    BOOL success = NO;
	NSFileManager *myFile = [NSFileManager defaultManager];

	NSArray *objects = [[applistArrayController arrangedObjects] objectsAtIndexes:rowIndexes];
 	NSMutableArray *items = [NSMutableArray array];
 	NSMutableArray *filePaths = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		NSManagedObjectID *objectID = [item objectID];
		NSURL *representedURL = [objectID URIRepresentation];
		[items addObject:representedURL];
		// for NSFilenamesPboardType
		if([myFile fileExistsAtPath:[item valueForKey:@"filePath"] isDirectory:nil]){
			[filePaths addObject:[item valueForKey:@"filePath"]];
		}
	}

	[pboard declareTypes:[NSArray arrayWithObjects:ApplistPboardType, nil] owner:nil];
	[pboard addTypes:[NSArray arrayWithObjects:ApplistPboardType, nil] owner:nil];
	[pboard setString:[items componentsJoinedByString:@","] forType:ApplistPboardType];        

	// for NSFilenamesPboardType
	if([filePaths count] > 0){
		[pboard setPropertyList:filePaths forType:NSFilenamesPboardType];        
	}

	if([items count] > 0){
		success = YES;
	}

	return success;

}

//------------------------------------
// validateDrop
//------------------------------------
- (NSDragOperation)tableView:(NSTableView*)tableView 
        validateDrop:(id <NSDraggingInfo>)info 
        proposedRow:(int)row 
        proposedDropOperation:(NSTableViewDropOperation)operation
{

    // Get pboard
    NSPasteboard*   pboard;
    pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

    if (!pboard) {
        return NSDragOperationNone;
    }

	// ignore other
	if (draggingSource != self) {
		return NSDragOperationNone;
	}

	// Check pboard type
    if ([[pboard types] containsObject:ApplistPboardType]){
		// drop above
		if(operation == NSTableViewDropAbove) {
			return NSDragOperationMove;
		}
	}

	return NSDragOperationNone;

}
//------------------------------------
// acceptDrop
//------------------------------------
- (BOOL)tableView:(NSTableView*)tableView 
						acceptDrop:(id <NSDraggingInfo>)info 
						row:(int)row 
						dropOperation:(NSTableViewDropOperation)operation
{
	
	NSPasteboard *pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

    if (!pboard) {
        return NO;
    }

	// ignore other
	if (draggingSource != self) {
		return NO;
	}

	// Check pboard type
	if ([[pboard types] containsObject:ApplistPboardType]){

		// get array from paste board string
		NSArray *items = [[pboard stringForType:ApplistPboardType] componentsSeparatedByString:@","];
		if([items count] > 0){
			[viewApplist sortItems:items targetRow:row];
			return YES;
		}
	}

	return NO;
}
//------------------------------------
// textDidEndEditing
//------------------------------------
- (void)textDidEndEditing:(NSNotification *)notification
{ 
    if ([[[notification userInfo] objectForKey:@"NSTextMovement"] intValue] == NSReturnTextMovement)
    {
        NSMutableDictionary * newUserInfo;
        NSNotification * newNotification;
        newUserInfo = [NSMutableDictionary dictionaryWithDictionary:[notification userInfo]]; 
        [newUserInfo setObject:[NSNumber numberWithInt:NSIllegalTextMovement] forKey:@"NSTextMovement"];
        newNotification = [NSNotification notificationWithName:[notification name] object:[notification object] userInfo:newUserInfo];
        [super textDidEndEditing:newNotification];
        [[self window] makeFirstResponder:self]; 
    } else {
        [super textDidEndEditing:notification]; 
    }
}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[super dealloc];
}
@end
