#import "ChapterExtension.h"
#import "FileAttributeExtension.h"
#import "ImageAttributeExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation NSObject(chapterExtension_)

//------------------------------------
// getChapterHomePath
//------------------------------------
- (NSString*)getChapterHomePath
{
	NSString* filePath = [self getAppSupportFolderPath];

	if([filePath isEqualToString:@""]){
		return @"";
	}else{
		return [NSString stringWithFormat: @"%@/%@", filePath, @"chapters"];
	}

}
//------------------------------------
// getChapterFolderPath
//------------------------------------
- (NSString*)getChapterFolderPath:(NSString*)uniqueId
{
	if([uniqueId isEqualToString:@""]){
		return @"";
	}

	NSString* filePath = [self getChapterHomePath];
	if([filePath isEqualToString:@""]){
		return @"";
	}else{
		return [NSString stringWithFormat: @"%@/%@", filePath, uniqueId];
	}

}
//------------------------------------
// getChapterImagePath
//------------------------------------
- (NSString*)getChapterImagePath:(NSString*)uniqueId chapterId:(NSString*)chapterId
{
	if([uniqueId isEqualToString:@""] || [chapterId isEqualToString:@""]){
		return @"";
	}

	NSString* filePath = [self getChapterFolderPath:uniqueId];
	if([filePath isEqualToString:@""]){
		return @"";
	}else{
		return [NSString stringWithFormat: @"%@/%@.png", filePath, chapterId];
	}
}
//------------------------------------
// getChapterImage
//------------------------------------
- (NSImage*)getChapterImage:(NSString*)filePath isSound:(BOOL)isSound
{

	NSString *imagePath;

	if([[NSFileManager defaultManager] fileExistsAtPath:filePath] == YES){
		imagePath = filePath;
	}else{
		if(isSound == YES){
			imagePath = [[NSBundle mainBundle] pathForResource:@"image_sound" ofType:@"png"];
		}else{
			imagePath = [[NSBundle mainBundle] pathForResource:@"image_noimage" ofType:@"png"];
		}
	}

	NSImage *image = [[[NSImage alloc] initByReferencingFile:imagePath] autorelease];	

	return image;
	
}
//------------------------------------
// checkAndCreateChapterFolder
//------------------------------------
- (BOOL)checkAndCreateChapterFolder:(NSString*)uniqueId showAlert:(BOOL)showAlert
{
	// Application support folder is not availble
	if([self checkFileIsCollect:[self getAppSupportFolderPath] fileType:@"folder"] != FILE_COLLECT_MATCH){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Application support folder is not available."
						infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getAppSupportFolderPath]]
						btnList:@"Cancel"
			];
		}
		return NO;
	}

	// Chapter home folder is not availble
	if([self checkAndCreateFolder:[self getChapterHomePath]] != FOLDER_CHECK_SUCCESS){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Chapter folder is not available."
						infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getChapterHomePath]]
						btnList:@"Cancel"
			];
		}
		return NO;
	}

	// Chapter folder is not availble
	if([self checkAndCreateFolder:[self getChapterFolderPath:uniqueId]] != FOLDER_CHECK_SUCCESS){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Chapter folder is not available."
						infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getChapterFolderPath:uniqueId]]
						btnList:@"Cancel"
			];
		}
		return NO;
	}

	return YES;

}
//------------------------------------
// createChapterImage
//------------------------------------
- (BOOL)createChapterImage:(NSImage*)image
					uniqueId:(NSString*)uniqueId
					chapterId:(NSString*)chapterId
					isCheckFolder:(BOOL)isCheckFolder
					showAlert:(BOOL)showAlert
{

	// image is null
	if(!image){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Image is null."
						infoText:@""
						btnList:@"Cancel"
			];
		}
		return NO;
	}


	// uniqueId is null
	if(!uniqueId || [uniqueId isEqualToString:@""]){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"UniqueId is null."
						infoText:@""
						btnList:@"Cancel"
			];
		}
		return NO;
	}

	// check and create chapter folder
	if(isCheckFolder == YES){
		if([self checkAndCreateChapterFolder:uniqueId showAlert:showAlert] == NO){
			return NO;
		}
	}

	NSSize chapterSize = [self defaultChapterSize];
	NSString *chapterPath = [self getChapterImagePath:uniqueId chapterId:chapterId];

	if(!chapterPath || [chapterPath isEqualToString:@""]){
		return NO;
	}

	// larger than chapterSize
	if([image size].width > chapterSize.width || [image size].height > chapterSize.height){
		[image setSize:[self setScaledImageSize:[image size] frameSize:chapterSize]];
		[image setScalesWhenResized:YES];
	}

	return [self saveImageAsPng:image filePath:chapterPath Interlaced:YES];

}

//------------------------------------
// removeChapterImage
//------------------------------------
- (BOOL)removeChapterImage:(NSString*)uniqueId chapterId:(NSString*)chapterId
{

	NSFileManager *myFile = [NSFileManager defaultManager];

	// uniqueId is null
	if(!uniqueId || [uniqueId isEqualToString:@""]){
		return NO;
	}

	// chapterId is null
	if(!chapterId || [chapterId isEqualToString:@""]){
		return NO;
	}

	// create chapter path
	NSString *filePath = [self getChapterImagePath:uniqueId chapterId:chapterId];

	// no path
	if([filePath isEqualToString:@""]){
		return NO;
	}

	BOOL isDir;
	// already not found
	if(![myFile fileExistsAtPath:filePath isDirectory:&isDir]){
		return YES;
	}
	// is directory
	if(isDir == YES){
		return NO;
	}

//	NSLog(@"filePath=%@", filePath);

	// remove file
	return [myFile removeFileAtPath:filePath handler:self];

}
//------------------------------------
// removeChapterFolder
//------------------------------------
- (BOOL)removeChapterFolder:(NSString*)uniqueId
{

	NSFileManager *myFile = [NSFileManager defaultManager];

	// create folder path
	NSString *filePath = [self getChapterFolderPath:uniqueId];
	if([filePath isEqualToString:@""]){
		return NO;
	}

	// already not found
	BOOL isDir;
	if(![myFile fileExistsAtPath:filePath isDirectory:&isDir]){
		return YES;
	}
	// is not directory
	if(isDir == NO){
		return NO;
	}

//	NSLog(@"filePath=%@", filePath);

	// remove folder
	return [myFile removeFileAtPath:filePath handler:self];

}
@end
