#import "ChapterMatrix.h"
#import "ChapterMatrixCell.h"
#import "ViewChapter.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "FileAttributeExtension.h"
#import "CellAttributeExtension.h"

static int	CELL_PADDING_WIDTH = 5;
static int	CELL_PADDING_HEIGHT = 10;

@implementation ChapterMatrix

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// init
	[self setStartIndex:0];
	[self setEndIndex:0];
	[self setSelectedIndex:0];
	[self setSelectedObject:nil];

	// delete focus ring
	[btnPlay setFocusRingType:NSFocusRingTypeNone];
	[btnAdd setFocusRingType:NSFocusRingTypeNone];
	[btnRemove setFocusRingType:NSFocusRingTypeNone];
	[btnGear setFocusRingType:NSFocusRingTypeNone];

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];

	// set cell size
	[self setCellSize:[self calcCellSize:[[self enclosingScrollView] contentSize] cols:[self maxCols]]];
	[self setIntercellSpacing:NSMakeSize(CELL_PADDING_WIDTH, CELL_PADDING_HEIGHT)];

	[self changePlayButtonEnable];
	[self changeAddButtonEnable];
	[self changeRemoveButtonEnable];
	[self changeGearButtonEnable];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(itemChapterDidChanged:) name:ITEM_NOTIF_CHAPTER_DID_CHANGED object:nil];

}
//------------------------------------
// changeMatrix
//------------------------------------
- (IBAction)changeMatrix:(id)sender
{

//	int startIndex = [self startIndex];
	int startIndex = 0;
	int lastIndex = [self lastIndex];

	[self createMatrix:startIndex endIndex:lastIndex];

	[self updateSelectedCell];

	[self changePlayButtonEnable];
	[self changeAddButtonEnable];
	[self changeRemoveButtonEnable];
	[self changeGearButtonEnable];

	[self setNeedsDisplay:YES];

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// changePlayButtonEnable
//------------------------------------
- (void)changePlayButtonEnable
{
	if([self totalResults] > 0 && [self selectedIndex] >= 0){
		[btnPlay setEnabled:YES];
	}else{
		[btnPlay setEnabled:NO];
	}
}
//------------------------------------
// changeAddButtonEnable
//------------------------------------
- (void)changeAddButtonEnable
{
	[btnAdd setEnabled:[viewChapter canCreateChapter]];
}
//------------------------------------
// changeRemoveButtonEnable
//------------------------------------
- (void)changeRemoveButtonEnable
{
	if([self totalResults] > 0 && [self selectedIndex] >= 0){
		[btnRemove setEnabled:YES];
	}else{
		[btnRemove setEnabled:NO];
	}
}
//------------------------------------
// changeGearButtonEnable
//------------------------------------
- (void)changeGearButtonEnable
{
	[btnGear setEnabled:[viewChapter canCreateChapter]];
}

//------------------------------------
// createMatrix
//------------------------------------
- (void)createMatrix:(int)startIndex endIndex:(int)endIndex
{

	int lastIndex = [self lastIndex];

	// over last index
	if(startIndex > lastIndex){
		startIndex = 0;
	}
	// over last index
	if(endIndex > lastIndex){
		endIndex = lastIndex;
	}

	// data count for matrix
	int dataCount = endIndex - startIndex + 1;
	// minus value
	if(dataCount < 0){
		dataCount = 0;
	}

	// set max rows & cols
	int maxCols = [self maxCols];
	int maxRows = 0;
	int col = 0;
	int row = 0;
	int itemIndex = -1;
	int i;

	// null matrix
	if(dataCount <= 0){
		maxCols = 0;
		maxRows = 0;
	}
	// set maxRows
	else{
		maxRows = dataCount / maxCols;
		if((dataCount % maxCols) != 0){
			maxRows++;
		}
	}

	// set startIndex
	[self setStartIndex:startIndex];
	[self setEndIndex:endIndex];

	// reset matrix cells
	[self renewRows:maxRows columns:maxCols];

	id record;

	for (i = startIndex; i <= endIndex; i++){

		record = [[viewChapter chapterList] objectAtIndex:i];

		if(col > maxCols - 1){
			row++;
			col = 0;
		}

		// create image cell
//		itemIndex++;
		itemIndex = i;
		ChapterMatrixCell *itemCell = [[ChapterMatrixCell alloc] initWithItemObject:record
																	itemIndex:itemIndex
																	isEnabled:YES
																	isExist:YES
									];

		// put cell
		[self putCell:itemCell atRow:row column:col];
		[itemCell release];
		col++;

	}

	// set null cell for rest
	// last row
	for (i = col; i < maxCols; i++){

		// already null cell
		if([(ChapterMatrixCell*)[self cellAtRow:row column:i] isEnabled] == NO){
			continue;
		}

		// create null cell
		ChapterMatrixCell *itemCell = [[ChapterMatrixCell alloc] initWithItemObject:nil
																	itemIndex:-1
																	isEnabled:NO
																	isExist:NO
									];
		[self putCell:itemCell atRow:row column:i];
		[itemCell release];
	}

	[self sizeToCells];

}
//------------------------------------
// clearMatrix
//------------------------------------
- (void)clearMatrix
{

	[self setSelectedObject:nil];

	int i;
	// remove matrix cells
	for (i = [self numberOfRows] - 1; i >= 0; i--){
		[self removeRow:i];
	}

	[self sizeToCells];

}
//------------------------------------
// updateSelectedCell
//------------------------------------
- (void)updateSelectedCell
{

	// clear selectedObject
	[self setSelectedObject:nil];

	// check selectedIndex
	int selectedIndex = [self selectedIndex];
	if(selectedIndex < 0){
		selectedIndex = 0;
	}
	else if(selectedIndex > [self lastIndex]){
		selectedIndex = [self lastIndex];
	}
	[self setSelectedIndex:selectedIndex];

//	NSLog(@"selectedIndex=%d", selectedIndex);

	// select cell
	int col = 0;
	int row = 0;
	int maxCols = [self maxCols];
	int i;
	int itemIndex;
	id itemObject = nil;

	// get max count
	int curRows, curCols;
	[self getNumberOfRows:&curRows columns:&curCols];
	int maxCount = curRows * curCols;

	id record;

	for(i = 0; i < [[self cells] count] && i < maxCount; i++){

		record = [[self cells] objectAtIndex:i];

		// null cell
		if([record isEnabled] == NO){
			continue;
		}

		itemIndex = [record itemIndex];
		itemObject = [record itemObject];
//		NSLog(@"itemIndex = %d", itemIndex);

		// select
		if(itemIndex == selectedIndex){
			[record setIsSelected:YES];
			[self setSelectedObject:itemObject];

		}else{
			[record setIsSelected:NO];
		}

		// adjust scroll
		if(itemIndex >= 0 && itemIndex == selectedIndex){
			row = i / maxCols;
			col = i % maxCols;
//			NSLog(@"itemIndex = %d", itemIndex);
//			NSLog(@"row=%d, col=%d", row, col);
			[self adjustScrollPosition:NSMakePoint(col, row)];
		}
	}
}
//------------------------------------
// selectMovedCell
//------------------------------------
- (BOOL)selectMovedCell:(int)arrow isShift:(BOOL)isShift isCmd:(BOOL)isCmd
{

	int selectedIndex = [self selectedIndex];
	int maxCols = [self maxCols];

	//
	// adjust for key up / down
	//
	// get max cell count
	int curRows, curCols;
	[self getNumberOfRows:&curRows columns:&curCols];
//	int maxCells = curRows * curCols;

	// up
	if(arrow == KEY_ARROW_UP){
		selectedIndex -= maxCols;
	}
	// down
	else if(arrow == KEY_ARROW_DOWN){
		selectedIndex += maxCols;
	}
	// left
	else if(arrow == KEY_ARROW_LEFT){
		selectedIndex--;
	}
	// right
	else if(arrow == KEY_ARROW_RIGHT){
		selectedIndex++;
	}

	// selectIndexCell
	return [self selectIndexCell:selectedIndex isShift:isShift isCmd:isCmd];

}
//------------------------------------
// selectIndexCell
//------------------------------------
- (BOOL)selectIndexCell:(int)index isShift:(BOOL)isShift isCmd:(BOOL)isCmd
{

	int lastIndex = [self lastIndex];

	BOOL isSelect = NO;

	if(index >= 0 && index <= lastIndex){
		[self changeSelectionIndexes:index isShift:isShift isCmd:isCmd];
		isSelect = YES;
	}

	return isSelect;
}
//------------------------------------
// changeSelectionIndexes
//------------------------------------
- (void)changeSelectionIndexes:(int)index
						isShift:(BOOL)isShift
						isCmd:(BOOL)isCmd
{

	[self setSelectedIndex:index];

	int startIndex = [self startIndex];
	int endIndex = [self endIndex];
	if(index >= startIndex && index <= endIndex){

		[self updateSelectedCell];

		[self changePlayButtonEnable];
		[self changeRemoveButtonEnable];

		[self setNeedsDisplay:YES];

	}else{
//		[self changeMatrix:nil];
	}

}
//------------------------------------
// adjustScrollPosition
//------------------------------------
- (void)adjustScrollPosition:(NSPoint)cellPosition
{

	NSScrollView *scrollView = [self enclosingScrollView];
	NSClipView *clipView = [scrollView contentView];

	NSRect clipRect = [clipView bounds];
	NSSize cellSize = [self cellSize];

	NSPoint cellPoint = NSZeroPoint;
	cellPoint.y = ((cellSize.height + CELL_PADDING_HEIGHT) * cellPosition.y);
//	NSLog(@"cellPoint=%f, clipRect=%f",cellPoint.y, clipRect.origin.y);
	if(cellPoint.y < clipRect.origin.y){
		[clipView scrollToPoint:cellPoint];
		[scrollView reflectScrolledClipView:clipView];
	}
	else if(cellPoint.y + cellSize.height > clipRect.origin.y + clipRect.size.height){	
//		NSLog(@"diff=%f", (cellPoint.y + cellSize.height) - (clipRect.origin.y + clipRect.size.height));
		cellPoint.y = clipRect.origin.y + ((cellPoint.y + cellSize.height) - (clipRect.origin.y + clipRect.size.height));
		[clipView scrollToPoint:cellPoint];
		[scrollView reflectScrolledClipView:clipView];
	}
}
//------------------------------------
// calcCellSize
//------------------------------------
- (NSSize)calcCellSize:(NSSize)frameSize cols:(int)cols
{
	float cellWidth = (frameSize.width - (CELL_PADDING_WIDTH * (cols - 1))) / cols;
	float cellHeight = cellWidth;
	return NSMakeSize(cellWidth, cellHeight);
}
//------------------------------------
// itemChapterDidChanged
//------------------------------------
-(void)itemChapterDidChanged:(NSNotification *)notification
{

	// force create matrix
	[self createMatrix:[self startIndex] endIndex:[self lastIndex]];

	[self updateSelectedCell];

	[self changePlayButtonEnable];
	[self changeAddButtonEnable];
	[self changeRemoveButtonEnable];
	[self changeGearButtonEnable];
	[self setNeedsDisplay:YES];
}

//------------------------------------
// setFrame
//------------------------------------
- (void)setFrame:(NSRect)frame
{

	// set cell size
	[self setCellSize:[self calcCellSize:[[self enclosingScrollView] contentSize] cols:[self maxCols]]];
	[self sizeToCells];

//	[super setFrame:frame];

}
/*
//------------------------------------
// viewDidEndLiveResize
//------------------------------------
- (void)viewDidEndLiveResize
{
//	[self setNeedsDisplay:YES];
//	[self changeMatrix];
}
*/
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)event
{

	BOOL isAction = NO;
	BOOL isShift = NO;
	BOOL isCmd = NO;

	NSString *keys = [event charactersIgnoringModifiers];

	if (keys && [keys length] > 0){

		unichar c = [keys characterAtIndex:0];

/*
		// get modifierFlags
		if([event modifierFlags] & NSShiftKeyMask){
			isShift = YES;
		}
		if([event modifierFlags] & NSCommandKeyMask){
			isCmd = YES;
		}
*/		
		if(c == NSUpArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_UP isShift:isShift isCmd:isCmd];
			isAction = YES;
		}
		else if(c == NSDownArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_DOWN isShift:isShift isCmd:isCmd];
			isAction = YES;
		}
		else if(c == NSLeftArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_LEFT isShift:isShift isCmd:isCmd];
			isAction = YES;
		}
		else if(c == NSRightArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_RIGHT isShift:isShift isCmd:isCmd];
			isAction = YES;
		}

	}

	// return key
	if([event keyCode] == 36){
		[viewChapter playItemChapter:nil];
		isAction = YES;
	}

	if(isAction == NO){
		[super keyDown:event];
	}

}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)event
{

	int row,col;

	// get cell point
	NSPoint mouseDownPoint = [self convertPoint:[event locationInWindow] fromView:nil];
	[self getRow:&row column:&col forPoint:mouseDownPoint];	

	// get clicked cell
	ChapterMatrixCell *itemCell = [self cellAtRow:row column:col];

	// enabled cell
	if(itemCell != nil && [itemCell isEnabled] == YES){

		int selectedIndex = [self selectedIndex];

		int itemIndex = [itemCell itemIndex];
		BOOL isContext = NO;
		BOOL isChange = NO;

		// changed index
		if(selectedIndex != itemIndex){
			isChange = YES;
		}

		// context
		if([event type] == NSRightMouseDown ||
			([event modifierFlags] & NSControlKeyMask)){
			isContext = YES;
		}

		// changeSelectionIndexes
		if(isChange == YES){
			[self changeSelectionIndexes:itemIndex
									isShift:NO
									isCmd:NO
			];
		}

		// double clicked
		if([event type] == NSLeftMouseDown){
			if([event clickCount] == 2){
				[viewChapter playItemChapter:nil];
			}
		}

		// context menu
		if(isContext ==YES){
//			[NSMenu popUpContextMenu:cmChapter withEvent:event forView:self];
		}
	
	}

	// get responder
	[[self enclosingScrollView] becomeFirstResponder];

	// for transparent window mouseDown
	[[self window] mouseDown:event];
	
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)event
{
	[self mouseDown:event];
}
//------------------------------------
// mouseDragged
//------------------------------------
- (void)mouseDragged:(NSEvent *)event
{
	// for transparent window dragging
	[[self window] mouseDragged:event];
}
/*
//------------------------------------
// scrollWheel
//------------------------------------
- (void)scrollWheel:(NSEvent *)theEvent
{
	int wheelDelta;

	wheelDelta = [theEvent deltaY];

	if(wheelDelta <= 0){
		// down
		if([theEvent modifierFlags] & NSCommandKeyMask){
			[self selectMovedCell:KEY_ARROW_DOWN isShift:NO isCmd:NO];
		}
		// right
		else{
			[self selectMovedCell:KEY_ARROW_RIGHT isShift:NO isCmd:NO];
		}
	}else{
		// up
		if([theEvent modifierFlags] & NSCommandKeyMask){
			[self selectMovedCell:KEY_ARROW_UP isShift:NO isCmd:NO];
		}
		// left
		else{
			[self selectMovedCell:KEY_ARROW_LEFT isShift:NO isCmd:NO];
		}
	}

}
*/
//------------------------------------
// acceptsFirstResponder
//------------------------------------
-(BOOL)acceptsFirstResponder
{
	return YES;
}
//------------------------------------
// maxCols
//------------------------------------
- (int)maxCols
{
	return 4;
}
//------------------------------------
// startIndex
//------------------------------------
- (void)setStartIndex:(int)index
{
	startIndex_ = index;
}
- (int)startIndex
{
	return startIndex_;

}
//------------------------------------
// endIndex
//------------------------------------
- (void)setEndIndex:(int)index
{
	endIndex_ = index;
}
- (int)endIndex
{
	return endIndex_;

}
//------------------------------------
// selectedObject
//------------------------------------
- (void)setSelectedObject:(id)selectedObject
{
	selectedObject_ = selectedObject;
}
- (id)selectedObject
{
	return selectedObject_;
}
//------------------------------------
// selectedIndex
//------------------------------------
- (void)setSelectedIndex:(int)index
{
	selectedIndex_ = index;
}
- (int)selectedIndex
{
	return selectedIndex_;

}
//------------------------------------
// lastIndex
//------------------------------------
- (int)lastIndex
{
	return [self totalResults] - 1;
}
//------------------------------------
// totalResults
//------------------------------------
- (int)totalResults
{
	return [[viewChapter chapterList] count];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{

	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[super dealloc];
}

@end
