#import "ChapterMatrixCell.h"
#import "ConvertExtension.h"
#import "ChapterExtension.h"
#import "ImageAttributeExtension.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

static float CELL_PADDING_WIDTH = 6.0;
static float IMAGE_PADDING_HEIGHT = 15;
static float IMAGE_SHADOW_OFFSET = 5;
static float IMAGE_POINT_Y_OFFSET = -13;
static float TEXT_BLANK_HEIGHT = 0.0;
static float TEXT_POINT_Y_OFFSET = -5;

@implementation ChapterMatrixCell

//------------------------------------
// init
//------------------------------------
- (id)initWithItemObject:(id)itemObject
				itemIndex:(int)itemIndex
				isEnabled:(BOOL)isEnabled
				isExist:(BOOL)isExist
{
    if (self = [super init])
	{
		[self setItemObject:itemObject];
		[self setItemIndex:itemIndex];
		[self setIsExist:isExist];
		[self setIsEnabled:isEnabled];
		[self setIsSelected:NO];
   }
    return self;
}
//------------------------------------
// drawInteriorWithFrame
//------------------------------------
- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView*)controlView
{

	if([self isEnabled] == NO || ![self itemObject]){
		return;
	}

	// innerRect
	float innerWidth = cellFrame.size.width - 5;
	float innerHeight = cellFrame.size.height - 5;
	NSRect innerRect;
	innerRect.origin.x = cellFrame.origin.x + ((cellFrame.size.width - innerWidth) / 2);
	innerRect.origin.y = cellFrame.origin.y + ((cellFrame.size.height - innerHeight) / 2);
	innerRect.size.width = innerWidth;
	innerRect.size.height = innerHeight;

	id record = [self itemObject];
	BOOL isExist = [self isExist];

	// title
	NSString *uniqueId = [record valueForKey:@"uniqueId"];
	NSString *chapterId = [record valueForKey:@"chapterId"];
	NSString *title = [record valueForKey:@"title"];
	int playTime = [[record valueForKey: @"playTime"] intValue];
	BOOL hasImage = [[record valueForKey: @"hasImage"] boolValue];

	// image
	NSImage *image = [self image];
//	NSImage *image = nil;
	if(!image){
		BOOL isSound = NO;
		// sound
		if(playTime && playTime > 0 && hasImage == NO){
			isSound = YES;
		}
		// get chapter image
		image = [self getChapterImage:[self getChapterImagePath:uniqueId chapterId:chapterId] isSound:isSound];
		[self setImage:image];
	}

    NSPoint	imagePoint;
    imagePoint.x = innerRect.origin.x;
    imagePoint.y = innerRect.origin.y;

	//
	// draw image
	//
    if(image){

		// set image size from original size
		[image setSize:[self setCellImageSize:[image size]
									frameSize:innerRect.size
									paddingSize:NSMakeSize(CELL_PADDING_WIDTH,IMAGE_PADDING_HEIGHT)
						]
		];

		// set image rect
		NSRect imageRect = [self setCellImageRect:[image size]
										frameRect:innerRect
							];

		// shift image rect
		imageRect.origin.y += IMAGE_POINT_Y_OFFSET;

		[image setScalesWhenResized:YES];

		imagePoint.x += imageRect.origin.x;
		imagePoint.y += imageRect.origin.y;

        if([controlView isFlipped]) {
            imagePoint.y += [image size].height;
        }

		// set image round rect
		NSRect imageRoundRect = [self setCellImageRoundRect:[image size]
													frameRect:innerRect
								];

		// shift image round rect
		imageRoundRect.origin.y += IMAGE_POINT_Y_OFFSET;

		if([self isSelected] == YES){
			//
			// draw highlight
			//
			NSRect highlightRect = imageRoundRect;
			highlightRect.origin.x -= IMAGE_SHADOW_OFFSET;
			highlightRect.origin.y -= IMAGE_SHADOW_OFFSET;
			highlightRect.size.width += (IMAGE_SHADOW_OFFSET * 2);
			highlightRect.size.height += (IMAGE_SHADOW_OFFSET * 2);

//			[[NSColor colorWithCalibratedRed:0.1 green:0.1 blue:1.0 alpha:1.0] set];
			[[NSColor blueColor] set];
			NSBezierPath *selectedBoxPath = [self setRoundPathForRect:highlightRect radius:10];
			[selectedBoxPath fill];
		}

		// draw image rect
		NSBezierPath *imagePath = [self setRoundPathForRect:imageRoundRect radius:5];
		[NSGraphicsContext saveGraphicsState];

		[imagePath addClip];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver];

		[NSGraphicsContext restoreGraphicsState];

	}

//	[image release];

	//
	// draw text
	//
//	if(title){

		NSMutableParagraphStyle *lineStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
		[lineStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];
		[lineStyle setAlignment:NSCenterTextAlignment];

		// color
		NSColor *titleColor = [NSColor whiteColor];
		// not exist
		if(isExist == NO){
			titleColor = [NSColor grayColor];
		}

		// attributes
		NSDictionary *titleAttr = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSFont systemFontOfSize:11.5], NSFontAttributeName,
										titleColor, NSForegroundColorAttributeName,
										lineStyle, NSParagraphStyleAttributeName,
										nil
									];

		// size
		NSSize titleSize = [title sizeWithAttributes:titleAttr];

		// text rect
		float textHeight;
		NSPoint textPoint;
		textPoint.x = innerRect.origin.x + CELL_PADDING_WIDTH;
		if(image){
			textHeight = innerRect.origin.y + innerRect.size.height - imagePoint.y + TEXT_POINT_Y_OFFSET;
			textPoint.y = imagePoint.y - TEXT_POINT_Y_OFFSET;
		}else{
			textHeight = titleSize.height + TEXT_BLANK_HEIGHT;
			textPoint.y = innerRect.origin.y + innerRect.size.height - textHeight;
		}

		// rect
		NSRect textRect = NSMakeRect(textPoint.x,
										textPoint.y,
//										innerRect.origin.y + (innerRect.size.height - textHeight) - TEXT_POINT_Y_OFFSET,
//										innerRect.origin.y + (innerRect.size.height - textHeight),
										innerRect.size.width - (CELL_PADDING_WIDTH * 2),
										textHeight
									);

		NSRect titleRect = NSMakeRect(textRect.origin.x, 
										textRect.origin.y,
										textRect.size.width,
										titleSize.height
									);

		float total1Height = titleRect.size.height;

		if(total1Height <= textRect.size.height){
			// draw title
			[title drawInRect:titleRect withAttributes:titleAttr];
		}

//	}


}
//------------------------------------
// itemObject
//------------------------------------
- (void)setItemObject:(id)itemObject
{
    itemObject_ = itemObject;
}
- (id)itemObject
{
    return itemObject_;
}
//------------------------------------
// itemIndex
//------------------------------------
- (void)setItemIndex:(int)itemIndex
{
    itemIndex_ = itemIndex;
}
- (int)itemIndex
{
    return itemIndex_;
}
//------------------------------------
// isExist
//------------------------------------
- (void)setIsExist:(BOOL)isExist
{
	isExist_ = isExist;
}
- (BOOL)isExist
{
	return isExist_;
}
//------------------------------------
// isEnabled
//------------------------------------
- (void)setIsEnabled:(BOOL)isEnabled
{
	isEnabled_ = isEnabled;
}
- (BOOL)isEnabled
{
	return isEnabled_;
}
//------------------------------------
// isSelected
//------------------------------------
- (void)setIsSelected:(BOOL)isSelected
{
	isSelected_ = isSelected;
}
- (BOOL)isSelected
{
	return isSelected_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
    [super dealloc];
}

@end
