#import "ControlSliderSeekTime.h"
#import "ControlSliderCell.h"

@implementation ControlSliderSeekTime

//=======================================================================
// awakeFromNib
//=======================================================================
- (void)awakeFromNib
{

	[self setEnabled:NO];
	[self setMaxValue:1.0];
	[self setDoubleValue:0.0];

	// delete focus ring
	[self setFocusRingType:NSFocusRingTypeNone];
	
	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleVideoObjectDidChanged:) name:VIDEO_NOTIF_OBJECT_DID_CHANGED object:nil];
	[nc addObserver:self selector:@selector(handleVideoTimeDidInited:) name:VIDEO_NOTIF_TIME_DID_INITED object:nil];
	[nc addObserver:self selector:@selector(handleVideoTimeDidChanged:) name:VIDEO_NOTIF_TIME_DID_CHANGED object:nil];
}
//=======================================================================
// method
//=======================================================================
//------------------------------------
// cellClass
//------------------------------------
+ (Class)cellClass
{
	return [ControlSliderCell class];
}

//------------------------------------
// initWithCoder
//------------------------------------
- initWithCoder: (NSCoder *)origCoder
{
	if(![origCoder isKindOfClass: [NSKeyedUnarchiver class]]){
		self = [super initWithCoder: origCoder]; 
	} else {
		NSKeyedUnarchiver *coder = (id)origCoder;
		
		NSString *oldClassName = [[[self superclass] cellClass] className];
		Class oldClass = [coder classForClassName: oldClassName];
		if(!oldClass)
			oldClass = [[super superclass] cellClass];
		[coder setClass: [[self class] cellClass] forClassName: oldClassName];
		self = [super initWithCoder: coder];
		[coder setClass: oldClass forClassName: oldClassName];
	}

	return self;
}
//------------------------------------
// handleVideoObjectDidChanged
//------------------------------------
- (void)handleVideoObjectDidChanged:(NSNotification *)notification
{
	NSDictionary *params = [notification object];
	BOOL hasDuration = [[params valueForKey:@"hasDuration"] boolValue];
	[self setEnabled:hasDuration];

}
//------------------------------------
// handleVideoObjectDidChanged
//------------------------------------
- (void)handleVideoTimeDidInited:(NSNotification *)notification
{

	NSDictionary *params = [notification object];

	double currentValue = [[params valueForKey:@"currentValue"] doubleValue];
	double durationValue = [[params valueForKey:@"durationValue"] doubleValue];

	[self setMaxValue:durationValue];
	[self setDoubleValue:currentValue];

	[self setNeedsDisplay:YES];
}
//------------------------------------
// handleVideoPlayDidChanged
//------------------------------------
- (void)handleVideoTimeDidChanged:(NSNotification *)notification
{

	if([self isEnabled] == NO){
		return;
	}

	NSDictionary *params = [notification object];

	double currentValue = [[params valueForKey:@"currentValue"] doubleValue];
//	double durationValue = [[params valueForKey:@"durationValue"] doubleValue];

//	[self setMaxValue:durationValue];
	[self setDoubleValue:currentValue];

	[self setNeedsDisplay:YES];

}
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{    

	BOOL isAction = NO;
	NSString *keys = [theEvent charactersIgnoringModifiers];

	if (keys && [keys length] > 0){
		unichar c = [keys characterAtIndex:0];
		if(c == NSUpArrowFunctionKey){
			isAction = YES;
		}
		else if(c == NSDownArrowFunctionKey){
			isAction = YES;
		}
		else if(c == NSLeftArrowFunctionKey){
			isAction = YES;
		}
		else if(c == NSRightArrowFunctionKey){
			isAction = YES;
		}

	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}

}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)event
{
	// for transparent window mouseDown
	if([self isEnabled] == YES){
		[super mouseDown:event];
	}else{
		[[self window] mouseDown:event];
	}
}
//------------------------------------
// mouseDragged
//------------------------------------
- (void)mouseDragged:(NSEvent *)event
{
	// for transparent window mouseDragged
	if([self isEnabled] == YES){
		[super mouseDragged:event];
	}else{
		[[self window] mouseDragged:event];
	}
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
