#import "DialogExtension.h"

@implementation NSObject(dialogExtension_)

//------------------------------------
// displayMessage
//------------------------------------
- (int)displayMessage:(NSString *)messageStyle
					messageText:(NSString *)messageText
					infoText:(NSString *)infoText
					btnList:(NSString *)btnList
{

	// post notification
	[self postDialogOpenedNotification];

	NSAlert *alert;
	int i,result;
	NSArray *btnArray;

	alert = [[[NSAlert alloc] init] autorelease];

	[alert setAlertStyle:NSWarningAlertStyle];
	[alert setMessageText:messageText];
	if(![infoText isEqualToString:@""]){
		[alert setInformativeText:infoText];
	}

	if([messageStyle isEqualToString:@"alert"]){
		[alert setIcon:[NSImage imageNamed:@"icon_alert"]];
	}
	else if([messageStyle isEqualToString:@"error"]){
		[alert setIcon:[NSImage imageNamed:@"icon_stop"]];
	}
	else if([messageStyle isEqualToString:@"info"]){
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}
	else{
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}

	btnArray = [btnList componentsSeparatedByString:@","];
	for(i = 0; i < [btnArray count]; i++){
		[alert addButtonWithTitle:[btnArray objectAtIndex:i]];
	}

	result = [alert runModal];
	return result;

}
//------------------------------------
// displayMessageWithIcon
//------------------------------------
- (int)displayMessageWithIcon:(NSString *)iconName
					messageText:(NSString *)messageText
					infoText:(NSString *)infoText
					btnList:(NSString *)btnList
{

	// post notification
	[self postDialogOpenedNotification];

	NSAlert *alert;
	int i,result;
	NSArray *btnArray;

	alert = [[[NSAlert alloc] init] autorelease];

	[alert setAlertStyle:NSWarningAlertStyle];
	[alert setMessageText:messageText];
	if(![infoText isEqualToString:@""]){
		[alert setInformativeText:infoText];
	}

	if(![iconName isEqualToString:@""]){
		[alert setIcon:[NSImage imageNamed:iconName]];
	}else{
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}

	btnArray = [btnList componentsSeparatedByString:@","];
	for(i = 0; i < [btnArray count]; i++){
		[alert addButtonWithTitle:[btnArray objectAtIndex:i]];
	}

	result = [alert runModal];
	return result;

}
//------------------------------------
// displayMessageAlertOpenMovie 
//------------------------------------
- (int)displayMessageAlertOpenMovie:(NSString*)message
{
	return [self displayMessage:@"alert"
						messageText:@"Can not open movie."
						infoText:message
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertAppIsNotFound 
//------------------------------------
- (int)displayMessageAlertAppIsNotFound:(NSString*)filePath
{
	return [self displayMessage:@"alert"
						messageText:@"Application is not found."
						infoText:[NSString stringWithFormat:@"Check file \"%@\"", filePath]
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertFileIsNotFound 
//------------------------------------
- (int)displayMessageAlertFileIsNotFound:(NSString*)filePath
{
	return [self displayMessage:@"alert"
						messageText:@"File is not found."
						infoText:[NSString stringWithFormat:@"Check file \"%@\"", filePath]
						btnList:@"Cancel"
			];
}
//------------------------------------
// selectFilePathWithOpenPanel
//------------------------------------
- (NSArray*)selectFilePathWithOpenPanel:(BOOL)isFile
						isDir:(BOOL)isDir
						isMultiSel:(BOOL)isMultiSel
						isPackage:(BOOL)isPackage
						isAlias:(BOOL)isAlias
						canCreateDir:(BOOL)canCreateDir
						defaultPath:(NSString*)defaultPath
{

	// post notification
	[self postDialogOpenedNotification];

	NSOpenPanel* openPanel;
	int	result;
	NSArray* filePaths = [[[NSArray alloc] init] autorelease];
//	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	openPanel = [NSOpenPanel openPanel];
	[openPanel setCanChooseFiles:isFile];
	[openPanel setCanChooseDirectories:isDir];
	[openPanel setAllowsMultipleSelection:isMultiSel];
	[openPanel setTreatsFilePackagesAsDirectories:isPackage];
	[openPanel setResolvesAliases:isAlias];
	[openPanel setCanCreateDirectories:canCreateDir];

	result = [openPanel runModalForDirectory:defaultPath file:nil types:nil];

	if (result == NSFileHandlingPanelOKButton) {
		filePaths = [openPanel filenames];
	}

	return filePaths;

}
//------------------------------------
// postDialogOpenedNotification
//------------------------------------
- (void)postDialogOpenedNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_DIALOG_DID_OPENED object:self];
}
@end