#import "FileItemManager.h"
#import "ViewLog.h"
#import "ViewProcStatus.h"
#import "TBArrayController.h"
#import "ThumbsExtension.h"
#import "ChapterExtension.h"
#import "FileAttributeExtension.h"
#import "ImageAttributeExtension.h"
#import "MovieAttributeExtension.h"
#import "DialogExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"

static int IMPORT_ALERT_MAX_COUNT = 50;
static int IMPORT_OVER_MAX_COUNT = 500;

@implementation FileItemManager
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	targetCount_ = 0;
	invalidCount_ = 0;
	checkedCount_ = 0;
	[self setLogString:@""];

}
//------------------------------------
// importItemWithFilePaths
//------------------------------------
- (BOOL)importItemWithFilePaths:(NSArray*)filePaths
					targetItem:(id)targetItem
					targetType:(int)targetType
{

	int result;

	// init
	targetCount_ = 0;
	invalidCount_ = 0;
	checkedCount_ = 0;
	BOOL isErr = NO;
	[self setLogString:@""];

	//
	// create import file paths 
	//
	NSMutableArray *importFilePaths = [self getImportFilePaths:filePaths isErr:&isErr];

	// error
	if(isErr == YES){
		[self displayMessage:@"alert"
							messageText:[NSString stringWithFormat: @"Can not import over %d files at one time.",
																		IMPORT_OVER_MAX_COUNT
										]
							infoText:@"Please import at several times."
							btnList:@"Cancel"
		];
		return NO;
	}

	// alert
	int importCount = [importFilePaths count];
	if(importCount > IMPORT_ALERT_MAX_COUNT){
		result = [self displayMessage:@"alert"
							messageText:[NSString stringWithFormat: @"Importing over %d (%d) files.\nContinue import?",
																		IMPORT_ALERT_MAX_COUNT,
																		importCount
										]
							infoText:@"It needs much time for importing."
							btnList:@"Import,Cancel"
				];
		if(result != NSAlertFirstButtonReturn){
			return NO;
		}
	}

//	NSLog(@"importCount=%d", importCount);
//	NSLog(@"importFilePaths=%@", [importFilePaths description]);

	//
	// create filelist
	//
	if(importCount > 0){

		NSString *title = @"";
		BOOL addPlist = NO;

		//
		// add to playlist
		//
		if(targetItem != nil && [[targetItem valueForKey:@"isFolder"] boolValue] == NO){

			// show alert
			if([self defaultBoolValue:@"optShowAlertAddToPlaylist"] == YES){
				title = [targetItem valueForKey:@"title"];

				// target is playlist
				if(targetType == IMPORT_TARGET_PLAYLIST){
					result = [self displayMessageWithIcon:@"icon_alert_playlist"
												messageText:[NSString stringWithFormat:@"Add item to Playlist \"%@\"", title]
												infoText:@""
												btnList:@"Add,Cancel"
							];
					// add
					if(result == NSAlertFirstButtonReturn){
						addPlist = YES;
					}
					// cancel
					else if(result == NSAlertSecondButtonReturn){
						return NO;
					}
				}
				// filelist
				else{
					result = [self displayMessageWithIcon:@"icon_alert_playlist"
												messageText:[NSString stringWithFormat:@"Playlist \"%@\" is selected.", title]
												infoText:@"Add item to playlist"
												btnList:@"Add,Skip,Cancel"
							];
					// add
					if(result == NSAlertFirstButtonReturn){
						addPlist = YES;
					}
					// skip
					else if(result == NSAlertSecondButtonReturn){
						addPlist = NO;
					}
					// cancel
					else if(result == NSAlertThirdButtonReturn){
						return NO;
					}
				}
			}
			else{
				addPlist = YES;
			}
		}

		//
		// insert filelist
		//
		NSArray *items = [self insertFilelistWithFilePaths:importFilePaths];
//		NSLog(@"items=%@", [items description]);

		//
		//  add to playlist
		//
		if([items count] > 0 && addPlist == YES){
			if([[targetItem valueForKey:@"isFolder"] boolValue] == NO){
				// add relationship
				[self addItemsToPlaylist:targetItem items:items];
			}
		}
	}

	// error
	if(invalidCount_ > 0){
		result = [self displayMessage:@"alert"
							messageText:@"Can not import some items."
							infoText:@"Please check import log."
							btnList:@"OK,Log"
				];
		// open log window
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[self logString]];
			[viewLog openLogWindow:nil];
		}
	}
/*
	else{
		// no import && all imported
		if(importCount <= 0 && checkedCount_ > 0){
			[self displayMessage:@"alert"
								messageText:@"All items are already added."
								infoText:@""
								btnList:@"OK"
			];
		}
	}
*/
	return YES;
}
//------------------------------------
// getImportFilePaths
//------------------------------------
- (NSMutableArray*)getImportFilePaths:(NSArray*)filePaths isErr:(BOOL*)isErr
{
	NSMutableArray *outPaths = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *checkedPaths = [[[NSMutableArray alloc] init] autorelease];
	id filePath;

	// get imported path
	[checkedPaths addObjectsFromArray:[[tbArrayController getAllObjects:@"filelist"] valueForKey:@"filePath"]];

	// get target file path
	NSEnumerator *enumFiles = [filePaths objectEnumerator];
	while (filePath = [enumFiles nextObject]) {
		[outPaths addObjectsFromArray:[self getTargetFilePaths:filePath checkedPaths:checkedPaths isErr:isErr]];
		if(*isErr == YES){
			break;
		}
	}

//	NSLog(@"outPaths=%@", [outPaths description]);

	return outPaths;
}
//----------------------------------------
// getTargetFilePaths
//----------------------------------------
- (NSMutableArray*)getTargetFilePaths:(NSString*)filePath checkedPaths:(NSMutableArray*)checkedPaths isErr:(BOOL*)isErr
{


	NSMutableArray *outPaths = [[[NSMutableArray alloc] init] autorelease];

	NSFileManager *myFile = [NSFileManager defaultManager];
	BOOL isDir;

	// over target count
	targetCount_++;
//	NSLog(@"targetCount_=%d", targetCount_);
	if(targetCount_ > IMPORT_OVER_MAX_COUNT){
		*isErr = YES;
		return outPaths;
	}

	// resolve alias
	NSString *targetPath = [self resolveAliasFileWithPath:filePath];

	// path error
	if(targetPath == nil){
		invalidCount_++;
		[self appendLogString:[NSString stringWithFormat: @"Original file is not found : %@\n", filePath]];
		return outPaths;
	}

	// already exists
    if([checkedPaths indexOfObject:targetPath] != NSNotFound){
		invalidCount_++;
//    	checkedCount_++;
		[self appendLogString:[NSString stringWithFormat: @"Already imported : %@\n", filePath]];
		return outPaths;
    }

	// add to checked path
	[checkedPaths addObject:targetPath];

	// file is not found
	if(![myFile fileExistsAtPath:targetPath isDirectory:&isDir]){
		invalidCount_++;
		[self appendLogString:[NSString stringWithFormat: @"File is not found : %@\n", targetPath]];
		return outPaths;
	}

	// hidden or Icon file
	if( [[[myFile displayNameAtPath:targetPath] substringToIndex:1] isEqualToString:@"."] ||
		[[myFile displayNameAtPath:targetPath] isEqualToString:@"Icon\x0d"]){
		return outPaths;
	}

	// not importable
	if([self checkAcceptFilePath:targetPath isDir:isDir] == NO){
		invalidCount_++;
		[self appendLogString:[NSString stringWithFormat: @"Not importable : %@\n", targetPath]];
		return outPaths;
	}

	// copy protected video
	if([self defaultBoolValue:@"optImportCopyProtectedVideo"] == NO){
		MDItemRef item = MDItemCreate(kCFAllocatorDefault, (CFStringRef)filePath);
		CFStringRef uti;
		if(item){
			uti = MDItemCopyAttribute(item, kMDItemContentType);
			CFRelease(item);
		}

		if(uti && UTTypeEqual(uti, CFSTR("com.apple.protected-mpeg-4-video"))){
			CFRelease(uti);
			invalidCount_++;
			[self appendLogString:[NSString stringWithFormat: @"Copy protected video : %@\n", targetPath]];
			return outPaths;
		}
	}

	// directory
	if(isDir == YES){

		// get contents path
		NSArray *contents = [myFile directoryContentsAtPath:targetPath];
	    NSEnumerator *enumContents = [contents objectEnumerator];
		id contentName;

		while(contentName = [enumContents nextObject]) {

			// create content path
			NSString *childPath = [NSString stringWithFormat: @"%@/%@", targetPath, contentName];

			// resolve alias
			NSString *targetChildPath = [self resolveAliasFileWithPath:childPath];
			// target path is null
			if(targetChildPath == nil){
				continue;
			}

			// call self method
			[outPaths addObjectsFromArray:[self getTargetFilePaths:targetChildPath checkedPaths:checkedPaths isErr:isErr]];
			// error
			if(*isErr == YES){
				break;
			}

		}
	}
	// file
	else{
//		NSLog(@"filePath=%@", filePath);
//		NSLog(@"targetPath=%@", targetPath);
		[outPaths addObject:filePath];
	}

	return outPaths;

}

//----------------------------------------
// checkAcceptFilePath
//----------------------------------------
- (BOOL)checkAcceptFilePath:(NSString*)filePath isDir:(BOOL)isDir
{

//	NSFileManager *myFile = [NSFileManager defaultManager];

	// application file
	if([[[filePath pathExtension] lowercaseString] isEqualToString:@"app"]){
		return NO;
	}
	// executale file
//	if(isDir == NO && [myFile isExecutableFileAtPath:filePath] == YES){
//		return NO;
//	}
	// can not init movie
	if(isDir == NO && [QTMovie canInitWithFile:filePath] == NO){
		return NO;
	}

	return YES;

}
//------------------------------------
// insertFilelistWithFilePaths
//------------------------------------
- (NSArray*)insertFilelistWithFilePaths:(NSArray*)filePaths
{

	NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];

	NSString *uniqueId;
	NSString *filePath;
	NSString *title;
	NSString *fileTypeName;
	NSDate *fileDate;
	int playTime;
	int fileSize;
	int imageWidth;
	int imageHeight;
	BOOL hasImage;
	int thumbsFrameType = [self defaultThumbsFrameType];
	float thumbsFrameSec = [self defaultThumbsFrameSec];

	// check and create thumbs folder
	BOOL canCreateThumbs = [self checkAndCreateThumbsFolder:NO];

	// get uniqueNo
	int uniqueNo = [self defaultIntValue:@"optUniqueNoForFlist"];

	// get max uniqueNo
	if(uniqueNo <= 0){
		uniqueNo = [tbArrayController getMaxIntValueFromEntityWithColumn:@"filelist" column:@"uniqueId"];
	}

	NSFileManager *myFile = [NSFileManager defaultManager];

	//---------------------------
	//	insert record 
	//---------------------------
	NSEntityDescription *entFile = [NSEntityDescription 
									entityForName:@"filelist"
									inManagedObjectContext:[MacVideoTunes_AppDelegate managedObjectContext]];

	// process status
	int procCount = 0;
	[viewProcStatus initProcStatus:@"Importing Items"
						targetName:@""
						procCount:procCount
						totalCount:[filePaths count]
	];

	// insert new records
	NSEnumerator *enumPaths = [filePaths objectEnumerator];

	while (filePath = [enumPaths nextObject]) {

		// resolve alias
		filePath = [self resolveAliasFileWithPath:filePath];

		uniqueNo++;
		uniqueId = [self convertToZeroFormat:uniqueNo];
		title = [myFile displayNameAtPath:filePath];
		fileTypeName = @"";
		playTime = 0;
		fileSize = 0;
		fileDate = nil;
		imageWidth = 0;
		imageHeight = 0;
		hasImage = NO;

		//
		// get file attributes
		//
		NSDictionary *fileAttb = [self getFileAttribute:filePath];
		if(fileAttb != nil && [[fileAttb valueForKey:@"fileExist"] boolValue] == YES){
			fileTypeName = [fileAttb valueForKey:@"fileTypeName"];
			fileSize = [[fileAttb valueForKey:@"fileSize"] intValue];
			fileDate = [fileAttb valueForKey:@"fileDate"];
		}

		// update process status
		procCount++;
		[viewProcStatus updateProcStatus:procCount targetName:title];

		//
		// create movie
		//
		NSError *error;
		QTMovie *movie = [QTMovie movieWithFile:filePath error:&error];

		if(movie != nil){

			//
			// create movie attributes
			//
			// duration
			QTTime time = [movie duration];
			// convert to second
			playTime = time.timeValue / time.timeScale;
			// image
			NSSize naturalSize = [[movie attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 
			imageWidth = naturalSize.width;
			imageHeight = naturalSize.height;
			if(naturalSize.width > 0 && naturalSize.height > 0){
				hasImage = YES;
			}
			// create title
			if([movie attributeForKey:QTMovieDisplayNameAttribute]){
				title = [movie attributeForKey:QTMovieDisplayNameAttribute];
			}

			//
			// create thumbnail image
			//
			if(canCreateThumbs == YES && hasImage == YES){

				NSImage *image = nil;
				if(thumbsFrameType == ITEM_THUMBS_FRAME_TYPE_POSTER){
					image = [movie posterImage];
//					NSLog(@"posterImage");
				}
				else{
					float duration = [movie duration].timeValue / [movie duration].timeScale;
					if(duration >= thumbsFrameSec){
						image = [self createMovieFrameImageAtTime:movie sec:thumbsFrameSec];
//						NSLog(@"frameImageAtTime");
					}
					else{
						image = [movie posterImage];
//						NSLog(@"posterImage");
					}
				}

				[self createThumbsImage:image
										uniqueId:uniqueId
										isCheckFolder:NO
										showAlert:NO
				];

			}

			//
			// add record
			//
			NSManagedObject *mo = [[[NSManagedObject alloc]
									initWithEntity:entFile
									insertIntoManagedObjectContext:[MacVideoTunes_AppDelegate managedObjectContext]]
									autorelease];

			[mo setValue:filePath forKey:@"filePath"];
			[mo setValue:fileTypeName forKey:@"fileTypeName"];
			[mo setValue:title forKey:@"title"];
			[mo setValue:[NSNumber numberWithInt:playTime] forKey:@"playTime"];
			[mo setValue:[NSNumber numberWithInt:fileSize] forKey:@"fileSize"];
			[mo setValue:[NSNumber numberWithInt:imageWidth] forKey:@"imageWidth"];
			[mo setValue:[NSNumber numberWithInt:imageHeight] forKey:@"imageHeight"];
			[mo setValue:[NSNumber numberWithBool:hasImage] forKey:@"hasImage"];
			[mo setValue:[NSNumber numberWithInt:0] forKey:@"labelColorNo"];
			[mo setValue:uniqueId forKey:@"uniqueId"];
			[mo setValue:[NSDate date] forKey:@"addDate"];
			[mo setValue:fileDate forKey:@"fileDate"];

			// insert items
			[items addObject:mo];
		}

	}

	// save default
	[self setDefaultIntValue:uniqueNo key:@"optUniqueNoForFlist"];

	// close process window
	[viewProcStatus closeProcStatus:YES];

	// create thumbs error
	if(canCreateThumbs == NO){
		[self displayMessage:@"alert"
					messageText:@"Can not create thumbnails folder."
					infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getThumbsFolderPath]]
					btnList:@"OK"
		];
	}

	return items;

}
//------------------------------------
// createItemThumbsImageWithItems
//------------------------------------
- (void)createItemThumbsImageWithItems:(NSArray*)items
{

	// check and create thumbs folder
	if([self checkAndCreateThumbsFolder:YES] == NO){
/*
		[self displayMessage:@"alert"
					messageText:@"Can not create thumbnails folder."
					infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getThumbsFolderPath]]
					btnList:@"NO"
		];
*/
		return;
	}

	[self setLogString:@""];

	// process status
	int procCount = 0;
	[viewProcStatus initProcStatus:@"Creating Thumbnails"
						targetName:@""
						procCount:procCount
						totalCount:[items count]
	];

	// create thumbnail
	NSString *uniqueId;
	NSString *title;
	NSString *filePath;
	int invalidCount = 0;
	int thumbsFrameType = [self defaultThumbsFrameType];
	float thumbsFrameSec = [self defaultThumbsFrameSec];

	NSEnumerator *enumItems = [items objectEnumerator];
	id record;

	while (record = [enumItems nextObject]) {

		uniqueId = [record valueForKey:@"uniqueId"];
		title = [record valueForKey:@"title"];
		filePath = [record valueForKey:@"filePath"];

		// update process status
		procCount++;
		[viewProcStatus updateProcStatus:procCount targetName:title];

		// file not found
		if([self checkFileExist:filePath] != FILE_EXIST_FILE){
			[self appendLogString:[NSString stringWithFormat:@"File is not found : %@\n", filePath]];
			invalidCount++;
			continue;
		}

		//
		// create movie
		//
		NSError *error;
		QTMovie *movie = [QTMovie movieWithFile:filePath error:&error];

		if(movie != nil){

			// no image
			NSSize naturalSize = [[movie attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 
			if(naturalSize.width <= 0 || naturalSize.height <= 0){
				[self appendLogString:[NSString stringWithFormat:@"Can not create image : %@\n", filePath]];
				invalidCount++;
				continue;
			}

			//
			// create thumbnail image
			//
			NSImage *image = nil;
			if(thumbsFrameType == ITEM_THUMBS_FRAME_TYPE_POSTER){
				image = [movie posterImage];
//				NSLog(@"posterImage");
			}
			else{
				float duration = [movie duration].timeValue / [movie duration].timeScale;
				if(duration >= thumbsFrameSec){
					image = [self createMovieFrameImageAtTime:movie sec:thumbsFrameSec];
//					NSLog(@"frameImageAtTime");
				}
				else{
					image = [movie posterImage];
//					NSLog(@"posterImage");
				}
			}

			[self createThumbsImage:image
									uniqueId:uniqueId
									isCheckFolder:NO
									showAlert:NO
			];

		}
		else{
			[self appendLogString:[NSString stringWithFormat:@"Can not create image : %@\n", filePath]];
			invalidCount++;
			continue;
		}


	}

	// close process window
	[viewProcStatus closeProcStatus:YES];

	// error
	if(invalidCount > 0){
		int result = [self displayMessage:@"alert"
							messageText:@"Can not create some images."
							infoText:@"Please check error log."
							btnList:@"OK,Log"
				];
		// open log window
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Create Error"];
			[viewLog setLogString:[self logString]];
			[viewLog openLogWindow:nil];
		}
	}

}
//------------------------------------
// removeFilelistWithItems
//------------------------------------
- (void)removeFilelistWithItems:(NSArray*)items
{

	NSString *uniqueId;


	NSEnumerator *enumItems = [items objectEnumerator];
	id record;
	// remove
	while (record = [enumItems nextObject]) {

		uniqueId = [record valueForKey:@"uniqueId"];

		// remove thumbs image
		[self removeThumbsImage:uniqueId];
	
		// remove chapter folder
		[self removeChapterFolder:uniqueId];

		// remove record
		[[MacVideoTunes_AppDelegate managedObjectContext] deleteObject:record];
	}

}


//------------------------------------
// addItemsToPlaylist
//------------------------------------
- (void)addItemsToPlaylist:(NSManagedObject*)targetItem items:(NSArray*)items
{

	if(targetItem == nil || [items count] <= 0){
		return;
	}

	NSMutableSet *rlt_filelist = [targetItem valueForKey:@"rlt_filelist"];

	int i;

	// add RelationShip with items
	for (i = 0; i < [items count]; i++) {

		// get object
		NSManagedObject *object = [items objectAtIndex:i];

		if (object != nil) {
			// add object to relationship in filelist 
			NSMutableSet *rlt_playlist = [object valueForKey:@"rlt_playlist"];
			if(![rlt_playlist containsObject:targetItem]){
				[rlt_playlist addObject:targetItem];
				[object setValue:rlt_playlist forKey:@"rlt_playlist"];
			}
			// add object to relationship in playlist 
			if(![rlt_filelist containsObject:object]){
				[rlt_filelist addObject:object];
				[targetItem setValue:rlt_filelist forKey:@"rlt_filelist"];
			}
		}
	}

}
//------------------------------------
// removeItemsFromPlaylist
//------------------------------------
- (void)removeItemsFromPlaylist:(NSManagedObject*)targetItem items:(NSArray*)items
{

	if(targetItem == nil || [items count] <= 0){
		return;
	}

	NSMutableSet *rlt_filelist = [targetItem valueForKey:@"rlt_filelist"];

	int i;

	// remove RelationShip with items
	for (i = 0; i < [items count]; i++) {

		// get object
		NSManagedObject *object = [items objectAtIndex:i];

		if (object != nil) {
			// remove object to relationship in filelist 
			NSMutableSet *rlt_playlist = [object valueForKey:@"rlt_playlist"];
			if([rlt_playlist containsObject:targetItem]){
				[rlt_playlist removeObject:targetItem];
				[object setValue:rlt_playlist forKey:@"rlt_playlist"];
			}
			// remove object from relationship in playlist 
			if([rlt_filelist containsObject:object]){
				[rlt_filelist removeObject:object];
				[targetItem setValue:rlt_filelist forKey:@"rlt_filelist"];
			}
		}
	}

}

//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString *)logString
{
	[logString retain];
    [logString_ release];
    logString_ = logString;

}
- (NSString *)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[logString_ release];
    [super dealloc];
}

@end
