#import "FilelistTableView.h"
#import "ViewMain.h"
#import "RoundImageCell.h"
#import "ItemDescTextCell.h"
#import "TableColumnManager.h"
#import "TableColumnExtension.h"
#import "ThumbsExtension.h"
#import "CellAttributeExtension.h"
#import "FileAttributeExtension.h"
#import "ImageAttributeExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"

@implementation FilelistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

	// set row height
	float rowHeight = [self defaultFloatValue:@"optMainRowHeight"];
	[sliderRowHeight setFloatValue:rowHeight];
	[self setRowHeight:rowHeight];

	// image column
	NSTableColumn *imageColumn = [self tableColumnWithIdentifier:@"image"];
	RoundImageCell *imageCell = [[[RoundImageCell alloc] init] autorelease];
	[imageColumn setDataCell:imageCell];

	// description column
	NSTableColumn *descColumn = [self tableColumnWithIdentifier:@"description"];
	ItemDescTextCell *descTextCell = [[[ItemDescTextCell alloc] init] autorelease];
	[descColumn setDataCell:descTextCell];

    // set click action
	[self setTarget:viewMain];
	[self setDoubleAction:@selector(playItem:)];

	// set table column state
	[self setTableColumnState:self key:@"stateTableColumnFilelist"];

	// init column controller
	if (tcm_ == nil) {
		tcm_ = [[TableColumnManager alloc] initWithTable:self];
	}

	// showTableColumns
	// set default hide columns
	NSArray *defaultHideCols = [NSArray arrayWithObjects:
									@"fileTypeName",
									@"addDate",
									@"uniqueId",
									nil
								];
	[self showTableColumns:tcm_ aTableView:self defaultHideCols:defaultHideCols key:@"stateTableColumnFilelist"];

	// set sort descriptor
	[self setArrayControllerSortDescriptor:filelistArrayController key:@"sortDescColumnFilelist"];

	// set header menu
	[[self headerView] setMenu:cmFilelistHeader];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[self window]];
	[nc addObserver:self selector:@selector(itemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED object:nil];

	// set registerForDraggedTypes
	[self registerForDraggedTypes:
				[NSArray arrayWithObjects:
						FilelistPboardType,
						NSFilenamesPboardType,
						nil]
				];

	// set drag operation
	[self setDraggingSourceOperationMask:NSDragOperationLink forLocal:NO];

}
//------------------------------------
// changeRowHeight
//------------------------------------
- (IBAction)changeRowHeight:(id)sender
{
	[self setRowHeight:[sender floatValue]];

	if([self numberOfSelectedRows] > 0){
		[self scrollRowToVisible:[self selectedRow]];
	}

}
//------------------------------------
// change table column state
//------------------------------------
- (IBAction)changeColumnState:(id)sender;
{
	[self setColumnState:tcm_ sender:sender];
}
//------------------------------------
// isShowColumn
//------------------------------------
- (BOOL)isShowColumn:(NSString*)identifier
{
	return [self isShowColumnState:tcm_ identifier:identifier];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{

	// table column state
	[self saveTableColumnState:self key:@"stateTableColumnFilelist"];
	// save sort descriptor
	[self saveArrayControllerSortDescriptor:filelistArrayController key:@"sortDescColumnFilelist"];

}
//------------------------------------
// itemInfoDidChanged
//------------------------------------
-(void)itemInfoDidChanged:(NSNotification *)notification
{
	[self reloadData];
	[viewMain changePreviewImage:nil];
}

//=======================================================================
// bind to tableview column
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
        objectValueForTableColumn:(NSTableColumn *)aTableColumn
        row:(int)rowIndex
{

	// get record from filelistArrayController
	id record = [[filelistArrayController arrangedObjects] objectAtIndex:rowIndex];

	// image
	if([[aTableColumn identifier] isEqualToString:@"image"]){

		NSString *uniqueId = [record valueForKey:@"uniqueId"];
		int playTime = [[record valueForKey: @"playTime"] intValue];
		BOOL hasImage = [[record valueForKey: @"hasImage"] boolValue];
		BOOL isSound = NO;
		BOOL isExist = NO;
		// sound
		if(playTime > 0 && hasImage == NO){
			isSound = YES;
		}
		// get thumbs image
		NSImage *image = [self getThumbsImage:[record valueForKey: @"filePath"]
									thumbsPath:[self getThumbsImagePath:uniqueId]
									isSound:isSound
						];
		// isExist
		if([self checkFileExist:[record valueForKey: @"filePath"]] == FILE_EXIST_FILE){
			isExist = YES;
		}
		RoundImageCell *imageCell = [aTableColumn dataCell];
		[imageCell setImage:image];
		[imageCell setIsExist:isExist];
	}
	// description
	else if([[aTableColumn identifier] isEqualToString:@"description"]){

		NSString *playTimeStr = @"-";
		NSString *fileSizeStr = @"-";
//		BOOL isExist = NO;
		int labelColorNo = 0;

		// playTime
		int playTime = [[record valueForKey: @"playTime"] intValue];
		// convert play time
		if(playTime > 0){
			playTimeStr = [self convertTimeToString:playTime];
		}

		// fileSize
		int fileSize = [[record valueForKey: @"fileSize"] intValue];
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}

		// isExist
//		if([self checkFileExist:[record valueForKey: @"filePath"]] == FILE_EXIST_FILE){
//			isExist = YES;
//		}
		// set label color
		labelColorNo = [[record valueForKey: @"labelColorNo"] intValue];

		NSString *desc1 = [NSString stringWithFormat:@"Time: %@", playTimeStr];
		NSString *desc2 = [NSString stringWithFormat:@"Size: %@", fileSizeStr];
		ItemDescTextCell *descTextCell = [aTableColumn dataCell];
		[descTextCell setStringValue:[NSString stringWithFormat:@"%@\t%@\t%@", [record valueForKey:@"title"], desc1, desc2]];
		[descTextCell setIsExist:YES];
		[descTextCell setLabelColorNo:labelColorNo];

		return descTextCell;

	}

	// filePath
	else if([[aTableColumn identifier] isEqualToString:@"filePath"]){
		NSString *filePath = [record valueForKey:@"filePath"];
		NSMutableAttributedString *attrString=[[[NSMutableAttributedString alloc] initWithString:filePath] autorelease];
		NSMutableParagraphStyle *style=[[[NSMutableParagraphStyle alloc] init] autorelease];

		[style setLineBreakMode:NSLineBreakByTruncatingMiddle];

		[attrString addAttribute:NSParagraphStyleAttributeName value:style range:NSMakeRange(0,[filePath length])];

		return attrString;
	}

	// fileIcon
	else if([[aTableColumn identifier] isEqualToString:@"fileIcon"]){
		id iconImage;
		int playTime = [[record valueForKey: @"playTime"] intValue];
		BOOL hasImage = [[record valueForKey: @"hasImage"] boolValue];
		if(playTime > 0){
			if(hasImage == YES){
				iconImage = [NSImage imageNamed: @"icon_video"];
			}else{
				iconImage = [NSImage imageNamed: @"icon_sound"];
			}
		}else{
			iconImage = [NSImage imageNamed: @"icon_picture"];
		}
		return iconImage;
	}
	// size
	else if([[aTableColumn identifier] isEqualToString:@"fileSize"]){
		NSString *fileSizeStr = @"-";
		int fileSize = [[record valueForKey: @"fileSize"] intValue];
		// convert file size
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}
		return fileSizeStr;
	}
	// time
	else if([[aTableColumn identifier] isEqualToString:@"playTime"]){
		NSString *playTimeStr = @"-";
		int playTime = [[record valueForKey: @"playTime"] intValue];
		// convert play time
		if(playTime > 0){
			playTimeStr = [self convertTimeToString:playTime];
		}
		return playTimeStr;
	}

    return nil;
}
//------------------------------------
// toolTipForCell
//------------------------------------
- (NSString *)tableView:(NSTableView *)aTableView toolTipForCell:(NSCell *)aCell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *)aTableColumn row:(int)row mouseLocation:(NSPoint)mouseLocation
{
    if( [[aTableColumn identifier] isEqualToString:@"description"]
	){
		id record = [[filelistArrayController arrangedObjects] objectAtIndex:row];

		NSString *playTimeStr = @"-";
		NSString *fileSizeStr = @"-";
		NSString *imageSizeStr = @"-";

		// title
		NSString *title = [record valueForKey: @"title"];

		// playTime
		int playTime = [[record valueForKey: @"playTime"] intValue];
		// convert play time
		if(playTime > 0){
			playTimeStr = [self convertTimeToString:playTime];
		}

		// fileSize
		int fileSize = [[record valueForKey: @"fileSize"] intValue];
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}

		// imageSize
		if([record valueForKey: @"imageWidth"] && [record valueForKey: @"imageHeight"]){
			int imageWidth = [[record valueForKey: @"imageWidth"] intValue];
			int imageHeight = [[record valueForKey: @"imageHeight"] intValue];
			if(imageWidth > 0 && imageHeight > 0){
				imageSizeStr = [NSString stringWithFormat:@"%d x %d", imageWidth, imageHeight];
			}
		}

		// filePath
		NSString *filePath = [record valueForKey: @"filePath"];

		NSString *toolChip = [NSString stringWithFormat:@"%@\n%@\n%@\n%@\n%@",
									title,
									[NSString stringWithFormat:@"Time: %@", playTimeStr],
									[NSString stringWithFormat:@"Video: %@", imageSizeStr],
									[NSString stringWithFormat:@"Size: %@", fileSizeStr],
									[NSString stringWithFormat:@"Path: %@", filePath]
							];


		return toolChip;
    }
	return nil;
}
//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[filelistArrayController arrangedObjects] count];
}
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// writeRowsWithIndexes
//------------------------------------
- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard {

    BOOL success = NO;
	NSFileManager *myFile = [NSFileManager defaultManager];

	NSArray *objects = [[filelistArrayController arrangedObjects] objectsAtIndexes:rowIndexes];
 	NSMutableArray *items = [NSMutableArray array];
 	NSMutableArray *filePaths = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		NSManagedObjectID *objectID = [item objectID];
		NSURL *representedURL = [objectID URIRepresentation];
		[items addObject:representedURL];
		// for NSFilenamesPboardType
		if([myFile fileExistsAtPath:[item valueForKey:@"filePath"] isDirectory:nil]){
			[filePaths addObject:[item valueForKey:@"filePath"]];
		}
	}
//	[pboard declareTypes:[NSArray arrayWithObject:FilelistPboardType] owner:nil];
//	[pboard addTypes:[NSArray arrayWithObject:FilelistPboardType] owner:nil];
//	success = [pboard setString:[items componentsJoinedByString:@","] forType:FilelistPboardType];        

	[pboard declareTypes:[NSArray arrayWithObjects:FilelistPboardType, NSFilenamesPboardType, nil] owner:nil];
	[pboard addTypes:[NSArray arrayWithObjects:FilelistPboardType, NSFilenamesPboardType, nil] owner:nil];
	[pboard setString:[items componentsJoinedByString:@","] forType:FilelistPboardType];        
	// for NSFilenamesPboardType
	if([filePaths count] > 0){
		[pboard setPropertyList:filePaths forType:NSFilenamesPboardType];        
	}

	if([items count] > 0){
		success = YES;
	}

	return success;

}
//------------------------------------
// dragImageForRowsWithIndexes
//------------------------------------
- (NSImage *)dragImageForRowsWithIndexes:(NSIndexSet*)dragRows
							tableColumns:(NSArray*)tableColumns
							event:(NSEvent*)dragEvent
							offset:(NSPointPointer)dragImageOffset
{
	// one item
/*
	if ([dragRows count] == 1){
		return [super dragImageForRowsWithIndexes:dragRows tableColumns:tableColumns event:dragEvent offset:dragImageOffset];
	}
	// many items
	else{
*/
		// create drag image
		NSImage	*dragImage = [[NSImage alloc] init];
		[dragImage setSize:NSMakeSize(65.0, 40.0)];

		// set count label bounds
		NSString *countStr = [NSString stringWithFormat:@"%i", [dragRows count]];
		NSRect imageBounds;
		imageBounds.origin = NSMakePoint(0, 0);
		imageBounds.size = [dragImage size];

		// create source image
		NSImage *sourceImage = [[[NSImage alloc] initByReferencingFile: [[NSBundle mainBundle] pathForResource:@"icon_video" ofType:@"png"]] autorelease];

		[sourceImage setScalesWhenResized:YES];
		[sourceImage setSize:NSMakeSize(24, 24)];

		NSPoint sourcePoint;
		sourcePoint.x = imageBounds.origin.x + ((imageBounds.size.width - [sourceImage size].width) / 2);
		sourcePoint.y = imageBounds.origin.y + ((imageBounds.size.height - [sourceImage size].height) / 2);

		[dragImage lockFocus];

		// draw label
		[self drawLabelAndString:imageBounds
						withString:countStr
						fontSize:LABEL_DRAG_FSIZE
						fontColor:[NSColor whiteColor]
						labelColor:[NSColor redColor]
						align:CELL_ALIGN_RIGHT
						valign:CELL_VALIGN_TOP
						hPadding:LABEL_DRAG_HPADDING
						vPadding:LABEL_DRAG_VPADDING
						radius:LABEL_DRAG_RADIUS
		];

		// draw source image
		[sourceImage compositeToPoint:sourcePoint operation:NSCompositeDestinationOver fraction:0.8];

		[dragImage unlockFocus];

		return [dragImage autorelease];
//	}
}
//------------------------------------
// validateDrop
//------------------------------------
- (NSDragOperation)tableView:(NSTableView*)tableView 
        validateDrop:(id <NSDraggingInfo>)info 
        proposedRow:(int)row 
        proposedDropOperation:(NSTableViewDropOperation)operation
{

    // Get pboard
    NSPasteboard*   pboard;
    pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

    if (!pboard) {
        return NSDragOperationNone;
    }

	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}

	// Check pboard type
	// from file path
    if ([[pboard types] containsObject:NSFilenamesPboardType]){
		// not accept drop from self
//		if([info draggingSourceOperationMask] == NSDragOperationLink){
//			return NSDragOperationNone;
//		}
		// drop above
		if(operation == NSTableViewDropAbove) {
			return NSDragOperationCopy;
		}
	}

	return NSDragOperationNone;

}
//------------------------------------
// acceptDrop
//------------------------------------
- (BOOL)tableView:(NSTableView*)tableView 
						acceptDrop:(id <NSDraggingInfo>)info 
						row:(int)row 
						dropOperation:(NSTableViewDropOperation)operation
{
	
	NSPasteboard *pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

    if (!pboard) {
        return NO;
    }

	// ignore self
	if (draggingSource == self) {
		return NO;
	}

	// Check pboard type
	// from file path
	if ([[pboard types] containsObject:NSFilenamesPboardType]){
		NSArray *filePaths = [pboard propertyListForType:NSFilenamesPboardType];
		if([filePaths count] > 0){	
			SEL sel = NSSelectorFromString(@"importItemWithFilePaths:");
			[viewMain performSelector:sel withObject:filePaths afterDelay:0.1];
			return YES;
		}
	}

	return NO;
}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
		[self rightMouseDown:theEvent];
		isAction = YES;
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)theEvent
{	

	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	BOOL isAction = NO;
	
	int i = [self rowAtPoint:p];
	
	if (i < [self numberOfRows]){
		// single selected
		if ([self numberOfSelectedRows] <= 1){
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
		}
		// multiple selected
		else{
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:YES];
		}
		[NSMenu popUpContextMenu:cmFilelist withEvent:theEvent forView:self];
		isAction = YES;
	}

	if(isAction == NO){
		[super rightMouseDown:theEvent];
	}

}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
