#import "FullScreenWindow.h"
#import "UserDefaultsExtension.h"

@implementation FullScreenWindow
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self setIsFullScreen:NO];
	[self setIsTopScreen:NO];
	[self setShowController:YES];

	oldRect_ = [self frame];

	// title bar height
	titleBarHeight_ = [self frame].size.height
					-([[self contentView] frame].size.height + [[self contentView] frame].origin.y);

//	NSLog(@"titleBarHeight_=%.2f", titleBarHeight_);

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleWindowFullScreenDidEntered:) name:WINDOW_NOTIF_FULLSCREEN_DID_ENTERED object:nil];
	[nc addObserver:self selector:@selector(handleWindowTopScreenDidEntered:) name:WINDOW_NOTIF_TOPSCREEN_DID_ENTERED object:nil];

}
//=======================================================================
// actions
//=======================================================================

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// changeFullScreen
//------------------------------------
- (void)changeFullScreen
{
	BOOL isFullScreen;

	// already full screen
	if([self isFullScreen] == YES){
		isFullScreen = NO;
	}
	// window
	else{
		isFullScreen = YES;
		// show controller
//		[self setShowController:YES];
		[self setShowController:NO];
	}

	// set full screen
	[self setFullScreen:isFullScreen animate:YES];

	// hide automaticaly
//	if(isFullScreen == YES){
//		SEL sel = NSSelectorFromString(@"changeFullScreenWithHideControllerWithTimer");
//		[self performSelector:sel withObject:nil afterDelay:1.0];
//	}

}
//------------------------------------
// changeFullScreenWithHideController
//------------------------------------
- (void)changeFullScreenWithHideController
{
	BOOL isFullScreen;

	// full screen
	if([self isFullScreen] == YES){
		// show controller 
		if([self showController] == NO){
			[self setShowController:YES];
		}
		// hide all
		else{
			[self setShowController:NO];
		}
		isFullScreen = YES;
	}
	// window
	else{
		// enter full screen & hide all
		[self setShowController:NO];
		isFullScreen = YES;
	}

	[self setFullScreen:isFullScreen animate:YES];

}
//------------------------------------
// changeFullScreenWithHideControllerWithTimer
//------------------------------------
- (void)changeFullScreenWithHideControllerWithTimer
{
	if([self isFullScreen] == YES){
		[self changeFullScreenWithHideController];
	}
}
//------------------------------------
// changeTopScreen
//------------------------------------
- (void)changeTopScreen
{
	[self changeWindowLevel];
}
//------------------------------------
// setFullScreen
//------------------------------------
- (void)setFullScreen:(BOOL)isFullScreen animate:(BOOL)animate
{

	// same style
//	if(isFullScreen == [self isFullScreen]){
//		return;
//	}
	BOOL isChanged = NO;
	if(isFullScreen != [self isFullScreen]){
		isChanged = YES;
	}

	// save oldrect
	if(isFullScreen == YES){
		if([self isFullScreen] == NO){
			oldRect_ = [self frame];
		}
	}

	[self setIsFullScreen:isFullScreen];

	if(isFullScreen == YES){
		[NSMenu setMenuBarVisible:NO];
		[self setShowsResizeIndicator:NO];
		[self setFrame:[[NSScreen mainScreen] frame] display:YES animate:animate];
		[self setLevel:NSNormalWindowLevel];
	}else{
		[NSMenu setMenuBarVisible:YES];
		[self setShowsResizeIndicator:YES];
		[self setFrame:oldRect_ display:YES animate:animate];
		[self restoreWindowLevel];

		// post notification
		[self postWindowFullScreenDidClosedNotification];
	}

//	[self setLevel:NSNormalWindowLevel];

	if(isChanged == YES){
		// post notification
		[self postWindowFullScreenDidChangedNotification:isFullScreen];
	}
}
//=======================================================================
// handler
//=======================================================================
//------------------------------------
// handleWindowFullScreenDidEntered
//------------------------------------
- (void)handleWindowFullScreenDidEntered:(NSNotification *)aNotification
{
	[self makeKeyAndOrderFront:self];
	[self changeFullScreen];
}
//------------------------------------
// handleWindowTopScreenDidEntered
//------------------------------------
- (void)handleWindowTopScreenDidEntered:(NSNotification *)aNotification
{
	[self changeTopScreen];
}
//=======================================================================
// post notification
//=======================================================================
//------------------------------------
// postWindowFullScreenDidChangedNotification
//------------------------------------
- (void)postWindowFullScreenDidChangedNotification:(BOOL)isFullScreen
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_FULLSCREEN_DID_CHANGED object:[NSNumber numberWithBool:isFullScreen]];
}
//------------------------------------
// postWindowFullScreenDidClosedNotification
//------------------------------------
- (void)postWindowFullScreenDidClosedNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_FULLSCREEN_DID_CLOSED object:self];
}
//------------------------------------
// postWindowTopScreenDidChangedNotification
//------------------------------------
- (void)postWindowTopScreenDidChangedNotification:(BOOL)isTopScreen
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_TOPSCREEN_DID_CHANGED object:[NSNumber numberWithBool:isTopScreen]];
}
//------------------------------------
// close
//------------------------------------
- (void)close
{
	if([self isFullScreen] == YES){
		[self setFullScreen:NO animate:YES];
	}else{
		[super close];
	}
	[self restoreWindowLevel];

	// post notification
	[self postWindowFullScreenDidChangedNotification:NO];
	[self postWindowFullScreenDidClosedNotification];

	// clear hide timer
//	[self setIsHideTimer:NO];
//	[self clearHideTimer];

}

//------------------------------------
// constrainFrameRect
//------------------------------------
-(NSRect)constrainFrameRect:(NSRect)frameRect toScreen:(NSScreen *)aScreen
{
	if([self isFullScreen] == YES){
		NSRect screenRect = [[NSScreen mainScreen] frame];
		screenRect.size.height += titleBarHeight_;

		// adjust control bar height
		if([self showController] == NO){
			screenRect.size.height += WINDOW_CONTROLLER_BAR_HEIGHT;
			screenRect.origin.y -= WINDOW_CONTROLLER_BAR_HEIGHT;
		}
		return screenRect;
	}else{
		return [super constrainFrameRect:frameRect toScreen:aScreen];
	}
}
//------------------------------------
// changeWindowLevel
//------------------------------------
- (void)changeWindowLevel
{
	BOOL isTopScreen;

	if([self level] == NSNormalWindowLevel){
		[self setLevel:NSModalPanelWindowLevel];
		isTopScreen = YES;
	}else{
		[self setLevel:NSNormalWindowLevel];
		isTopScreen = NO;
	}

	[self setIsTopScreen:isTopScreen];
	[self postWindowTopScreenDidChangedNotification:isTopScreen];
}
//------------------------------------
// restoreWindowLevel
//------------------------------------
- (void)restoreWindowLevel
{
	if([self isTopScreen] == YES){
		[self setLevel:NSModalPanelWindowLevel];
	}else{
		[self setLevel:NSNormalWindowLevel];
	}
}
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{    
	BOOL isAction = NO;

	// escape key
	if([theEvent keyCode] == 53){
		if([self isFullScreen] == YES){
			[self setFullScreen:NO animate:YES];
			isAction = YES;
		}
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}
}

//------------------------------------
// canBecomeKeyWindow
//------------------------------------
- (BOOL)canBecomeKeyWindow
{
    return YES;
}
//------------------------------------
// isFullScreen
//------------------------------------
- (void)setIsFullScreen:(BOOL)isFullScreen
{
	isFullScreen_ = isFullScreen;
}
- (BOOL)isFullScreen
{
    return isFullScreen_;
}
//------------------------------------
// isTopScreen
//------------------------------------
- (void)setIsTopScreen:(BOOL)isTopScreen
{
	isTopScreen_ = isTopScreen;
}
- (BOOL)isTopScreen
{
    return isTopScreen_;
}

//------------------------------------
// showController
//------------------------------------
- (void)setShowController:(BOOL)showController
{
	showController_ = showController;
}
- (BOOL)showController
{
    return showController_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}
@end
