#import "GalleryWindow.h"

static const float FADE_DURATION = 0.5;

@implementation GalleryWindow

//------------------------------------
// initWithContentRect
//------------------------------------
- (id)initWithContentRect:(NSRect)contentRect 
				styleMask:(unsigned int)aStyle 
				  backing:(NSBackingStoreType)bufferingType 
					defer:(BOOL)flag {
	
	if (self = [super initWithContentRect:contentRect 
										styleMask:NSBorderlessWindowMask 
										  backing:NSBackingStoreBuffered 
								   defer:NO]) {
		[self setLevel:NSStatusWindowLevel];
//		[self setLevel:NSModalPanelWindowLevel];
		[self setBackgroundColor:[NSColor clearColor]];
		[self setAlphaValue:1.0];
		[self setOpaque:NO];
		[self setShowsResizeIndicator:NO];
		[self setHasShadow:NO];
		
		return self;
	}

	return nil;
}
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self setWindowRect:YES];

}
//=======================================================================
// action
//=======================================================================
//=======================================================================
// method
//=======================================================================
//------------------------------------
// open
//------------------------------------
- (void)open
{

//	[self center];
	[self setAlphaValue:0.0];
	[self makeKeyAndOrderFront:self];
	[self fadeInWindow];

}
//------------------------------------
// close
//------------------------------------
- (void)close
{

	if([self isVisible] == YES){
		[self fadeOutWindow];
//		[super close];
	}

}
//------------------------------------
// setWindowRect
//------------------------------------
- (void)setWindowRect:(BOOL)isPos
{
	// set window size
	NSRect frame = [[NSScreen mainScreen] frame];
	NSRect rect = [self frame];
	rect.size.width = frame.size.width * 0.8;
	rect.size.height = frame.size.height * 0.5;

	// set window position
	if(isPos == YES){
		rect.origin.x = (frame.size.width - rect.size.width) / 2;
		rect.origin.y = (frame.size.height - rect.size.height) / 2;
	}

	[self setFrame:rect display:NO animate:NO];
}
//------------------------------------
// fadeInWindow
//------------------------------------
- (void)fadeInWindow
{

	NSMutableDictionary* dict = [NSMutableDictionary dictionary];
	[dict setObject:self forKey:NSViewAnimationTargetKey];
	[dict setObject:NSViewAnimationFadeInEffect forKey:NSViewAnimationEffectKey];
	
	NSViewAnimation *anim = [[NSViewAnimation alloc] initWithViewAnimations:
											[NSArray arrayWithObject:dict]
								];
	[anim setDuration:FADE_DURATION];
	[anim setAnimationCurve:NSAnimationEaseIn];
	
	[anim startAnimation];
	[anim release];

}
//------------------------------------
// fadeOutWindow
//------------------------------------
- (void)fadeOutWindow
{

	NSMutableDictionary* dict = [NSMutableDictionary dictionary];
	[dict setObject:self forKey:NSViewAnimationTargetKey];
	[dict setObject:NSViewAnimationFadeOutEffect forKey:NSViewAnimationEffectKey];
	
	NSViewAnimation *anim = [[NSViewAnimation alloc] initWithViewAnimations:
											[NSArray arrayWithObject:dict]
								];
	[anim setDuration:FADE_DURATION];
	[anim setAnimationCurve:NSAnimationEaseIn];
	[anim setDelegate:self];

	[anim startAnimation];
	[anim release];

}
//------------------------------------
// animationDidEnd
//------------------------------------
- (void)animationDidEnd:(NSAnimation *)animation
{

	[super close];

	// post nitification
	[self postWindowClosedNotification];

}
//------------------------------------
// postWindowClosedNotification
//------------------------------------
- (void)postWindowClosedNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_GALLERY_DID_CLOSED object:self];
}
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{	 
	BOOL isAction = NO;

/*
	NSString *chars = [theEvent charactersIgnoringModifiers];
//	NSLog(@"chars=%@", chars);
	if(chars && [chars length] > 0){
		// cmd + w
		if([[chars lowercaseString] isEqualToString:@"w"]){
			if([theEvent modifierFlags] & NSCommandKeyMask){
				[self close];
				isAction = YES;
			}
		}
	}
*/

	// escape key
	if([theEvent keyCode] == 53){
		[self close];
		isAction = YES;
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}
}

//------------------------------------
// mouseDragged
//------------------------------------
- (void)mouseDragged:(NSEvent *)theEvent
{

	NSPoint currentLocation;
	currentLocation = [self convertBaseToScreen:[self mouseLocationOutsideOfEventStream]];

	NSPoint newOrigin;
	NSRect screenFrame = [[NSScreen mainScreen] frame];
	NSRect windowFrame = [self frame];

	newOrigin.x = currentLocation.x - initialLocation.x;
	newOrigin.y = currentLocation.y - initialLocation.y;

//	NSLog(@"currentLocation=%.2f, %.2f", currentLocation.x, currentLocation.y);
//	NSLog(@"initialLocation=%.2f, %.2f", initialLocation.x, initialLocation.y);
//	NSLog(@"newOrigin=%.2f, %.2f", newOrigin.x, newOrigin.y);

	if( (newOrigin.y + windowFrame.size.height) > (NSMaxY(screenFrame) - [NSMenuView menuBarHeight]) ){
		// Prevent dragging into the menu bar area
		newOrigin.y = NSMaxY(screenFrame) - windowFrame.size.height - [NSMenuView menuBarHeight];
	}

/*
	if (newOrigin.y < NSMinY(screenFrame)) {
		// Prevent dragging off bottom of screen
		newOrigin.y = NSMinY(screenFrame);
	}
	if (newOrigin.x < NSMinX(screenFrame)) {
		// Prevent dragging off left of screen
		newOrigin.x = NSMinX(screenFrame);
	}
	if (newOrigin.x > NSMaxX(screenFrame) - windowFrame.size.width) {
		// Prevent dragging off right of screen
		newOrigin.x = NSMaxX(screenFrame) - windowFrame.size.width;
	}
*/

	[self setFrameOrigin:newOrigin];

}

//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	 
	NSRect windowFrame = [self frame];
	
	// Get mouse location in global coordinates
	initialLocation = [self convertBaseToScreen:[theEvent locationInWindow]];
	initialLocation.x -= windowFrame.origin.x;
	initialLocation.y -= windowFrame.origin.y;

}
//------------------------------------
// canBecomeKeyWindow
//------------------------------------
- (BOOL)canBecomeKeyWindow
{
	return YES;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

@end
