#import "ImageAttributeExtension.h"

@implementation NSObject(imageAttributeExtension_)

//------------------------------------
// setScaledImageSize
//------------------------------------
- (NSSize)setScaledImageSize:(NSSize)imageSize frameSize:(NSSize)frameSize
{

	float imageWidth = imageSize.width;
	float imageHeight = imageSize.height;

	if(imageWidth < 1){imageWidth = 1;}
	if(imageHeight < 1){imageHeight = 1;}

	float frameWidth = frameSize.width;
	float frameHeight = frameSize.height;
	float imgSizeScale = 1.0;

	if(imageWidth > imageHeight){
		imgSizeScale = frameWidth / imageWidth;
	}else{
		imgSizeScale = frameHeight / imageHeight;
	}
	imageWidth = imageWidth * imgSizeScale;
	imageHeight = imageHeight * imgSizeScale;

	if(imageWidth < 1){imageWidth = 1;}
	if(imageHeight < 1){imageHeight = 1;}

	return NSMakeSize(imageWidth,imageHeight);
}
//------------------------------------
// create grayscale image
//------------------------------------
- (NSImage*)createGrayScaleImage:(NSImage*)theImage
{

	if(theImage == nil){
		return theImage;
	}

	// get NSImage to bitmaprep
	NSBitmapImageRep *abitmap;
	abitmap = (NSBitmapImageRep*)[theImage bestRepresentationForDevice:nil];
	int pw =  [abitmap pixelsWide];
	int ph = [abitmap pixelsHigh];

	NSBitmapImageRep *bitmap = [[NSBitmapImageRep alloc]
									initWithBitmapDataPlanes:NULL
									pixelsWide:pw
									pixelsHigh:ph
									bitsPerSample:8
									samplesPerPixel:1
									hasAlpha:NO
									isPlanar:NO
									colorSpaceName:NSCalibratedWhiteColorSpace
									bytesPerRow:0
									bitsPerPixel:0
								];
	   
	[bitmap setSize:[theImage size]];

	NSImage *image =[[NSImage alloc] initWithSize:[theImage size]];

	NSGraphicsContext *nsContext = [NSGraphicsContext graphicsContextWithBitmapImageRep:bitmap];

	[NSGraphicsContext saveGraphicsState];

	[NSGraphicsContext setCurrentContext:nsContext];

	// Do I need a lockFocus here?
	[theImage drawAtPoint:NSMakePoint(0,0) fromRect:NSZeroRect operation:NSCompositeCopy fraction:1.0];

	// Restore the previous graphics context and state.
	[NSGraphicsContext restoreGraphicsState];

	[image addRepresentation:bitmap];
	[bitmap release];

	return [image autorelease];

}

//------------------------------------
// saveImageAsPng
//------------------------------------
- (BOOL)saveImageAsPng:(NSImage *)image
				filePath:(NSString *)filePath
				Interlaced:(BOOL)interlaced
{

	NSBitmapImageRep *bitmapImageRep;
	NSDictionary *params;
	NSData *data;

	params = [NSDictionary dictionaryWithObjectsAndKeys:
					[NSNumber numberWithBool:interlaced], NSImageInterlaced,
					nil
			];

	bitmapImageRep = [NSBitmapImageRep imageRepWithData:[image TIFFRepresentation]];
	data = [bitmapImageRep representationUsingType:NSPNGFileType properties:params];

	return [data writeToFile:filePath atomically:YES];

}

@end