#import "MenuItemLabelColor.h"
#import "TBArrayController.h"
#import "CellAttributeExtension.h"

@implementation MenuItemLabelColor

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"labelColorMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate:self];

}

//------------------------------------
// changeLabelColor
//------------------------------------
- (IBAction)changeLabelColor:(id)sender
{
	int colorNo = [[sender representedObject] intValue];
	id record;

	// filelist
	if([self tag] == 0){
		// get objects from filelist
		NSArray *selectedObjects = [tbArrayController getSelectedObjects:@"filelist"];

		// update filelist
		NSEnumerator *enumObject = [selectedObjects objectEnumerator];
		while (record = [enumObject nextObject]) {
			[record setValue:[NSNumber numberWithInt:colorNo] forKey:@"labelColorNo"];
		}
	}
	// playlist
	else if([self tag] == 1){
		record = [tbArrayController getSelectedObject:@"playlist"];
		if (record != nil) {
			[record setValue:[NSNumber numberWithInt:colorNo] forKey:@"labelColorNo"];
		}
	}

}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	NSArray *cols;
	NSString *itemName;
	NSColor *itemColor;
	int colorNo;
	int i;
	id record;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuArray = [self labelColorList];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuArray count]; i++){

		cols = [[menuArray objectAtIndex:i] componentsSeparatedByString:@","];
		
		// add item separator
        if([[cols objectAtIndex:0] isEqualToString:@"-"]){
			[aMenu addItem:[NSMenuItem separatorItem]];
			continue;
		}

		itemName = [cols objectAtIndex:0];
		colorNo = [[cols objectAtIndex:1] intValue];

		// label color
		itemColor = [self getLabelColor:colorNo];
		if(itemColor == nil){
 			itemColor = [NSColor blackColor];
		}
 
		NSDictionary *attributes = [NSDictionary dictionaryWithObjectsAndKeys:
										itemColor, NSForegroundColorAttributeName,
										nil
									];

		// set action
 		SEL sel = NSSelectorFromString(@"changeLabelColor:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:@"" action:sel keyEquivalent:@""] autorelease];
		[menuItem setAttributedTitle:[[[NSMutableAttributedString alloc] initWithString:itemName attributes:attributes] autorelease]];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:[NSNumber numberWithInt:colorNo]];

		[aMenu addItem:menuItem];
	}

	[self setSubmenu:aMenu];

//	[menuArray release];

}
//------------------------------------
// updateSubMenu
//------------------------------------
- (void)updateSubMenu:(NSMenu*)aMenu
{

	int colorNo;
	id menuItem;
	NSArray *selectedObjects;

	// get selected object
	if([self tag] == 0){
		selectedObjects = [[tbArrayController getSelectedObjects:@"filelist"] valueForKey:@"labelColorNo"];
	}
	else if([self tag] == 1){
		selectedObjects = [[tbArrayController getSelectedObjects:@"playlist"] valueForKey:@"labelColorNo"];
	}

	NSEnumerator *enumArray = [[aMenu itemArray] objectEnumerator];
	while (menuItem = [enumArray nextObject]) {
		colorNo = [[menuItem representedObject] intValue];
		// set state
		if(colorNo > 0 && [selectedObjects containsObject:[NSNumber numberWithInt:colorNo]]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}
	}

}

//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateSubMenu:menu];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
    [super dealloc];
}


@end
