#import "MenuItemShowInPlaylist.h"
#import "TBArrayController.h"
#import "ViewPlaylist.h"

@implementation MenuItemShowInPlaylist

//=======================================================================
// awakeFromNib
//=======================================================================
- (void)awakeFromNib
{
	// createSubMenu
	menuShowPlaylist_ = [[[NSMenu alloc] initWithTitle:@"ShowPlaylistMenu"] autorelease];
	[self setSubMenuPlaylist:menuShowPlaylist_];

	//set delegate menu
	[menuShowPlaylist_ setDelegate: self ];
}


//------------------------------------
// setSubMenuPlaylist
//------------------------------------
-(void)setSubMenuPlaylist:(NSMenu*)aMenu
{
	id record;
	NSString *itemName;
	NSImage *iconImage;
	NSManagedObject *parentObject;
	NSManagedObject *childObject;

	// remove menuItem
	NSEnumerator *enumArray = [[aMenu itemArray] objectEnumerator];
	while (record = [enumArray nextObject]) {
		[aMenu removeItem:record];
	}

	// get selected object
	NSArray *selectedArray = [tbArrayController getSelectedObjects:@"filelist"];

	// get related playlist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"ANY rlt_filelist IN %@", selectedArray];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// get parent item
	NSMutableArray *parentArray = [[NSMutableArray alloc] init];
	int i;
	id parentItem = nil;
	for(i = 0; i < [fetchArray count]; i++){
		parentItem = [[fetchArray objectAtIndex:i] valueForKey:@"parent"];
		if(parentItem){
			if(![parentArray containsObject:parentItem]){
				[parentArray addObject:parentItem];
			}
		}else{
			[parentArray addObject:[fetchArray objectAtIndex:i]];
		}
	}

	// sort
	NSSortDescriptor *desc1=[[[NSSortDescriptor alloc] initWithKey:@"isFolder" ascending:NO selector:nil] autorelease];
	NSSortDescriptor *desc2=[[[NSSortDescriptor alloc] initWithKey:@"title" ascending:YES selector:NSSelectorFromString(@"caseInsensitiveCompare:")] autorelease];
	NSSortDescriptor *desc3=[[[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES] autorelease];
	NSArray *sortedArray = [parentArray sortedArrayUsingDescriptors:[NSArray arrayWithObjects:desc1,desc2,nil]];

	// add library menu
	SEL sel = nil;
	if([viewPlaylist isSelectItem] == YES || [viewPlaylist isSelectFolder] == YES){
		sel = NSSelectorFromString(@"showItem:");
	}

	NSMenuItem *menuItemTop = [[[NSMenuItem alloc] initWithTitle:@"Library" action:sel keyEquivalent:@""] autorelease];
	[menuItemTop setTarget:viewPlaylist];
	[menuItemTop setRepresentedObject:nil];
	[aMenu addItem:menuItemTop];

	// add separator
	[aMenu addItem:[NSMenuItem separatorItem]];

	// add item from parent
	NSEnumerator *enumParents = [sortedArray objectEnumerator];
	NSMenuItem *menuItem;
	while (parentObject = [enumParents nextObject]) {

		itemName = [parentObject valueForKey: @"title" ];

		// playlist
		if([[parentObject valueForKey:@"isFolder"] boolValue] == NO){
			iconImage = [NSImage imageNamed:@"icon_playlist"];
		}
		// folder
		else{
			iconImage = [NSImage imageNamed:@"icon_folder"];
		}

		menuItem = [[[NSMenuItem alloc] initWithTitle:itemName action:@selector(showItem:) keyEquivalent:@""] autorelease];
		[menuItem setTarget:viewPlaylist];
		[menuItem setImage:iconImage];
		[menuItem setRepresentedObject:parentObject];
		
		[aMenu addItem:menuItem];

		// add item from children
		if([parentObject valueForKey:@"children"]){
			NSArray *childrenArray = [[parentObject valueForKey:@"children"] allObjects];
			childrenArray = [childrenArray sortedArrayUsingDescriptors:[NSArray arrayWithObject:desc3]];

			NSEnumerator *enumChildren = [childrenArray objectEnumerator];
			while (childObject = [enumChildren nextObject]) {

				// skip folder
				if([[childObject valueForKey:@"isFolder"] boolValue] == YES){
					continue;
				}
				// not contains
				if(![fetchArray containsObject:childObject]){
					continue;
				}

				itemName = [childObject valueForKey: @"title" ];

				menuItem = [[[NSMenuItem alloc] initWithTitle:itemName action:@selector(showItem:) keyEquivalent:@""] autorelease];
				[menuItem setIndentationLevel:1];
				[menuItem setTarget:viewPlaylist];
				[menuItem setRepresentedObject:childObject];
				[menuItem setImage:[NSImage imageNamed:@"icon_playlist"]];
				 
				[aMenu addItem:menuItem];

			}
		}

		// add separator
		[aMenu addItem:[NSMenuItem separatorItem]];
	}

	[self setSubmenu:aMenu];

	[parentArray release];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[menuShowPlaylist_ release];	
    [super dealloc];
}

//=======================================================================
#pragma mark - Delegate -
//=======================================================================
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self setSubMenuPlaylist:menu];
}


@end
