#import "MoviePlayerView.h"
#import "FileAttributeExtension.h"
#import "MovieAttributeExtension.h"
#import "UserDefaultsExtension.h"
#import "FullScreenWindow.h"

static NSString *defaultKeyPlayVolume = @"optPlayVolume";
static NSString *defaultKeyMovieKeepAspect = @"optMovieKeepAspect";

@implementation MoviePlayerView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self setPlayRate:1.0];
	[self setIsCurrentPlaying:NO];
	[self setIsBeforeProcPlaying:NO];
	[self setIsAvailable:NO];

	[self setErrorDescription:@""];
	[self setControllerVisible:NO];
	[self setMovieVolume:[self defaultPlayVolume]];
	[self setMovieAspectRatio:[self defaultMovieKeepAspect]];

	[self setTimeStatusTimer:nil];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];

	[nc addObserver:self selector:@selector(handleMoviePlayDidChanged:) name:QTMovieRateDidChangeNotification object:[self movie]];
	[nc addObserver:self selector:@selector(handleMoviePlayDidEnded:) name:QTMovieDidEndNotification object:[self movie]];
	[nc addObserver:self selector:@selector(handleMovieVolumeDidChanged:) name:QTMovieVolumeDidChangeNotification object:[self movie]];
	[nc addObserver:self selector:@selector(handleMovieLoadStateDidChanged:) name:QTMovieLoadStateDidChangeNotification object:[self movie]];

	[nc addObserver:self selector:@selector(handleWindowFullScreenDidChanged:) name:WINDOW_NOTIF_FULLSCREEN_DID_CHANGED object:nil];
	[nc addObserver:self selector:@selector(handleVideoProcDidChanged:) name:VIDEO_NOTIF_PROC_DID_CHANGED object:nil];
	[nc addObserver:self selector:@selector(handleVideoPlayDidChanged:) name:VIDEO_NOTIF_PLAY_DID_CHANGED object:nil];
	[nc addObserver:self selector:@selector(handleVideoSizeScaleDidChanged:) name:VIDEO_NOTIF_SIZE_SCALE_DID_CHANGED object:nil];
	[nc addObserver:self selector:@selector(handleVideoRateDidEntered:) name:VIDEO_NOTIF_RATE_DID_ENTERED object:nil];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:defaultKeyPlayVolume options:0 context:nil];
	[defaults addObserver:self forKeyPath:defaultKeyMovieKeepAspect options:0 context:nil];

}

//=======================================================================
// Action
//=======================================================================
- (void)viewWillStartLiveResize
{
	[self postVideoSizeInResizeNotification:YES];
}
- (void)viewDidEndLiveResize
{
	[self postVideoSizeInResizeNotification:NO];
}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// setPlayerView
//------------------------------------
- (BOOL)setPlayerView:(NSDictionary*)params isPlay:(BOOL)isPlay
{

	// stop movie
	[self setMoviePlay:NO isAction:NO];

	NSError *error = nil;
	QTMovie *movie = nil;
	BOOL hasDuration = NO;
	BOOL isAvailable = NO;

	[self postVideoLoadingChangedNotification:YES];

	NSString *filePath = [params valueForKey:@"filePath"];
	NSString *title = [params valueForKey:@"title"];

	// create movie
	if([self checkFileExist:filePath] == FILE_EXIST_FILE){
		movie = [QTMovie movieWithFile:filePath error:&error];
		if(movie != nil && error == nil){
			isAvailable = YES;
			float duration = [movie duration].timeValue / [movie duration].timeScale;
			if(duration > 0.0){
				hasDuration = YES;
			}else{
				hasDuration = NO;
			}
		}
	}

	// failed or not available
	if(isAvailable == NO){
		movie = [QTMovie movieWithFile:[[NSBundle mainBundle] pathForResource:@"image_nomovie" ofType:@"png"] error:&error];
	}
/*
	// error
	if(error != nil){
		[self setErrorDescription:[NSString stringWithFormat:@"error = %@", error]];
		return NO;
	}
	// error
	if(movie == nil){
		[self setErrorDescription:@"Can not create movie"];
		return NO;
	}
*/

/*
	// add sound image
	float duration = [movie duration].timeValue / [movie duration].timeScale;
	NSSize naturalSize = [[movie attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 
	if(duration > 0.0 && naturalSize.width <= 0 && naturalSize.height <= 0){
		[movie setAttribute:[NSNumber numberWithBool:YES] forKey:QTMovieEditableAttribute];
		[movie addImage:[NSImage imageNamed:@"image_sound"]
			forDuration:[movie duration]
			withAttributes:[NSDictionary dictionaryWithObjectsAndKeys:
								@"png", QTAddImageCodecType,
								[NSNumber numberWithLong:duration], QTTrackTimeScaleAttribute,
								nil
							]
		];
	}
*/

	// set attribute
	[movie gotoBeginning];
	[movie setAttribute:[NSNumber numberWithBool:NO] forKey:QTMovieLoopsAttribute];
	[movie setAttribute:[NSNumber numberWithBool:NO] forKey:QTMovieLoopsBackAndForthAttribute];

	// set movie to view
	[self setMovie:movie];
	[self setMovieVolume:[self defaultPlayVolume]];
	[self setIsAvailable:isAvailable];
	[[self window] setTitle:title];

	if(isAvailable == YES){
		[self setMoviePlay:isPlay isAction:NO];
	}

	[self setNeedsDisplay:YES];

	// post notification
	[self postVideoLoadingChangedNotification:NO];
	[self postVideoObjectChangedNotification:[NSDictionary dictionaryWithObjectsAndKeys:
													[NSNumber numberWithBool:YES], @"hasVideo" ,
													[NSNumber numberWithBool:hasDuration], @"hasDuration" ,
													[NSNumber numberWithBool:isAvailable], @"isAvailable" ,
													nil
												]
	];
	[self postVideoLoadedChangedNotification:YES];
	[self postVideoStatusChangedNotification:
				[NSDictionary dictionaryWithObjectsAndKeys:
										title, @"title",
										nil
				]
	];


	[self initTimeStatus];

	return YES;

}

//------------------------------------
// closeMovieView
//------------------------------------
- (void)deletePlayerView
{

	[self setMoviePlay:NO isAction:NO];
	[self setErrorDescription:@""];

	[self clearTimeStatusTimer];
	[self setTimeStatusTimer:nil];

//	[self setMovie:nil];
//	[self postClearNotifications];

}
//------------------------------------
// setMoviePlay
//------------------------------------
- (void)setMoviePlay:(BOOL)isPlay isAction:(BOOL)isAction
{
	if([self hasMovie] == YES){

		// need before rate is changed
		[self setIsCurrentPlaying:isPlay];

		if(isPlay == YES){
			[[self movie] play];
		}else{
			[[self movie] stop];
		}
		// post notifications
//		if([self isFullScreen] == NO){
			if(isAction == YES){
				[self postVideoPlayActionDidChangedNotification:isPlay];
			}
//		}
	}

}
//----------------------------------------
// setMovieVolume
//----------------------------------------
- (void)setMovieVolume:(float)volume
{
	// set volume
	if([self hasMovie] == YES){
		[[self movie] setVolume:volume];
	}
}
//----------------------------------------
// setMovieRate
//----------------------------------------
- (void)setMovieRate:(float)rate
{

	[self setPlayRate:rate];

	if([self isPlaying] == YES){
		[self setMoviePlay:YES isAction:NO];
	}

	// post notification
	[self postVideoRateChangedNotification:rate];		

}
//----------------------------------------
// setMovieAspectRatio
//----------------------------------------
- (void)setMovieAspectRatio:(BOOL)keepAspect
{
	[self setPreservesAspectRatio:keepAspect];
}
//----------------------------------------
// replayMovie
//----------------------------------------
- (void)replayMovie
{
	if([self hasMovie] == YES){
		[self gotoMovieTime:VIDEO_GO_TO_START];		
		[self setMoviePlay:YES isAction:NO];
	}
}
//----------------------------------------
// setMovieCurrentTime
//----------------------------------------
- (void)setMovieCurrentTime:(NSTimeInterval)value
{

	QTMovie *movie = [self movie];

	if(movie != nil){
		QTTime time;
		time.timeValue = value;
		time.timeScale = [movie duration].timeScale;
		time.flags = 0;

		[movie setCurrentTime:time];

		// update time status
		[self updateTimeStatus];

	}
}
//----------------------------------------
// setMovieCurrentTimeWithSecond
//----------------------------------------
- (void)setMovieCurrentTimeWithSecond:(float)sec;
{

	QTMovie *movie = [self movie];

	if(movie != nil){

		NSTimeInterval value = [movie duration].timeScale * sec;

		[self setMovieCurrentTime:value];

	}
}
//----------------------------------------
// skipMovieTime
//----------------------------------------
- (void)skipMovieTime:(float)interval
{

	QTMovie *movie = [self movie];

	if(movie != nil){

		QTTime time = [movie currentTime];
		QTTime duration = [movie duration];

		float sec = time.timeValue / time.timeScale;
		sec += interval;

		time.timeScale = duration.timeScale;
		time.timeValue = sec * time.timeScale;

		// over duration
		if(time.timeValue >= duration.timeValue){
			time.timeValue = duration.timeValue;
			// stop movie
			[self setIsCurrentPlaying:NO];
			[self setMoviePlay:NO isAction:NO];
		}
		if(time.timeValue < 0){
			time.timeValue = 0;
		}

		[movie setCurrentTime:time];

		// update time status
		[self updateTimeStatus];

		// post notifications
//		if([self isFullScreen] == NO){
			[self postVideoPlayActionDidChangedNotification:NO];
//		}
	}
}
//----------------------------------------
// gotoMovieTime
//----------------------------------------
- (void)gotoMovieTime:(int)toPos
{

	NSTimeInterval value;

	if(toPos == VIDEO_GO_TO_START){
		value = 0.0;
	}
	else if(toPos == VIDEO_GO_TO_END){
		value = [[self movie] duration].timeValue;
		// stop movie
		[self setIsCurrentPlaying:NO];
		[self setMoviePlay:NO isAction:NO];
	}

	[self setMovieCurrentTime:value];

	// post notifications
//	if([self isFullScreen] == NO){
		[self postVideoPlayActionDidChangedNotification:NO];
//	}

}
//------------------------------------
// startTimeStatusTimer
//------------------------------------
- (void)startTimeStatusTimer
{

//	NSLog(@"startTimeStatusTimer");
	// clear timer
	[self clearTimeStatusTimer];

	if([self hasMovie] == YES){
		// set timer for update time status
		[self setTimeStatusTimer:[NSTimer scheduledTimerWithTimeInterval:1.0
									target:self 
									selector:@selector(updateTimeStatus)
									userInfo:nil
									repeats:YES
								]
		];
		[[NSRunLoop currentRunLoop] addTimer:[self timeStatusTimer] forMode:(NSString*)kCFRunLoopCommonModes];
	}else{
		[self setTimeStatusTimer:nil];
	}

}
//------------------------------------
// initTimeStatus
//------------------------------------
- (void)initTimeStatus
{

//	NSLog(@"initTimeStatus");
	QTMovie *movie = [self movie];

	// time status
	if(movie != nil){

		QTTime qtCurrentTime = [movie currentTime];
		QTTime qtDuration = [movie duration];

		int currentTime = qtCurrentTime.timeValue / qtCurrentTime.timeScale;
		int duration = qtDuration.timeValue / qtDuration.timeScale;
		double durationValue = qtDuration.timeValue;
//		double currentValue = qtCurrentTime.timeValue;
		double currentValue = 0.0;

	//	NSLog(@"initTimeStatus currentTime=%d duration=%d", currentTime, duration);
		[self postVideoTimeInitedNotification:[NSDictionary dictionaryWithObjectsAndKeys:
													[NSNumber numberWithInt:currentTime], @"currentTime" ,
													[NSNumber numberWithInt:duration], @"duration" ,
													[NSNumber numberWithDouble:currentValue], @"currentValue" ,
													[NSNumber numberWithDouble:durationValue], @"durationValue" ,
													nil
												]
		];

	}

}
//------------------------------------
// updateTimeStatus
//------------------------------------
- (void)updateTimeStatus
{

//	NSLog(@"updateTimeStatus");
	QTMovie *movie = [self movie];

	// time status
	if(movie != nil){

		QTTime qtCurrentTime = [movie currentTime];
		QTTime qtDuration = [movie duration];

		int currentTime = qtCurrentTime.timeValue / qtCurrentTime.timeScale;
		int duration = qtDuration.timeValue / qtDuration.timeScale;
		double durationValue = qtDuration.timeValue;
		double currentValue = qtCurrentTime.timeValue;

	//	NSLog(@"updateTimeStatus currentTime=%d duration=%d", currentTime, duration);
		[self postVideoTimeChangedNotification:[NSDictionary dictionaryWithObjectsAndKeys:
													[NSNumber numberWithInt:currentTime], @"currentTime" ,
													[NSNumber numberWithInt:duration], @"duration" ,
													[NSNumber numberWithDouble:currentValue], @"currentValue" ,
													[NSNumber numberWithDouble:durationValue], @"durationValue" ,
													nil
												]
		];

	}

}
//------------------------------------
// changeVideoSizeScale
//------------------------------------
- (void)changeVideoSizeScale:(float)scale
{

	// control bar height
//	int offsetBottomY = 30;
	float offsetBottomY = [self frame].origin.y;
	float offsetTopY = [[self window] frame].size.height - ([self frame].size.height + offsetBottomY);

	// get original movie size
	NSSize baseSize = [[[self movie] attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 

	// default
	if(baseSize.width <= 0 || baseSize.height <= 0){
		baseSize.width = 240;
		baseSize.height = 180;
	}

	// scale size
	NSSize scaledSize = NSZeroSize;
	scaledSize.width = baseSize.width * scale;
	scaledSize.height = baseSize.height * scale;

	// adjust screen rect
	NSRect screenRect = [[NSScreen mainScreen] visibleFrame];
	if(scaledSize.width > screenRect.size.width){
		scale = screenRect.size.width / scaledSize.width;
		scaledSize.width *= scale;
		scaledSize.height *= scale;
	}
	if(scaledSize.height > (screenRect.size.height - offsetBottomY)){
		scale = (screenRect.size.height - offsetBottomY) / scaledSize.height;
		scaledSize.width *= scale;
		scaledSize.height *= scale;
	}

	// adjust min size
	NSSize minSize = [[self window] minSize];
	if(scaledSize.width < minSize.width){
		scaledSize.width = minSize.width;
	}
	if(scaledSize.height < (minSize.height - offsetBottomY - offsetTopY)){
		scaledSize.height = (minSize.height - offsetBottomY - offsetTopY);
	}

	// get current window size
	NSRect oldRect = [NSWindow contentRectForFrameRect:[[self window] frame] styleMask:[[self window] styleMask]];

	// set point
	NSPoint newPoint;
//	newPoint.x = NSMinX(oldRect);
//	newPoint.y = NSMaxY(oldRect) - (scaledSize.height + offsetBottomY);
	// centering x only
	newPoint.x = NSMinX(oldRect) - ((scaledSize.width / 2) -  (oldRect.size.width / 2));
//	newPoint.y = NSMinY(oldRect) - (((scaledSize.height + offsetBottomY) / 2) -  (oldRect.size.height / 2));
	newPoint.y = NSMaxY(oldRect) - (scaledSize.height + offsetBottomY);

	// set new window size
	NSRect newRect = [NSWindow frameRectForContentRect:
													NSMakeRect( 
																newPoint.x,
																newPoint.y,
																scaledSize.width,
																scaledSize.height + offsetBottomY)
												styleMask:[[self window] styleMask]
						 ];

	[[self window] setFrame:newRect display:YES animate:[[self window] isVisible]];

}

//=======================================================================
// handler
//=======================================================================
//------------------------------------
// handleMoviePlayDidChanged
//------------------------------------
- (void)handleMoviePlayDidChanged:(NSNotification *)aNotification
{

	BOOL isPlaying = [self isPlaying];

	// post notification
	[self postVideoPlayChangedNotification:isPlaying];

}
//------------------------------------
// handleMoviePlayDidEnded
//------------------------------------
- (void)handleMoviePlayDidEnded:(NSNotification *)aNotification
{

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_PLAY_DID_ENDED object:self];

}

//------------------------------------
// handleMovieVolumeDidChanged
//------------------------------------
- (void)handleMovieVolumeDidChanged:(NSNotification *)aNotification
{

//	NSLog(@"handleMovieVolumeDidChanged");
	float volume = [[self movie] volume];

	[self setDefaultFloatValue:volume key:defaultKeyPlayVolume];

}
//------------------------------------
// handleMovieLoadStateDidChanged
//------------------------------------
- (void)handleMovieLoadStateDidChanged:(NSNotification *)aNotification
{

//	NSLog(@"handleMovieLoadStateDidChanged");

	QTMovie *movie = [aNotification object];

	long loadState = [[movie attributeForKey:QTMovieLoadStateAttribute] longValue];
	// kMovieLoadStatePlaythroughOK
	if (loadState >= 20000){
		// set notification
/*
		NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];

		[self setVideoVolume:[self defaulPlayVolume]];
		[self setVideoPlay:[self defaultQTMovieAutoPlay]];
*/
	}
	// kMovieLoadStatePlayable
	else if (loadState >= 10000){

	}
	// error
	else if (loadState == -1){
	}

}
//------------------------------------
// handleVideoProcDidChanged
//------------------------------------
- (void)handleVideoProcDidChanged:(NSNotification *)aNotification
{
	BOOL isProc = [[aNotification object] boolValue];

	if(isProc == YES){
		// save playing status
		[self setIsBeforeProcPlaying:[self isPlaying]];

		// stop
		[self setMoviePlay:NO isAction:NO];
	}else{
		// stop
		if([self isEndFrame] == YES){
			[self setMoviePlay:NO isAction:NO];
		}
		// restart
		else{
			[self setMoviePlay:[self isBeforeProcPlaying] isAction:NO];
		}
	}
}
//------------------------------------
// handleWindowFullScreenDidChanged
//------------------------------------
- (void)handleWindowFullScreenDidChanged:(NSNotification *)notification
{

	// ful screen
	BOOL isFullScreen = [[notification object] boolValue];
	if(isFullScreen == YES){
		// start tracking
		[self createMouseTracking];
	}
	// window
	else{
		// clear tracking
		[self removeMouseTracking];
	}

}
//------------------------------------
// handleVideoSizeScaleDidChanged
//------------------------------------
- (void)handleVideoSizeScaleDidChanged:(NSNotification *)notification
{
	float scale = [[notification object] floatValue];

	if([self hasMovie] == YES){
		[self changeVideoSizeScale:scale];
	}
}
//------------------------------------
// handleVideoRateDidEntered
//------------------------------------
- (void)handleVideoRateDidEntered:(NSNotification *)notification
{
	float rate = [[notification object] floatValue];

	if([self hasMovie] == YES){
		[self setMovieRate:rate];
	}
}

//=======================================================================
// post notifications
//=======================================================================
//------------------------------------
// postVideoObjectChangedNotification
//------------------------------------
- (void)postVideoObjectChangedNotification:(NSDictionary*)params
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_OBJECT_DID_CHANGED object:params];
}
//------------------------------------
// postVideoLoadingChangedNotification
//------------------------------------
- (void)postVideoLoadingChangedNotification:(BOOL)isLoading
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_LOADING_DID_CHANGED object:[NSNumber numberWithBool:isLoading]];
}
//------------------------------------
// postVideoLoadedChangedNotification
//------------------------------------
- (void)postVideoLoadedChangedNotification:(BOOL)isLoaded
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_LOADED_DID_CHANGED object:[NSNumber numberWithBool:isLoaded]];
}
//------------------------------------
// postVideoPlayChangedNotification
//------------------------------------
- (void)postVideoPlayChangedNotification:(BOOL)isPlaying
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_PLAY_DID_CHANGED object:[NSNumber numberWithBool:isPlaying]];
}
//------------------------------------
// postVideoTimeInitedNotification
//------------------------------------
- (void)postVideoTimeInitedNotification:(NSDictionary*)params
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_TIME_DID_INITED object:params];
}
//------------------------------------
// postVideoTimeChangedNotification
//------------------------------------
- (void)postVideoTimeChangedNotification:(NSDictionary*)params
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_TIME_DID_CHANGED object:params];
}
//------------------------------------
// postVideoRateChangedNotification
//------------------------------------
- (void)postVideoRateChangedNotification:(float)rate
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_RATE_DID_CHANGED object:[NSNumber numberWithFloat:rate]];
}
//------------------------------------
// postVideoStatusChangedNotification
//------------------------------------
- (void)postVideoStatusChangedNotification:(NSDictionary*)params
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_STATUS_DID_CHANGED object:params];
}
//------------------------------------
// postVideoSizeInResizeNotification
//------------------------------------
- (void)postVideoSizeInResizeNotification:(BOOL)inResize
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:VIDEO_NOTIF_SIZE_DID_CHANGED object:[NSNumber numberWithBool:inResize]];
}

//------------------------------------
// postClearNotifications
//------------------------------------
- (void)postClearNotifications
{
	[self postVideoObjectChangedNotification:[NSDictionary dictionaryWithObjectsAndKeys:
													[NSNumber numberWithBool:NO], @"hasVideo" ,
													[NSNumber numberWithBool:NO], @"hasDuration" ,
													[NSNumber numberWithBool:NO], @"isAvailable" ,
													nil
												]
	];
	[self postVideoLoadedChangedNotification:NO];
}
//------------------------------------
// handleVideoPlayDidChanged
//------------------------------------
- (void)handleVideoPlayDidChanged:(NSNotification *)notification
{

	if([self hasMovie] == YES){

		BOOL isPlaying = [[notification object] boolValue];

		// not end frame, set playing status
		if([self isEndFrame] == NO){
			[self setIsCurrentPlaying:isPlaying];
		}

		// set playRate
		float currentRate = [[self movie] rate];
		float playRate = [self playRate];
		if(currentRate > 0.0 && currentRate != playRate){
			[[self movie] setRate:playRate];
		}

		// time status
		if(isPlaying == YES){
			[self startTimeStatusTimer];
		}else{
			[self clearTimeStatusTimer];
			[self updateTimeStatus];
		}

	}


}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:defaultKeyPlayVolume]){
		[self setMovieVolume:[self defaultPlayVolume]];
	}
	else if([keyPath isEqualToString:defaultKeyMovieKeepAspect]){
		[self setMovieAspectRatio:[self defaultMovieKeepAspect]];
	}

}
//------------------------------------
// menuForEvent
//------------------------------------
- (NSMenu*)menuForEvent:(NSEvent *)theEvent
{
	return cmMovieView;
}
//------------------------------------
// acceptsFirstResponder
//------------------------------------
-(BOOL)acceptsFirstResponder
{
    return YES;
}
//------------------------------------
// scrollWheel
//------------------------------------
- (void)scrollWheel:(NSEvent *)theEvent
{
	BOOL isAction = NO;

//	int wheelDelta = [theEvent deltaY];

//		isAction = YES;

	// update time status
	[self updateTimeStatus];

	if(isAction == NO){
		[super scrollWheel:theEvent];
	}

}
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{    

	BOOL isAction = NO;
	NSString *keys = [theEvent charactersIgnoringModifiers];
	int flags = [theEvent modifierFlags];

	if (keys && [keys length] > 0){
		unichar c = [keys characterAtIndex:0];
		// up
		if(c == NSUpArrowFunctionKey){
			isAction = YES;
		}
		// down
		else if(c == NSDownArrowFunctionKey){
			isAction = YES;
		}
		// left
		else if(c == NSLeftArrowFunctionKey){
			// shift
			if(flags & NSShiftKeyMask){
				[self gotoMovieTime:VIDEO_GO_TO_START];
			}
			else{
				[self skipMovieTime:([self defaultSkipInterval] * -1)];
			}
			isAction = YES;
		}
		// right
		else if(c == NSRightArrowFunctionKey){
			// shift
			if(flags & NSShiftKeyMask){
				[self gotoMovieTime:VIDEO_GO_TO_END];
			}
			else{
				[self skipMovieTime:[self defaultSkipInterval]];
			}
			isAction = YES;
		}

	}

	NSString *chars = [theEvent characters];
	if(chars && [chars length] > 0){
		// space key
        if([chars isEqualToString:@" "]){
			[self setMoviePlay:![self isPlaying] isAction:YES];
			isAction = YES;
		}
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}

}
/*
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)event
{
	[NSMenu popUpContextMenu:cmMovieView withEvent:event forView:self];
}
*/
//=======================================================================
// mouse tracing method
//=======================================================================
/*
//------------------------------------
// viewDidMoveToWindow
//------------------------------------
- (void)viewDidMoveToWindow
{
	// create tracking rect
	[self createMouseTracking];
}
//------------------------------------
// setFrame
//------------------------------------
- (void)setFrame:(NSRect)frame
{

	[super setFrame:frame];

	// create tracking rect
	[self createMouseTracking];
}
//------------------------------------
// mouseEntered
//------------------------------------
- (void)mouseEntered:(NSEvent *)theEvent
{
	NSLog(@"mouseEntered");
}
//------------------------------------
// mouseExited
//------------------------------------
- (void)mouseExited:(NSEvent *)theEvent
{
	NSLog(@"mouseExited");
}
*/
//------------------------------------
// mouseMoved
//------------------------------------
- (void)mouseMoved:(NSEvent *)event
{

	if([self isFullScreen] == YES){
		if(abs([event deltaX]) > 2.0 || abs([event deltaY]) > 2.0){
			// post notification
			NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
			[nc postNotificationName:WINDOW_NOTIF_MOUSE_DID_MOVED object:self];
		}
	}

	[super mouseMoved:event];

}
//------------------------------------
// createMouseTracking
//------------------------------------
- (void)createMouseTracking
{

	// remove tracking
	[self removeMouseTracking];

	trackingRect_ = [self addTrackingRect:[self bounds] owner:self userData:NULL assumeInside:NO];

	[[self window] setAcceptsMouseMovedEvents:YES];
	[[self window] makeFirstResponder:self]; 

}
//------------------------------------
// removeMouseTracking
//------------------------------------
- (void)removeMouseTracking
{
	// remove
	[self removeTrackingRect:trackingRect_];
	[[self window] setAcceptsMouseMovedEvents:NO];
}
//------------------------------------
// timeStatusTimer
//------------------------------------
- (void)setTimeStatusTimer:(NSTimer*)timeStatusTimer
{
	[timeStatusTimer retain];
	[timeStatusTimer_ release];
	timeStatusTimer_ = timeStatusTimer;
}
- (void)clearTimeStatusTimer
{
	if([[self timeStatusTimer] isValid] == YES){
		[[self timeStatusTimer] invalidate];
	}
}
- (NSTimer*)timeStatusTimer
{
	return timeStatusTimer_;
}
//------------------------------------
// playRate
//------------------------------------
- (void)setPlayRate:(float)playRate
{
	playRate_ = playRate;
}
- (float)playRate
{
	return playRate_;
}

//------------------------------------
// errorDescription
//------------------------------------
- (void)setErrorDescription:(NSString*)errorDescription
{
	[errorDescription retain];
	[errorDescription_ release];
	errorDescription_ = errorDescription;
}
- (NSString*)errorDescription
{
	return errorDescription_;
}
//------------------------------------
// isCurrentPlaying
//------------------------------------
- (void)setIsCurrentPlaying:(BOOL)isCurrentPlaying
{
	isCurrentPlaying_ = isCurrentPlaying;
}
- (BOOL)isCurrentPlaying
{
	return isCurrentPlaying_;
}
//------------------------------------
// isBeforeProcPlaying
//------------------------------------
- (void)setIsBeforeProcPlaying:(BOOL)isBeforeProcPlaying
{
	isBeforeProcPlaying_ = isBeforeProcPlaying;
}
- (BOOL)isBeforeProcPlaying
{
	return isBeforeProcPlaying_;
}
//------------------------------------
// isAvailable
//------------------------------------
- (void)setIsAvailable:(BOOL)isAvailable
{
	isAvailable_ = isAvailable;
}
- (BOOL)isAvailable
{
	return isAvailable_;
}
//------------------------------------
// isFullScreen
//------------------------------------
- (BOOL)isFullScreen
{
	return [(FullScreenWindow*)[self window] isFullScreen];
}

//------------------------------------
// isPlaying
//------------------------------------
- (BOOL)isPlaying
{
	return ([self hasMovie] == YES) && ([[self movie] rate] != 0);
}
//------------------------------------
// isContinuePlaying
//------------------------------------
- (BOOL)isContinuePlaying
{
	BOOL isPlaying = NO;

	// movie is playing
	if([self isPlaying] == YES){
		isPlaying = YES;
	}

	// repeat and last state is playing
	if( [self defaultPlayRepeat] != VIDEO_PLAY_REPEAT_OFF &&
		[self isCurrentPlaying] == YES
	){
		isPlaying = YES;
	}

	return isPlaying;
}
//------------------------------------
// isEndFrame
//------------------------------------
- (BOOL)isEndFrame
{
	if([self hasMovie] == YES){
		QTTime qtCurrentTime = [[self movie] currentTime];
		QTTime qtDuration    = [[self movie] duration];
		return (QTTimeCompare(qtCurrentTime, qtDuration) == NSOrderedSame);
//		return (QTTimeCompare(qtCurrentTime, qtDuration) == NSOrderedDescending);
//		return (QTTimeCompare(qtCurrentTime, qtDuration) == NSOrderedAscending);
	}else{
		return YES;
	}
}
//------------------------------------
// hasDuration
//------------------------------------
- (BOOL)hasDuration
{
	QTMovie *movie = [self movie];

	if(movie != nil){
		float duration = [movie duration].timeValue / [movie duration].timeScale;
		if(duration > 0.0){
			return YES;
		}else{
			return NO;
		}
	}else{
		return NO;
	}
}
//------------------------------------
// hasMovie
//------------------------------------
- (BOOL)hasMovie
{
	if([self movie] != nil){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// hasImage
//------------------------------------
- (BOOL)hasImage
{
	// get movie size
	NSSize naturalSize = [[[self movie] attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 
	// not create size
	if(naturalSize.width > 0 && naturalSize.height > 0){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// movieCurrentTimeSec
//------------------------------------
- (int)movieCurrentTimeSec
{
	int timeSec = 0;

	QTMovie *movie = [self movie];
	if(movie != nil){
		QTTime time = [movie currentTime];
		timeSec = time.timeValue / time.timeScale;
	}

	return timeSec;

}

//------------------------------------
// movieCurrentFrameImage
//------------------------------------
- (NSImage*)movieCurrentFrameImage
{
	if([self movie] != nil){
		return [[self movie] currentFrameImage];
	}else{
		return nil;
	}
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults removeObserver:self forKeyPath:defaultKeyPlayVolume];
	[defaults removeObserver:self forKeyPath:defaultKeyMovieKeepAspect];

	[errorDescription_ release];
	[timeStatusTimer_ release];

	[super dealloc];
}

@end
