#import "OverlayControllerWindow.h"
#import "UserDefaultsExtension.h"

static const int FADE_TYPE_IN = 0;
static const int FADE_TYPE_OUT = 1;

@implementation OverlayControllerWindow

//------------------------------------
// initWithContentRect
//------------------------------------
- (id)initWithContentRect:(NSRect)contentRect 
				styleMask:(unsigned int)aStyle 
				  backing:(NSBackingStoreType)bufferingType 
					defer:(BOOL)flag {
	
	if (self = [super initWithContentRect:contentRect 
										styleMask:NSBorderlessWindowMask 
										  backing:NSBackingStoreBuffered 
								   defer:NO]) {
		[self setLevel:NSNormalWindowLevel];		
		[self setBackgroundColor:[NSColor clearColor]];
		[self setAlphaValue:0.0];
		[self setOpaque:NO];
		[self setShowsResizeIndicator:NO];
		[self setHasShadow:NO];

		return self;
	}

	return nil;
}
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[super setDelegate:self];

	[self setIsOpenTimer:NO];
	[self setOpenTimer:nil];

	[self setIsCloseTimer:NO];
	[self setCloseTimer:nil];

	[self setIsFullScreen:NO];

	// add self to player window
	[playerWindow addChildWindow:self ordered:NSWindowAbove];
	[self adjustFrame:[self frame] parentRect:[[self parentWindow] frame] isDisplay:NO];

	titleBarHeight_ = [self frame].size.height
					-([[self contentView] frame].size.height + [[self contentView] frame].origin.y);

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
//	[nc addObserver:self selector:@selector(handleParentWindowDidBecomeKey:) name:NSWindowDidBecomeKeyNotification object:[self parentWindow]];
	[nc addObserver:self selector:@selector(handleParentWindowWillClose:) name:NSWindowWillCloseNotification object:[self parentWindow]];
	[nc addObserver:self selector:@selector(handleParentWindowLiveResize:) name:NSWindowDidResizeNotification object:[self parentWindow]];
	[nc addObserver:self selector:@selector(handleWindowFullScreenDidChanged:) name:WINDOW_NOTIF_FULLSCREEN_DID_CHANGED object:nil];
	[nc addObserver:self selector:@selector(handleVideoPlayActionDidChanged:) name:VIDEO_NOTIF_PLAY_ACTION_DID_CHANGED object:nil];

}
//=======================================================================
// action
//=======================================================================
//=======================================================================
// method
//=======================================================================
//------------------------------------
// open
//------------------------------------
- (void)open
{

//	NSLog(@"open isVisible=%d", [self isVisible]);

	// clear timer
	[self clearOpenTimer];
	[self clearCloseTimer];

	[self adjustFrame:[self frame] parentRect:[[self parentWindow] frame] isDisplay:NO];
	
	// force draw for white out frame
	[self display];
	[self makeKeyAndOrderFront:self];

}
//------------------------------------
// openWithFade
//------------------------------------
- (void)openWithFade
{
	if([self alphaValue] < 1.0){
		[self fadeInWindow];
	}
}
//------------------------------------
// close
//------------------------------------
- (void)close
{
//	NSLog(@"close");

	// clear timer
	[self clearOpenTimer];
	[self clearCloseTimer];

	[self setAlphaValue:0.0];

	// parent window is close too becase "NSWindowAbove" mode.
//	[super close];

//	NSLog(@"close isVisible=%d", [self isVisible]);

}
//------------------------------------
// closeWithFade
//------------------------------------
- (void)closeWithFade
{
	// mouse location is in controller
	NSPoint point = [NSEvent mouseLocation];
	if(NSPointInRect(point, [self frame]) == YES){
		return;
	}

	if([self alphaValue] > 0.0){
		[self fadeOutWindow];
	}
}

//------------------------------------
// startOpenTimer
//------------------------------------
- (void)startOpenTimer
{

//	NSLog(@"startOpenTimer");

	// clear timer
	[self clearOpenTimer];

	// set timer
	[self setIsOpenTimer:YES];
	[self setOpenTimer:[NSTimer scheduledTimerWithTimeInterval:[self defaultFloatValue:@"optOverlayControllerOpenInterval"]
							target:self 
							selector:@selector(openWithTimer)
							userInfo:nil
							repeats:NO]
	];
	[[NSRunLoop currentRunLoop] addTimer:[self openTimer] forMode:(NSString*)kCFRunLoopCommonModes];

}
//------------------------------------
// startCloseTimer
//------------------------------------
- (void)startCloseTimer
{

//	NSLog(@"startCloseTimer");

	// clear timer
	[self clearCloseTimer];

	// set timer
	[self setIsCloseTimer:YES];
	[self setCloseTimer:[NSTimer scheduledTimerWithTimeInterval:[self defaultFloatValue:@"optOverlayControllerCloseInterval"]
							target:self 
							selector:@selector(closeWithTimer)
							userInfo:nil
							repeats:NO]
	];
	[[NSRunLoop currentRunLoop] addTimer:[self closeTimer] forMode:(NSString*)kCFRunLoopCommonModes];

}
//------------------------------------
// openWithTimer
//------------------------------------
- (void)openWithTimer
{
	// open timer is enabled
	if([self isOpenTimer] == YES){
		[self openWithFade];			
	}
}
//------------------------------------
// closeWithTimer
//------------------------------------
- (void)closeWithTimer
{
	// close timer is enabled
	if([self isCloseTimer] == YES){
		[self closeWithFade];			
	}
}
//------------------------------------
// fadeInWindow
//------------------------------------
- (void)fadeInWindow
{

	NSMutableDictionary* dict = [NSMutableDictionary dictionary];
	[dict setObject:self forKey:NSViewAnimationTargetKey];
	[dict setObject:NSViewAnimationFadeInEffect forKey:NSViewAnimationEffectKey];

	OverlayAnimation *anim = [[OverlayAnimation alloc]
									initWithViewAnimations:[NSArray arrayWithObject:dict]
									fadeType:FADE_TYPE_IN
									notifNames:[NSArray arrayWithObjects:
													VIDEO_NOTIF_SIZE_DID_CHANGED,
													WINDOW_NOTIF_PLAYER_DID_CLOSED,
													nil
												]
							];
	[anim setDuration:[self defaultFloatValue:@"optOverlayControllerFadeDuration"]];
	[anim setAnimationCurve:NSAnimationEaseIn];
//	[anim setAnimationBlockingMode:NSAnimationBlocking];
	[anim setDelegate:self];

	[anim startAnimation];
	[anim release];

}
//------------------------------------
// fadeOutWindow
//------------------------------------
- (void)fadeOutWindow
{

	NSMutableDictionary* dict = [NSMutableDictionary dictionary];
	[dict setObject:self forKey:NSViewAnimationTargetKey];
	[dict setObject:NSViewAnimationFadeOutEffect forKey:NSViewAnimationEffectKey];
	
	OverlayAnimation *anim = [[OverlayAnimation alloc]
									initWithViewAnimations:[NSArray arrayWithObject:dict]
									fadeType:FADE_TYPE_OUT
									notifNames:[NSArray arrayWithObjects:
													VIDEO_NOTIF_SIZE_DID_CHANGED,
													WINDOW_NOTIF_PLAYER_DID_CLOSED,
													nil
												]
							];
	[anim setDuration:[self defaultFloatValue:@"optOverlayControllerFadeDuration"]];
	[anim setAnimationCurve:NSAnimationEaseIn];
//	[anim setAnimationBlockingMode:NSAnimationBlocking];
	[anim setDelegate:self];

	[anim startAnimation];
	[anim release];

}

//------------------------------------
// animationShouldStart
//------------------------------------
- (BOOL)animationShouldStart:(OverlayAnimation *)animation
{
	if([animation fadeType] == FADE_TYPE_IN){
		[self open];
	}
	return YES;
}
//------------------------------------
// animationDidEnd
//------------------------------------
- (void)animationDidEnd:(OverlayAnimation *)animation
{
	if([animation fadeType] == FADE_TYPE_IN){
		// mouse location is out of controller
		NSPoint point = [NSEvent mouseLocation];
		if(NSPointInRect(point, [self frame]) == NO){
			[self startCloseTimer];
		}
	}

	if([animation fadeType] == FADE_TYPE_OUT){
		[self close];
	}

}
//------------------------------------
// adjustFrame
//------------------------------------
- (void)adjustFrame:(NSRect)childRect parentRect:(NSRect)parentRect isDisplay:(BOOL)isDisplay
{

	childRect.origin.x = parentRect.origin.x;
	childRect.origin.y = parentRect.origin.y + WINDOW_CONTROLLER_BAR_HEIGHT;
	childRect.size.width = parentRect.size.width;

	[self setFrame:childRect display:isDisplay animate:NO];

}
//=======================================================================
// handler
//=======================================================================
//------------------------------------
// handleVideoPlayActionDidChanged
//------------------------------------
- (void)handleVideoPlayActionDidChanged:(NSNotification *)notification
{

	BOOL isPlay = [[notification object] boolValue];

	if(isPlay == NO){
		[self openWithFade];
		[self startCloseTimer];		// for skip
	}else{
//		[self startCloseTimer];
	}

}
//------------------------------------
// handlePlayerWindowDidOpened
//------------------------------------
- (void)handleParentWindowDidBecomeKey:(NSNotification *)notification
{
//	[self orderFront:self];
}
//------------------------------------
// handlePlayerWindowDidOpened
//------------------------------------
- (void)handleParentWindowWillClose:(NSNotification *)notification
{

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_PLAYER_DID_CLOSED object:[NSNumber numberWithBool:YES]];

	// force close
	[self close];			
}

//------------------------------------
// handleParentWindowLiveResize
//------------------------------------
- (void)handleParentWindowLiveResize:(NSNotification *)notification
{

	// force close
	if([self alphaValue] > 0.0){
		[self close];
	}

	[self adjustFrame:[self frame] parentRect:[[self parentWindow] frame] isDisplay:NO];
}
//------------------------------------
// handleWindowFullScreenDidChanged
//------------------------------------
- (void)handleWindowFullScreenDidChanged:(NSNotification *)notification
{

	BOOL isFullScreen = [[notification object] boolValue];
//	NSLog(@"isFullScreen=%d", isFullScreen);

	[self setIsFullScreen:isFullScreen];

/*
	// goto full screen
	if(isFullScreen == YES){
		// clear tracking
		[self removeMouseTracking];
//		[self closeWithFade];		// exec with resize notification
	}
	// exit full screen
	else{
		// start tracking
		[self createMouseTracking];
	}
*/
}
//=======================================================================
// mouse tracking methods
//=======================================================================
//------------------------------------
// mouseEntered
//------------------------------------
- (void)mouseEntered:(NSEvent *)event
{
//	NSLog(@"mouse entered");

	if([[self parentWindow] isKeyWindow] == YES){

		// clear timer
		[self clearCloseTimer];

		// start timer
		[self startOpenTimer];
	}

}
//------------------------------------
// mouseExited
//------------------------------------
- (void)mouseExited:(NSEvent *)event
{
//	NSLog(@"mouse exited");

	// clear timer
	[self clearOpenTimer];

	// start timer
	[self startCloseTimer];

}
/*
//------------------------------------
// mouseMoved
//------------------------------------
- (void)mouseMoved:(NSEvent *)event
{

//	NSLog(@"mouseMoved");
//	if([self isCloseTimer] == YES){
//		[self clearCloseTimer];
//	}

}
*/
//------------------------------------
// createMouseTracking
//------------------------------------
- (void)createMouseTracking
{

//	NSLog(@"createMouseTracking");

	// remove
	[self removeMouseTracking];

	// only normal window mode
//	if([self isFullScreen] == NO){
		NSRect bounds = [[self contentView] bounds];
		NSRect rect = NSMakeRect(
									bounds.origin.x, 
									bounds.origin.y, 
									bounds.size.width, 
									bounds.size.height + titleBarHeight_
								);
//		NSLog(@"%.2f, %.2f, %.2f, %.2f, ", frame.origin.x, frame.origin.y, frame.size.width, frame.size.height);

		trackingRect_ = [[self contentView] addTrackingRect:rect owner:self userData:NULL assumeInside:NO];
//	}

}
//------------------------------------
// removeMouseTracking
//------------------------------------
- (void)removeMouseTracking
{
	// remove
	[[self contentView] removeTrackingRect:trackingRect_];
}
/*
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{
	BOOL isAction = NO;

	NSString *chars = [theEvent charactersIgnoringModifiers];
//	NSLog(@"chars=%@", chars);
	if(chars && [chars length] > 0){
		// cmd + w
		if([[chars lowercaseString] isEqualToString:@"w"]){
			if([theEvent modifierFlags] & NSCommandKeyMask){
				[[self parentWindow] close];
				isAction = YES;
			}
		}
	}
	if(isAction == NO){
		[[self parentWindow] keyDown:theEvent];
	}
}
*/
//------------------------------------
// canBecomeKeyWindow
//------------------------------------
- (BOOL)canBecomeKeyWindow
{
	// if set "YES", get response in fullscreen mode 
	return NO;
}
//------------------------------------
// windowDidMove
//------------------------------------
- (void)windowDidMove:(NSNotification *)aNotofication
{
	[self createMouseTracking];
}
//------------------------------------
// windowDidResize
//------------------------------------
- (void)windowDidResize:(NSNotification *)aNotofication
{
	[self createMouseTracking];
}
//------------------------------------
// openTimer
//------------------------------------
- (void)setOpenTimer:(NSTimer*)openTimer
{
	[openTimer retain];
	[openTimer_ release];
	openTimer_ = openTimer;
}
- (void)clearOpenTimer
{
	[self setIsOpenTimer:NO];
	if([[self openTimer] isValid] == YES){
		[[self openTimer] invalidate];
//		NSLog(@"clearOpenTimer");
	}
}
- (NSTimer*)openTimer
{
	return openTimer_;
}
//------------------------------------
// isOpenTimer
//------------------------------------
- (void)setIsOpenTimer:(BOOL)isOpenTimer
{
	isOpenTimer_ = isOpenTimer;
}
- (BOOL)isOpenTimer
{
    return isOpenTimer_;
}
//------------------------------------
// closeTimer
//------------------------------------
- (void)setCloseTimer:(NSTimer*)closeTimer
{
	[closeTimer retain];
	[closeTimer_ release];
	closeTimer_ = closeTimer;
}
- (void)clearCloseTimer
{
	[self setIsCloseTimer:NO];
	if([[self closeTimer] isValid] == YES){
		[[self closeTimer] invalidate];
//		NSLog(@"clearCloseTimer");
	}
}
- (NSTimer*)closeTimer
{
	return closeTimer_;
}
//------------------------------------
// isCloseTimer
//------------------------------------
- (void)setIsCloseTimer:(BOOL)isCloseTimer
{
	isCloseTimer_ = isCloseTimer;
}
- (BOOL)isCloseTimer
{
    return isCloseTimer_;
}
//------------------------------------
// isFullScreen
//------------------------------------
- (void)setIsFullScreen:(BOOL)isFullScreen
{
	isFullScreen_ = isFullScreen;
}
- (BOOL)isFullScreen
{
    return isFullScreen_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[openTimer_ release];
	[closeTimer_ release];

	[super dealloc];
}

@end
