#import "PreviewImageView.h"
#import "ViewMain.h"
#import "CellAttributeExtension.h"

@implementation PreviewImageView

//------------------------------------
// init
//------------------------------------
- (id)init
{
    if (self = [super init])
	{
		// init code
		[self setIsExist:NO];
		[self setUniqueId:@""];
     }
    return self;
}
//------------------------------------
// drawRect
//------------------------------------
- (void)drawRect:(NSRect)rect
{

	// draw background
	[[NSColor blackColor] set];
	NSRectFill([self bounds]);

    NSPoint	imagePoint;
    imagePoint.x = [self bounds].origin.x;
    imagePoint.y = [self bounds].origin.y;

    // draw image
    if([self image]){

		NSImage *image = [[self image] copy];
 
		// set image size
		[image setSize:[self setCellImageSize:[image size]
									frameSize:[self bounds].size
									paddingSize:NSMakeSize(2,2)
						]
		];
		[image setScalesWhenResized:YES];

		// set image rect
		NSRect imageRect = [self setCellImageRect:[image size]
										frameRect:[self bounds]
							];

		imagePoint.x += imageRect.origin.x;
		imagePoint.y += imageRect.origin.y;

		// set image round rect

		NSRect imageRoundRect = [self setCellImageRoundRect:[image size]
													frameRect:[self bounds]
								];

		// draw shadow rect
/*
		NSRect shadowRect = imageRoundRect;
		shadowRect.origin.x += 5;
		shadowRect.origin.y += 5;
		NSBezierPath *shadowPath = [self setRoundPathForRect:shadowRect radius:5];
		[[NSColor blackColor] set];
		[shadowPath fill];
*/
		// draw image rect
		NSBezierPath *imagePath = [self setRoundPathForRect:imageRoundRect radius:5];
		[NSGraphicsContext saveGraphicsState];

		[imagePath addClip];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver];

		[NSGraphicsContext restoreGraphicsState];

		[image release];

		//
		// draw label
		//
		if([self isExist] == NO){
			// now origin.y is buggy
			NSRect imageBounds;
			imageBounds.origin.x = rect.origin.x + imageRect.origin.x;
			imageBounds.origin.y = rect.origin.y + imageRect.origin.y;
//			if([controlView isFlipped]) {
//				imageBounds.origin.y += (imageRect.size.height - imageRect.origin.y);
//			}
			imageBounds.size = imageRect.size;

			// draw label
			NSString *labelTitle = LABEL_NAME_NOT_FOUND;
			if(![labelTitle isEqualToString:@""]){
				NSSize stringSize = [labelTitle sizeWithAttributes:nil];
				if( imageBounds.size.width > stringSize.width + 6 &&
					imageBounds.size.height > stringSize.height + 4
				){
					// draw label
					[self drawLabelAndString:imageBounds
									withString:labelTitle
									fontSize:LABEL_STATUS_FSIZE
									fontColor:[NSColor whiteColor]
									labelColor:[self getLabelColor:LABEL_COLOR_NOT_FOUND]
									align:CELL_ALIGN_RIGHT
									valign:CELL_VALIGN_BOTTOM
									hPadding:LABEL_STATUS_HPADDING
									vPadding:LABEL_STATUS_VPADDING
									radius:LABEL_STATUS_RADIUS
					];
				}
			}
		}

    }

}

//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)event
{
	// double clicked
	if([event type] == NSLeftMouseDown){
		if([event clickCount] == 2){
			[viewMain playItem:nil];
		}
	}
}
//------------------------------------
// scrollWheel
//------------------------------------
- (void)scrollWheel:(NSEvent *)theEvent
{
	int wheelDelta;

	wheelDelta = [theEvent deltaY];

	if(wheelDelta <= 0){
		[filelistArrayController selectNext:nil];
	}else{
		[filelistArrayController selectPrevious:nil];
	}
}
//------------------------------------
// isExist
//------------------------------------
- (void)setIsExist:(BOOL)isExist
{
	isExist_ = isExist;
}
- (BOOL)isExist
{
	return isExist_;
}
//------------------------------------
// uniqueId
//------------------------------------
- (void)setUniqueId:(NSString*)uniqueId
{
	[uniqueId retain];
    [uniqueId_ release];
    uniqueId_ = uniqueId;

}
- (NSString*)uniqueId
{
	return uniqueId_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[uniqueId_ release];
    [super dealloc];
}

@end
