#import "TBArrayController.h"
#import "ConvertExtension.h"
#import "USerDefaultsExtension.h"

@implementation TBArrayController
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

}

//--------------------------------------
// getMaxIntValueFromEntityWithColumn
//--------------------------------------
- (int)getMaxIntValueFromEntityWithColumn:(NSString*)entityName column:(NSString*)column
{

	NSArray *array = [self getAllObjects:entityName];
	return [self getMaxIntValueFromArrayWithColumn:array column:column];

}
//--------------------------------------
// getMaxIntValueFromArrayWithColumn
//--------------------------------------
- (int)getMaxIntValueFromArrayWithColumn:(NSArray*)array column:(NSString*)column
{

	int value = 0;

	if([array count] > 0){
		NSString *keyPath = [NSString stringWithFormat: @"@max.%@", column];
		value = [[array valueForKeyPath:keyPath] intValue];
	}

	return value;

}


//------------------------------------
// getMaxIndexFromChildItems
//------------------------------------	  
- (int)getMaxIndexFromChildItems:(NSManagedObject*)item
{

	NSArray *childItems = [self getChildObjects:item];
	return [self getMaxIntValueFromArrayWithColumn:childItems column:@"index"];

}
//------------------------------------
// getManagedObjectContext
//------------------------------------
- (NSManagedObjectContext*)getManagedObjectContext:(NSString*)entityName
{

	NSManagedObjectContext *context;

	if([entityName isEqualToString:@"filelist"]){
		context = [filelistArrayController managedObjectContext];
	}
	else if([entityName isEqualToString:@"applist"]){
		context = [applistArrayController managedObjectContext];
	}
	else if([entityName isEqualToString:@"playlist"]){
		context = [playlistTreeController managedObjectContext];
	}
	return context;
}
//------------------------------------
// getSelectedObject
//------------------------------------
- (NSManagedObject*)getSelectedObject:(NSString*)entityName
{

	NSManagedObject *object = nil;

 	NSArray *fetchArray = [self getSelectedObjects:entityName];

	if([fetchArray count] > 0){
		object = [fetchArray objectAtIndex:0];
	}

	return object;
}
//------------------------------------
// getSelectedObjects
//------------------------------------
- (NSArray*)getSelectedObjects:(NSString*)entityName
{

 	NSArray *fetchArray = nil;

	if([entityName isEqualToString:@"filelist"]){
		fetchArray = [filelistArrayController selectedObjects];
	}
	if([entityName isEqualToString:@"playlist"]){
		fetchArray = [playlistTreeController selectedObjects];
	}

	return fetchArray;
}
//------------------------------------
// getAllObjects
//------------------------------------
- (NSArray*)getAllObjects:(NSString*)entityName
{

 	NSArray *fetchedObjects = [[[NSArray alloc] init] autorelease];
	NSFetchRequest *request;

	// set entity
	NSEntityDescription *entity = [NSEntityDescription entityForName:entityName
				inManagedObjectContext:[MacVideoTunes_AppDelegate managedObjectContext]];

	// set fetch request
	request = [[[NSFetchRequest alloc] init] autorelease];
	[request setEntity:entity];

	// execute fetch
	fetchedObjects = [[MacVideoTunes_AppDelegate managedObjectContext] 
						executeFetchRequest:request error:nil];

	return fetchedObjects;
}
//------------------------------------
// getChildObjects
//------------------------------------
- (NSArray*)getChildObjects:(NSManagedObject*)parent
{

	NSPredicate* pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"parent==%@", parent];

	return [self getObjectsWithPred:@"playlist" pred:pred];

}
//------------------------------------
// getObjectsWithPred
//------------------------------------
- (NSArray*)getObjectsWithPred:(NSString*)entityName pred:(NSPredicate*)pred
{

 	NSArray *fetchedObjects = [[[NSArray alloc] init] autorelease];
	NSFetchRequest *request;

	// set entity
	NSEntityDescription *entity = [NSEntityDescription entityForName:entityName
				inManagedObjectContext:[MacVideoTunes_AppDelegate managedObjectContext]];

	// set fetch request
	request = [[[NSFetchRequest alloc] init] autorelease];
	[request setEntity:entity];
	if(pred){
		[request setPredicate:pred];
	}

	// execute fetch
	fetchedObjects = [[MacVideoTunes_AppDelegate managedObjectContext] 
							executeFetchRequest:request error:nil];

	return fetchedObjects;
}

//------------------------------------
// removeAllObjects
//------------------------------------
- (void)removeAllObjects:(NSString*)entityName
{

	// get objects
	NSArray *fetchedArray = [self getAllObjects:entityName];

	// remove objects
	[self removeObjectsWithArray:fetchedArray];

}
//------------------------------------
// removeObjectsWithArray
//------------------------------------
- (void)removeObjectsWithArray:(NSArray*)array
{

	NSEnumerator *enumArray = [array objectEnumerator];
	id record;

	while (record = [enumArray nextObject]) {
		[self removeObject:record];
	}

}
//------------------------------------
// removeObject
//------------------------------------
- (void)removeObject:(id)record
{

	[[MacVideoTunes_AppDelegate managedObjectContext] deleteObject:record];

}
//------------------------------------
// insertPlaylist
//------------------------------------	  
- (NSManagedObject *)insertPlaylist:(int)index
					title:(NSString*)title
					parentItem:(id)parentItem
					isFolder:(BOOL)isFolder
{

	// get uniqueNo
	int uniqueNo = [self defaultIntValue:@"optUniqueNoForPlist"];

	// get max uniqueNo
	if(uniqueNo <= 0){
		uniqueNo = [self getMaxIntValueFromEntityWithColumn:@"playlist" column:@"uniqueId"];
	}

	uniqueNo++;

	//---------------------------
	//	insert record 
	//---------------------------
	NSEntityDescription *entItem = [NSEntityDescription 
									entityForName:@"playlist"
									inManagedObjectContext:[MacVideoTunes_AppDelegate managedObjectContext]];

	NSManagedObject *mo = [[[NSManagedObject alloc]
							initWithEntity:entItem
							insertIntoManagedObjectContext:[MacVideoTunes_AppDelegate managedObjectContext]]
							autorelease];

	[mo setValue:[NSNumber numberWithBool:isFolder] forKey:@"isFolder"];
	[mo setValue:[NSNumber numberWithBool:NO] forKey:@"isExpand"];
	[mo setValue:title forKey:@"title"];
	[mo setValue:[self convertToZeroFormat:uniqueNo] forKey:@"uniqueId"];
	[mo setValue:[NSNumber numberWithInt:0] forKey:@"groupNo"];
	[mo setValue:[NSNumber numberWithInt:index] forKey:@"index"];
	[mo setValue:[NSNumber numberWithInt:0] forKey:@"labelColorNo"];

	// set default
	[self setDefaultIntValue:uniqueNo key:@"optUniqueNoForPlist"];

	return mo;

}
//------------------------------------
// addItemToParent
//------------------------------------	  
- (void)addItemToParent:(NSManagedObject *)targetItem item:(NSManagedObject *)item maxNum:(BOOL)maxNum
{

	// get parent item
	NSManagedObject *parentItem = [self getUpdateTargetItem:targetItem];

	// update relation
//	if(parentItem){
//		[[parentItem mutableSetValueForKey:@"children"] addObject:item];
		[item setValue:parentItem forKey:@"parent"];
//	}

	// set max index
	if(maxNum == YES){
		int index = [self getMaxIndexFromChildItems:parentItem];
		index++;
		[item setValue:[NSNumber numberWithInt:index] forKey:@"index"]; 
	}

}
//------------------------------------
// getUpdateTargetItem
//------------------------------------	  
- (NSManagedObject*)getUpdateTargetItem:(NSManagedObject*)item
{

	NSManagedObject *targetItem = nil;

	if(item){
		// folder
		if([[item valueForKey:@"isFolder"] boolValue] == YES){
			targetItem = item;
		}
		// file -> parentItem
		else{
			NSManagedObject *parentItem = [item valueForKey:@"parent"];
			// folder
			if(parentItem && [[parentItem valueForKey:@"isFolder"] boolValue] == YES){
				targetItem = parentItem;
			}
		}
	}

	return targetItem;

}
//------------------------------------
// getTreeItems
//------------------------------------	  
- (NSMutableArray*)getTreeItems:(NSManagedObject*)item
{

	NSMutableArray *outArray = [[[NSMutableArray alloc] init] autorelease];

	// parent is folder -> get child
	if([[item valueForKey:@"isFolder"] boolValue] == YES){

		NSEnumerator *enumerator = [[item mutableSetValueForKey:@"children"] objectEnumerator];
		NSManagedObject *childItem;
		while(childItem = [enumerator nextObject]) {

			// add object from children
			if([[childItem valueForKey:@"isFolder"] boolValue] == YES){
				[outArray addObjectsFromArray:[self getTreeItems:childItem]];
			}else{
				[outArray addObject:childItem];
			}

		}
	}
	// parent is not folder
	else{
		[outArray addObject:item];
	}

	return outArray;

}
//------------------------------------
// removeTreeItems
//------------------------------------	  
- (void)removeTreeItems:(NSManagedObject*)item
{

	// item is folder -> remove child
	if([[item valueForKey:@"isFolder"] boolValue] == YES){

		NSEnumerator *enumerator = [[item mutableSetValueForKey:@"children"] objectEnumerator];
		id childItem;
		while(childItem = [enumerator nextObject]) {

			if([[childItem valueForKey:@"isFolder"] boolValue] == YES){
				// remove child
				[self removeTreeItems:childItem];
			}
			// remove childItem
			[self removeObject:childItem];
		}
	}

	//remove item
	[self removeObject:item];

}
//-------------------------------------------
// resetItemIndex
//-------------------------------------------
- (void)resetItemIndex:(NSManagedObject*)insItem
			parentItem:(NSManagedObject*)parentItem
			targetItem:(NSManagedObject*)targetItem
			insIndex:(int)insIndex
{

	int index = -1;

//	NSLog(@"insIndex=%d", insIndex);
//	NSLog(@"orgIndex=%d", [[insItem valueForKey:@"index"] intValue]);

	// same parent & drop down
	if(parentItem == targetItem){
//		NSLog(@"same parent!!");
		if(insIndex > [[insItem valueForKey:@"index"] intValue]){
			insIndex--;
		}
	}

	id insParent = [insItem valueForKey:@"parent"];
	NSArray *array = [self getChildObjects:insParent];
 
	NSSortDescriptor *descriptor=[[[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES] autorelease];
	array = [array sortedArrayUsingDescriptors:[NSArray arrayWithObject:descriptor]];

	NSEnumerator *enumObject = [array objectEnumerator];
	id item;
	while (item = [enumObject nextObject]){

		if(item == insItem){
			[item setValue:[NSNumber numberWithInt:insIndex] forKey:@"index"]; 
			continue;
		}
		index++;
		// skip count index
		if(index == insIndex){
			index++;	
		}
		[item setValue:[NSNumber numberWithInt:index] forKey:@"index"]; 
	}

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
    [super dealloc];
}

@end
