#import "ThumbsExtension.h"
#import "FileAttributeExtension.h"
#import "ImageAttributeExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation NSObject(thumbsExtension_)

//------------------------------------
// getThumbsFolderPath
//------------------------------------
- (NSString*)getThumbsFolderPath
{
	NSString* filePath = [self getAppSupportFolderPath];

	if([filePath isEqualToString:@""]){
		return @"";
	}else{
		return [NSString stringWithFormat: @"%@/%@", filePath, @"thumbs"];
	}

}
//------------------------------------
// getThumbsImagePath
//------------------------------------
- (NSString*)getThumbsImagePath:(NSString*)uniqueId
{
	if([uniqueId isEqualToString:@""]){
		return @"";
	}

	NSString* filePath = [self getThumbsFolderPath];

	if([filePath isEqualToString:@""]){
		return @"";
	}else{
		return [NSString stringWithFormat: @"%@/%@.png", filePath, uniqueId];
	}
}
//------------------------------------
// getThumbsImage
//------------------------------------
- (NSImage*)getThumbsImage:(NSString*)filePath
				thumbsPath:(NSString*)thumbsPath
				isSound:(BOOL)isSound
{

	NSString *imagePath = @"";
//	BOOL isExist = NO;

	// thumbnail is exist
	if([self checkFileExist:thumbsPath] == FILE_EXIST_FILE){
		imagePath = thumbsPath;
//		isExist = YES;
	}else{
		// sound 
		if(isSound == YES){
			imagePath = [[NSBundle mainBundle] pathForResource:@"image_sound" ofType:@"png"];
		}else{
			imagePath = [[NSBundle mainBundle] pathForResource:@"image_noimage" ofType:@"png"];
		}
	}

	NSImage *image = [[[NSImage alloc] initByReferencingFile:imagePath] autorelease];	

//	if(isExist == YES && [self checkFileExist:filePath] != FILE_EXIST_FILE){
//		image = [self createGrayScaleImage:image];
//	}

	return image;
	
}
//------------------------------------
// checkAndCreateThumbsFolder
//------------------------------------
- (BOOL)checkAndCreateThumbsFolder:(BOOL)showAlert
{

	// Application support folder is not availble
	if([self checkFileIsCollect:[self getAppSupportFolderPath] fileType:@"folder"] != FILE_COLLECT_MATCH){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Application support folder is not available."
						infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getAppSupportFolderPath]]
						btnList:@"Cancel"
			];
		}
		return NO;

	}

	// Thumbs folder is not availble
	if([self checkAndCreateFolder:[self getThumbsFolderPath]] != FOLDER_CHECK_SUCCESS){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Thumbs folder is not available."
						infoText:[NSString stringWithFormat:@"Please check \"%@\"", [self getThumbsFolderPath]]
						btnList:@"Cancel"
			];
		}
		return NO;
	}
	
	return YES;

}
//------------------------------------
// createThumbsImage
//------------------------------------
- (BOOL)createThumbsImage:(NSImage*)image
					uniqueId:(NSString*)uniqueId
					isCheckFolder:(BOOL)isCheckFolder
					showAlert:(BOOL)showAlert
{

	// image is null
	if(!image){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"Image is null."
						infoText:@""
						btnList:@"Cancel"
			];
		}
		return NO;
	}


	// uniqueId is null
	if(!uniqueId || [uniqueId isEqualToString:@""]){
		if(showAlert == YES){
			[self displayMessage:@"alert"
						messageText:@"UniqueId is null."
						infoText:@""
						btnList:@"Cancel"
			];
		}
		return NO;
	}

	// check and create thumbs folder
	if(isCheckFolder == YES){
		if([self checkAndCreateThumbsFolder:showAlert] == NO){
			return NO;
		}
	}

	NSSize thumbsSize = [self defaultThumbsSize];
	NSString *thumbsPath = [self getThumbsImagePath:uniqueId];

	if(!thumbsPath || [thumbsPath isEqualToString:@""]){
		return NO;
	}

	// larger than thumbsSize
	if([image size].width > thumbsSize.width || [image size].height > thumbsSize.height){
		[image setSize:[self setScaledImageSize:[image size] frameSize:thumbsSize]];
		[image setScalesWhenResized:YES];
	}

	return [self saveImageAsPng:image filePath:thumbsPath Interlaced:YES];

}
//------------------------------------
// removeThumbsImage
//------------------------------------
- (BOOL)removeThumbsImage:(NSString*)uniqueId
{

	NSFileManager *myFile = [NSFileManager defaultManager];

	// uniqueId is null
	if(!uniqueId || [uniqueId isEqualToString:@""]){
		return NO;
	}

	// create thumbs path
	NSString *filePath = [self getThumbsImagePath:uniqueId];

	// no path
	if([filePath isEqualToString:@""]){
		return NO;
	}

	BOOL isDir;
	// already not found
	if(![myFile fileExistsAtPath:filePath isDirectory:&isDir]){
		return YES;
	}
	// is directory
	if(isDir == YES){
		return NO;
	}

//	NSLog(@"filePath=%@", filePath);

	// remove file
	return [myFile removeFileAtPath:filePath handler:self];

}

@end