#import "ThumbsMatrix.h"
#import "ThumbsMatrixCell.h"
#import "ViewMain.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "FileAttributeExtension.h"
#import "CellAttributeExtension.h"

static int	CELL_PADDING_WIDTH = 5;
static int	CELL_PADDING_HEIGHT = 10;

@implementation ThumbsMatrix

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	
	// init
	[self setStartIndex:0];
	[self setEndIndex:0];
	[self setSelectedIndex:0];
	[self setPointedIndex:0];
	isChangedArray_ = NO;
	isChangedIndex_ = NO;
	isPointedCell_ = NO;

	// set cell size
	[self setCellSize:[self calcCellSize:[[self enclosingScrollView] contentSize] cols:[self maxCols]]];
	[self setIntercellSpacing:NSMakeSize(CELL_PADDING_WIDTH, CELL_PADDING_HEIGHT)];

	// set observer
	[filelistArrayController addObserver:self forKeyPath:@"arrangedObjects" options:0 context:nil];
	[filelistArrayController addObserver:self forKeyPath:@"selection" options:0 context:nil];

	[txtPageResult setStringValue:@"Results"];
	[self changePageButtonEnable];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:@"optTabViewMainIndex" options:0 context:nil];
	[defaults addObserver:self forKeyPath:@"optMaxResultsType" options:0 context:nil];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(frameDidResize:) name:NSViewFrameDidChangeNotification object:[self enclosingScrollView]];
	[nc addObserver:self selector:@selector(itemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED object:nil];

	// set registerForDraggedTypes
	[self registerForDraggedTypes:
				[NSArray arrayWithObjects:
						FilelistPboardType,
						NSFilenamesPboardType,
						nil]
				];

}
//------------------------------------
// changeMatrix
//------------------------------------
- (IBAction)changeMatrix:(id)sender
{

//	NSLog(@"changeMatrix");

//	NSLog(@"isPointedCell_=%d", isPointedCell_);

	// pointed index
	if(isPointedCell_ == YES){
		[self setSelectedIndex:[self pointedIndex]];
	}else{
		int selectedIndex = 0;
		NSIndexSet *selectionIndexes = [filelistArrayController selectionIndexes];
		if([selectionIndexes count] > 0){
			selectedIndex = [selectionIndexes firstIndex];
		}
		[self setSelectedIndex:selectedIndex];
	}

	// force set pointed index
	if(isPointedCell_ == NO){
		[self setPointedIndex:[self selectedIndex]];
	}

//	NSLog(@"selectedIndex=%d", [self selectedIndex]);
//	NSLog(@"pointedIndex=%d", [self pointedIndex]);

	isPointedCell_ = NO;

	// check spage is changed
	NSDictionary *params = [self checkIsChangedPage];
	BOOL isChangedPage = [[params valueForKey:@"isChangedPage"] boolValue];
	int startIndex = [[params valueForKey:@"startIndex"] intValue];
	int endIndex = [[params valueForKey:@"endIndex"] intValue];

//	NSLog(@"startIndex=%d -> endIndex=%d", startIndex, endIndex);
//	NSLog(@"selectedIndex=%d", [self selectedIndex]);
//	NSLog(@"isChangedPage=%d", isChangedPage);

	// if changed array or startIndex, create maxtirx
	if(isChangedArray_ == YES || isChangedPage == YES){
//		[self clearMatrix];		// not used
		[self createMatrix:startIndex endIndex:endIndex];
		isChangedArray_ = NO;
	}

	[self updateSelectedCell];

	[self setPageResultStatus];
	[self changePageButtonEnable];

	[self setNeedsDisplay:YES];

}
//------------------------------------
// changeCellSize
//------------------------------------
- (IBAction)changeCellSize:(id)sender
{

	// set cell size
	[self setCellSize:[self calcCellSize:[[self enclosingScrollView] contentSize] cols:[self maxCols]]];

	// retile
	[self retileMatrix];
	[self updateSelectedCell];

}
//------------------------------------
// changeMatrixCols
//------------------------------------
- (IBAction)changeMatrixCols:(id)sender
{

	// set cell size
	[self setCellSize:[self calcCellSize:[[self enclosingScrollView] contentSize] cols:[self maxCols]]];

	// retile
	[self retileMatrix];
	[self updateSelectedCell];

}
//------------------------------------
// changeMatrixPage
//------------------------------------
- (IBAction)changeMatrixPage:(id)sender
{

	int tag = [sender tag];

	int startIndex = [self startIndex];
	int lastIndex = [self lastIndex];
	int maxResults = [self maxResults];

	// prev
	if(tag == 0){
		if((startIndex - maxResults) >= 0){
			startIndex -= maxResults;
		}else{
			startIndex = 0;
		}
	}
	// next
	else{
		if((startIndex + maxResults) <= lastIndex){
			startIndex += maxResults;
		}
	}

	// change selection
	[filelistArrayController setSelectionIndex:startIndex];

}
//------------------------------------
// moveMatrixPage
//------------------------------------
- (IBAction)moveMatrixPage:(id)sender
{

	int pageIndex = [[sender representedObject] intValue];

	int maxResults = [self maxResults];
//	int startIndex = ((pageIndex - 1) * maxResults) + 1;
	int startIndex = ((pageIndex - 1) * maxResults);

	// change selection
	[filelistArrayController setSelectionIndex:startIndex];

}
//------------------------------------
// selectAll
//------------------------------------
- (IBAction)selectAll:(id)sender
{
	NSMutableIndexSet *selectionIndexes = [NSMutableIndexSet indexSet];

	int startIndex = [self startIndex];
	int endIndex = [self endIndex];

	[selectionIndexes addIndexesInRange:NSMakeRange(startIndex, endIndex - startIndex + 1)];

	[filelistArrayController setSelectionIndexes:selectionIndexes];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// changePageButtonEnable
//------------------------------------
- (void)changePageButtonEnable
{

	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	// list
	if(index == 0){
		[btnPagePrev setEnabled:NO];
		[btnPageNext setEnabled:NO];
	}
	// grid
	else if(index == 1){

		int startIndex = [self startIndex];
		int endIndex =  [self endIndex];
		int lastIndex =  [self lastIndex];
//		int totalResults =  [self totalResults];
//		int maxResults = [self maxResults];

		// prev
		if(startIndex > 1){
			[btnPagePrev setEnabled:YES];
		}else{
			[btnPagePrev setEnabled:NO];
		}
		// next
//		if(startIndex + maxResults <= totalResults){
		if(endIndex < lastIndex){
			[btnPageNext setEnabled:YES];
		}else{
			[btnPageNext setEnabled:NO];
		}
	}
}
//------------------------------------
// checkIsChangedPage
//------------------------------------
- (NSDictionary*)checkIsChangedPage
{

	int startIndex = 0;
	int endIndex = 0;
	int selectedIndex = 0;
	int maxResults = 0;
	BOOL isChangedPage = NO;

	int totalResults = [self totalResults];

	if(totalResults > 0){

		// selectedIndex
		selectedIndex = [self selectedIndex];
		// no selection
		if(selectedIndex < 0){
			selectedIndex = 0;
		}

		// maxResults
		maxResults = [self maxResults];

		if(maxResults > 0){
			startIndex = (selectedIndex / maxResults) * maxResults;
		}else{
			startIndex = 0;
		}

		// endIndex
		endIndex = startIndex + maxResults - 1;
	}

	if(startIndex != [self startIndex]){
		isChangedPage = YES;
	}

	return [NSDictionary dictionaryWithObjectsAndKeys:
						[NSNumber numberWithInt:startIndex], @"startIndex",
						[NSNumber numberWithInt:endIndex], @"endIndex",
						[NSNumber numberWithBool:isChangedPage], @"isChangedPage",
						nil
			];
}
//------------------------------------
// setPageResultStatus
//------------------------------------
- (void)setPageResultStatus
{

	int startIndex = [self startIndex] + 1;
	int endIndex = [self endIndex] + 1;
	int totalResults =  [self totalResults];

	// result string
	NSString *resultString = [self convertToResultString:startIndex endIndex:endIndex totalResults:totalResults];
	[txtPageResult setStringValue:resultString];

}

//------------------------------------
// createMatrix
//------------------------------------
- (void)createMatrix:(int)startIndex endIndex:(int)endIndex
{

	int lastIndex = [self lastIndex];

	// over last index
	if(startIndex > lastIndex){
		startIndex = 0;
	}
	// over last index
	if(endIndex > lastIndex){
		endIndex = lastIndex;
	}
	// minus value
//	if(endIndex < 0){
//		endIndex = 0;
//	}

	// data count for matrix
	int dataCount = endIndex - startIndex + 1;
	// minus value
	if(dataCount < 0){
		dataCount = 0;
	}

	// set max rows & cols
	int maxCols = [self maxCols];
	int maxRows = 1;
	int col = 0;
	int row = 0;
	int itemIndex = -1;
	int i;

	// null matrix
	if(dataCount <= 0){
		maxCols = 0;
		maxRows = 0;
	}
	// add row for overflow
	else if(dataCount > maxCols){
		maxRows = dataCount / maxCols;
		if((dataCount % maxCols) != 0){
			maxRows++;
		}
	}

	// set startIndex
	[self setStartIndex:startIndex];
	[self setEndIndex:endIndex];

	// reset matrix cells
	[self renewRows:maxRows columns:maxCols];

	id record;

	for (i = startIndex; i <= endIndex; i++){

		record = [[filelistArrayController arrangedObjects] objectAtIndex:i];

		BOOL isExist = NO;
		// isExist
		if([self checkFileExist:[record valueForKey: @"filePath"]] == FILE_EXIST_FILE){
			isExist = YES;
		}

		if(col > maxCols - 1){
			row++;
			col = 0;
		}

		// create image cell
//		itemIndex++;
		itemIndex = i;
		ThumbsMatrixCell *itemCell = [[ThumbsMatrixCell alloc] initWithItemObject:record
																	itemIndex:itemIndex
																	isEnabled:YES
																	isExist:isExist
									];

		//
		// tool tip
		//
		NSString *playTimeStr = @"-";
		NSString *fileSizeStr = @"-";
		NSString *imageSizeStr = @"-";

		// title
		NSString *title = [record valueForKey: @"title"];

		// playTime
		int playTime = [[record valueForKey: @"playTime"] intValue];
		// convert play time
		if(playTime > 0){
			playTimeStr = [self convertTimeToString:playTime];
		}

		// fileSize
		int fileSize = [[record valueForKey: @"fileSize"] intValue];
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}
		// imageSize
		if([record valueForKey: @"imageWidth"] && [record valueForKey: @"imageHeight"]){
			int imageWidth = [[record valueForKey: @"imageWidth"] intValue];
			int imageHeight = [[record valueForKey: @"imageHeight"] intValue];
			if(imageWidth > 0 && imageHeight > 0){
				imageSizeStr = [NSString stringWithFormat:@"%d x %d", imageWidth, imageHeight];
			}
		}

		// filePath
		NSString *filePath = [record valueForKey: @"filePath"];

		NSString *toolChip = [NSString stringWithFormat:@"%@\n%@\n%@\n%@\n%@",
									title,
									[NSString stringWithFormat:@"Time: %@", playTimeStr],
									[NSString stringWithFormat:@"Video: %@", imageSizeStr],
									[NSString stringWithFormat:@"Size: %@", fileSizeStr],
									[NSString stringWithFormat:@"Path: %@", filePath]
							];

		[self setToolTip:toolChip forCell:itemCell];

		// put cell
		[self putCell:itemCell atRow:row column:col];
		[itemCell release];
		col++;

	}

	// set null cell for rest
	// last row
	for (i = col; i < maxCols; i++){

		// already null cell
		if([(ThumbsMatrixCell*)[self cellAtRow:row column:i] isEnabled] == NO){
			continue;
		}

		// create null cell
		ThumbsMatrixCell *itemCell = [[ThumbsMatrixCell alloc] initWithItemObject:nil
																	itemIndex:-1
																	isEnabled:NO
																	isExist:NO
									];
		[self setToolTip:nil forCell:itemCell];
		[self putCell:itemCell atRow:row column:i];
		[itemCell release];
	}

	[self sizeToCells];

}
//------------------------------------
// retileMatrix
//------------------------------------
- (void)retileMatrix
{

	// data count for matrix
	int count = [self endIndex] - [self startIndex] + 1;

	// minus value
	if(count < 0){
		count = 0;
	}

	int cols = [self maxCols];
	int rows = 1;

	// same cols 
	if(cols == [self numberOfColumns]){
		[self sizeToCells];
		[self setNeedsDisplay:YES];
		return;
	}

	if(count > cols){
		rows = count / cols;
		if(count % cols > 0){
			rows++;
		}
	}
	int maxCells = cols * rows;
	[self renewRows:rows columns:cols];

	int restCells = maxCells - count;
//	NSLog(@"cols=%d, rows=%d", cols, rows);
//	NSLog(@"restCells=%d", restCells);

	// null matrix cell
	if(restCells > 0){
		int i;
		int lastRowIndex = rows - 1;
		for (i = cols - restCells; i < cols; i++){
			// create null cell
			ThumbsMatrixCell *itemCell = [[ThumbsMatrixCell alloc] initWithItemObject:nil
																		itemIndex:-1
																		isEnabled:NO
																		isExist:NO
										];
			[self setToolTip:nil forCell:itemCell];
			[self putCell:itemCell atRow:lastRowIndex column:i];
			[itemCell release];
		}
	}

	[self sizeToCells];
	[self setNeedsDisplay:YES];
}
//------------------------------------
// clearMatrix
//------------------------------------
- (void)clearMatrix
{
	int i;
	// remove matrix cells
	for (i = [self numberOfRows] - 1; i >= 0; i--){
		[self removeRow:i];
	}
}
//------------------------------------
// updateSelectedCell
//------------------------------------
- (void)updateSelectedCell
{

	NSIndexSet *selectionIndexes = [filelistArrayController selectionIndexes];
	int pointedIndex = [self pointedIndex];

/*
	int selectedIndex = 0;
	if([selectionIndexes count] > 0){
		selectedIndex = [selectionIndexes firstIndex];
	}
	NSLog(@"selectedIndex=%d", selectedIndex);
*/
//	NSLog(@"pointedIndex=%d", pointedIndex);
//	NSLog(@"selectionIndexes = %@", [selectionIndexes description]);

	// select cell
	int col = 0;
	int row = 0;
	int maxCols = [self maxCols];
	int i;
	int itemIndex;

	// get max count
	int curRows, curCols;
	[self getNumberOfRows:&curRows columns:&curCols];
	int maxCount = curRows * curCols;

	id record;

	for(i = 0; i < [[self cells] count] && i < maxCount; i++){

		record = [[self cells] objectAtIndex:i];

		// null cell
		if([record isEnabled] == NO){
			continue;
		}

		itemIndex = [record itemIndex];
//		NSLog(@"itemIndex = %d", itemIndex);

		// select
		if([selectionIndexes containsIndex:itemIndex]){
			[record setIsSelected:YES];
		}else{
			[record setIsSelected:NO];
		}

		// adjust scroll
		if(itemIndex >= 0 && itemIndex == pointedIndex){
			row = i / maxCols;
			col = i % maxCols;
//			NSLog(@"itemIndex = %d", itemIndex);
//			NSLog(@"row=%d, col=%d", row, col);
			[self adjustScrollPosition:NSMakePoint(col, row)];
		}
	}
}
//------------------------------------
// selectMovedCell
//------------------------------------
- (BOOL)selectMovedCell:(int)arrow isShift:(BOOL)isShift isCmd:(BOOL)isCmd
{

	int pointedIndex = [self pointedIndex];
	int startIndex = [self startIndex];
	int endIndex = [self endIndex];
	int maxCols = [self maxCols];

	//
	// adjust for key up / down
	//
	// get max cell count
	int curRows, curCols;
	[self getNumberOfRows:&curRows columns:&curCols];
//	int maxCells = curRows * curCols;

	// displayed item count
	int itemCount = (endIndex - startIndex) + 1;

	// item / rest cols
	int itemCols = itemCount % maxCols;
	int restCols = maxCols - itemCols;

	// pointed col position
	int pointedColIndex = (pointedIndex - startIndex) % maxCols + 1;

	// get adjust cells
	int adjustCells = 0;
	// pointed col position is item cols
	if(pointedColIndex <= itemCols){
		adjustCells = restCols;
	}else{
		adjustCells = itemCols * -1;
	}

//	NSLog(@"pointedColIndex = %d", pointedColIndex);
//	NSLog(@"itemCols = %d, restCols=%d", itemCols, restCols);
//	NSLog(@"adjustCells = %d", adjustCells);

	// up
	if(arrow == KEY_ARROW_UP){
		pointedIndex -= maxCols;
		if(pointedIndex < startIndex){
			pointedIndex += adjustCells;
		}
	}
	// down
	else if(arrow == KEY_ARROW_DOWN){
		pointedIndex += maxCols;
		if(pointedIndex > endIndex){
			pointedIndex -= adjustCells;
		}
	}
	// left
	else if(arrow == KEY_ARROW_LEFT){
		pointedIndex--;
	}
	// right
	else if(arrow == KEY_ARROW_RIGHT){
		pointedIndex++;
	}

	// selectIndexCell
	return [self selectIndexCell:pointedIndex isShift:isShift isCmd:isCmd];

}
//------------------------------------
// selectIndexCell
//------------------------------------
- (BOOL)selectIndexCell:(int)index isShift:(BOOL)isShift isCmd:(BOOL)isCmd
{

	int lastIndex = [self lastIndex];
//	int startIndex = [self startIndex];
//	int endIndex = [self endIndex];

	BOOL isSelect = NO;

//	if(index >= startIndex && index <= endIndex){
	if(index >= 0 && index <= lastIndex){
		isPointedCell_ = YES;
		[self changeSelectionIndexes:index isShift:isShift isCmd:isCmd];
		isSelect = YES;
	}

	return isSelect;
}
//------------------------------------
// changeSelectionIndexes
//------------------------------------
- (void)changeSelectionIndexes:(int)index
						isShift:(BOOL)isShift
						isCmd:(BOOL)isCmd
{

	[self setPointedIndex:index];

	if(isShift == YES || isCmd == YES){

		// no select
		if(isShift == YES && [[filelistArrayController selectionIndexes] count] <= 0){
			return;
		}

		NSMutableIndexSet *selectionIndexes = [NSMutableIndexSet indexSet];

		[selectionIndexes addIndexes:[filelistArrayController selectionIndexes]];

		if(isShift == YES){
			// less
			if(index < [selectionIndexes firstIndex]){
				[selectionIndexes addIndexesInRange:NSMakeRange(index, [selectionIndexes firstIndex]-index)];
			}
			// greater
			else if(index > [selectionIndexes lastIndex]){
				[selectionIndexes addIndexesInRange:NSMakeRange([selectionIndexes lastIndex]+1, index - [selectionIndexes lastIndex])];
			}
			// in selection
			else{
				[selectionIndexes addIndexesInRange:NSMakeRange([selectionIndexes firstIndex]+1, index - [selectionIndexes firstIndex])];
			}
		}
		if(isCmd == YES){
			// remove
			if([selectionIndexes containsIndex:index]){
				[selectionIndexes removeIndex:index];
			}
			// add
			else{
				[selectionIndexes addIndex:index];
			}
		}
//		NSLog(@"selectionIndexes=%@", [selectionIndexes description]);
		[filelistArrayController setSelectionIndexes:selectionIndexes];
	}
	else{
		[filelistArrayController setSelectionIndex:index];
	}

}
//------------------------------------
// adjustScrollPosition
//------------------------------------
- (void)adjustScrollPosition:(NSPoint)cellPosition
{

	NSScrollView *scrollView = [self enclosingScrollView];
	NSClipView *clipView = [scrollView contentView];

	NSRect clipRect = [clipView bounds];
//	NSSize cellSize = [self calcCellSize:[scrollView contentSize] cols:[self maxCols]];
	NSSize cellSize = [self cellSize];

	NSPoint cellPoint = NSZeroPoint;
	cellPoint.y = ((cellSize.height + CELL_PADDING_HEIGHT) * cellPosition.y);
//	NSLog(@"cellPoint=%f, clipRect=%f",cellPoint.y, clipRect.origin.y);
	if(cellPoint.y < clipRect.origin.y){
		[clipView scrollToPoint:cellPoint];
		[scrollView reflectScrolledClipView:clipView];
	}
	else if(cellPoint.y + cellSize.height > clipRect.origin.y + clipRect.size.height){	
//		NSLog(@"diff=%f", (cellPoint.y + cellSize.height) - (clipRect.origin.y + clipRect.size.height));
		cellPoint.y = clipRect.origin.y + ((cellPoint.y + cellSize.height) - (clipRect.origin.y + clipRect.size.height));
		[clipView scrollToPoint:cellPoint];
		[scrollView reflectScrolledClipView:clipView];
	}
}

//------------------------------------
// calcCellSize
//------------------------------------
- (NSSize)calcCellSize:(NSSize)frameSize cols:(int)cols
{
	float cellWidth = (frameSize.width - (CELL_PADDING_WIDTH * (cols - 1))) / cols;
	float cellHeight = [self defaultMainMatrixCellSize];
	return NSMakeSize(cellWidth, cellHeight);
}
//------------------------------------
// itemInfoDidChanged
//------------------------------------
-(void)itemInfoDidChanged:(NSNotification *)notification
{

	// force create matrix
	[self createMatrix:[self startIndex] endIndex:[self endIndex]];
	[self updateSelectedCell];
	[self setNeedsDisplay:YES];
}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
//	NSLog(@"keyPath=%@", keyPath);
	if([keyPath isEqualToString:@"arrangedObjects"]){
		isChangedArray_ = YES;
//		[self changeMatrix:nil];
	}
	else if([keyPath isEqualToString:@"selection"]){
		[self changeMatrix:nil];
	}
	else if([keyPath isEqualToString:@"optMaxResultsType"]){
		// for recreate matrix
		isChangedArray_ = YES;
		[self changeMatrix:nil];
	}
	else if([keyPath isEqualToString:@"optTabViewMainIndex"]){
		[self changePageButtonEnable];
	}
	
}
//------------------------------------
// setFrame
//------------------------------------
- (void)setFrame:(NSRect)frame
{

	// set cell size
	[self setCellSize:[self calcCellSize:[[self enclosingScrollView] contentSize] cols:[self maxCols]]];
	[self sizeToCells];

//	[super setFrame:frame];

}
/*
//------------------------------------
// viewDidEndLiveResize
//------------------------------------
- (void)viewDidEndLiveResize
{
//	[self setNeedsDisplay:YES];
//	[self changeMatrix];
}
*/
//------------------------------------
// frameDidResize
//------------------------------------
-(void)frameDidResize:(NSNotification *)notification
{
	// retile
	[self retileMatrix];

}
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)event
{

	BOOL isAction = NO;
	BOOL isShift = NO;
	BOOL isCmd = NO;

	NSString *keys = [event charactersIgnoringModifiers];

	if (keys && [keys length] > 0){

		unichar c = [keys characterAtIndex:0];

		// get modifierFlags
		if([event modifierFlags] & NSShiftKeyMask){
			isShift = YES;
		}
		if([event modifierFlags] & NSCommandKeyMask){
			isCmd = YES;
		}
		
		if(c == NSUpArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_UP isShift:isShift isCmd:isCmd];
			isAction = YES;
		}
		else if(c == NSDownArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_DOWN isShift:isShift isCmd:isCmd];
			isAction = YES;
		}
		else if(c == NSLeftArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_LEFT isShift:isShift isCmd:isCmd];
			isAction = YES;
		}
		else if(c == NSRightArrowFunctionKey){
			[self selectMovedCell:KEY_ARROW_RIGHT isShift:isShift isCmd:isCmd];
			isAction = YES;
		}

	}


	if(isAction == NO){
		[super keyDown:event];
	}

}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)event
{

	int row,col;
	isChangedIndex_ = NO;

	// get cell point
	NSPoint mouseDownPoint = [self convertPoint:[event locationInWindow] fromView:nil];
	[self getRow:&row column:&col forPoint:mouseDownPoint];	

	// get clicked cell
	ThumbsMatrixCell *itemCell = [self cellAtRow:row column:col];

	// enabled cell
	if(itemCell != nil && [itemCell isEnabled] == YES){

		NSIndexSet *selectionIndexes = [filelistArrayController selectionIndexes];

		int itemIndex = [itemCell itemIndex];
		BOOL isContext = NO;
		BOOL isShift = NO;
		BOOL isCmd = NO;
		BOOL isChange = NO;

		// get modifierFlags
		if([event modifierFlags] & NSShiftKeyMask){
			isShift = YES;
		}
		if([event modifierFlags] & NSCommandKeyMask){
			isCmd = YES;
		}
		if([event type] == NSRightMouseDown ||
			([event modifierFlags] & NSControlKeyMask)){
			isContext = YES;
		}
			
		// show context
		if(isContext == YES){
			// force off
			isShift = NO;
			isCmd = NO;
			// out of selectionIndexes
			if(![selectionIndexes containsIndex:itemIndex]){
				isChange = YES;
				// force extend selection
				if([selectionIndexes count] > 1){
					isCmd = YES;
				}
			}
		}
		// left click
		else{
			if(isShift == YES || isCmd == YES){
				isChange = YES;
			}
		}

		// changeSelectionIndexes
		if(isChange == YES){
			isChangedIndex_ = YES;
			isPointedCell_ = YES;
			[self changeSelectionIndexes:itemIndex
									isShift:isShift
									isCmd:isCmd
			];
		}

		// double clicked
		if([event type] == NSLeftMouseDown){
			if([event clickCount] == 2){
				[viewMain playItem:nil];
			}
		}

		// context menu
		if(isContext ==YES){
			[NSMenu popUpContextMenu:cmFilelist withEvent:event forView:self];
		}
	
	}

	// get responder
	[[self enclosingScrollView] becomeFirstResponder];

}
//------------------------------------
// mouseUp
//------------------------------------
- (void)mouseUp:(NSEvent *)event
{

	int row,col;

	// get cell point
	NSPoint mouseDownPoint = [self convertPoint:[event locationInWindow] fromView:nil];
	[self getRow:&row column:&col forPoint:mouseDownPoint];	

	// get clicked cell
	ThumbsMatrixCell *itemCell = [self cellAtRow:row column:col];

	// enabled cell
	if(itemCell != nil && [itemCell isEnabled] == YES){

		int itemIndex = [itemCell itemIndex];
		int pointedIndex = [self pointedIndex];
		BOOL isContext = NO;
		BOOL isShift = NO;
		BOOL isCmd = NO;

		// get modifierFlags
		if([event modifierFlags] & NSShiftKeyMask){
			isShift = YES;
		}
		if([event modifierFlags] & NSCommandKeyMask){
			isCmd = YES;
		}
		if([event type] == NSRightMouseDown ||
			([event modifierFlags] & NSControlKeyMask)){
			isContext = YES;
		}

		// not changed index
		if(isChangedIndex_ == NO){
			// changed pointed index
			if(pointedIndex != itemIndex){
				isPointedCell_ = YES;
			}
			[self changeSelectionIndexes:itemIndex
									isShift:isShift
									isCmd:isCmd
			];
		}
	}

	isChangedIndex_ = NO;
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)event
{
	[self mouseDown:event];

}
//------------------------------------
// mouseDragged
//------------------------------------
- (void)mouseDragged:(NSEvent *)event
{
	[super mouseDragged:event];

	int row,col,i;

	// get cell point
	NSPoint mouseDownPoint = [self convertPoint:[event locationInWindow] fromView:nil];
	[self getRow:&row column:&col forPoint:mouseDownPoint];	

	// get clicked cell
	ThumbsMatrixCell *itemCell = [self cellAtRow:row column:col];

	// enabled cell
	if(itemCell != nil && [itemCell isEnabled] == YES){

		NSPasteboard *pboard = [NSPasteboard pasteboardWithName:NSDragPboard];

		NSMutableArray *objects = [NSMutableArray array];
		NSMutableArray *items = [NSMutableArray array];
		NSMutableArray *filePaths = [NSMutableArray array];
		NSFileManager *myFile = [NSFileManager defaultManager];

		// create source objects array
		NSIndexSet *selectedIndexes = [filelistArrayController selectionIndexes];
		int itemIndex = [itemCell itemIndex];
		if([selectedIndexes containsIndex:itemIndex]){
			// from selectedObjects
			[objects addObjectsFromArray:[filelistArrayController selectedObjects]];
		}else{
			// from dragged object
			[objects addObject:[itemCell itemObject]];
		}

		// create paste board objects array
		for (i = 0; i < [objects count]; i++) {
			NSManagedObject *item = [objects objectAtIndex:i];
			NSManagedObjectID *objectID = [item objectID];
			NSURL *representedURL = [objectID URIRepresentation];
			[items addObject:representedURL];
			// for NSFilenamesPboardType
			if([myFile fileExistsAtPath:[item valueForKey:@"filePath"] isDirectory:nil]){
				[filePaths addObject:[item valueForKey:@"filePath"]];
			}
		}

		[pboard declareTypes:[NSArray arrayWithObjects:FilelistPboardType, NSFilenamesPboardType, nil] owner:nil];
		[pboard addTypes:[NSArray arrayWithObjects:FilelistPboardType, NSFilenamesPboardType, nil] owner:nil];

		// for FilelistPboardType
		[pboard setString:[items componentsJoinedByString:@","] forType:FilelistPboardType];		
		// for NSFilenamesPboardType
		if([filePaths count] > 0){
			[pboard setPropertyList:filePaths forType:NSFilenamesPboardType];		 
		}
	
		//
		// drag image
		//

		// create drag image
		NSImage	*dragImage = [[NSImage alloc] init];
		[dragImage setSize:NSMakeSize(65.0, 40.0)];
		NSSize dragImageSize = [dragImage size];

		NSPoint dragPoint; 
		dragPoint.x = mouseDownPoint.x - (dragImageSize.width) / 2;
		dragPoint.y = mouseDownPoint.y + (dragImageSize.height) / 2;

		// create source image
		NSImage *sourceImage = [[[NSImage alloc] initByReferencingFile: [[NSBundle mainBundle] pathForResource:@"icon_video" ofType:@"png"]] autorelease];
		[sourceImage setScalesWhenResized:YES];
		[sourceImage setSize:NSMakeSize(24, 24)];

		// set count label bounds
		NSString *countStr = [NSString stringWithFormat:@"%i", [items count]];
		NSRect imageBounds;
		imageBounds.origin = NSMakePoint(0.0, 0.0);
		imageBounds.size = dragImageSize;

		// draw source image
		NSPoint sourcePoint;
		sourcePoint.x = imageBounds.origin.x + ((imageBounds.size.width - [sourceImage size].width) / 2);
		sourcePoint.y = imageBounds.origin.y + ((imageBounds.size.height - [sourceImage size].height) / 2);
//		[sourceImage dissolveToPoint:NSMakePoint(0,0) fraction: 0.6]; 

		[dragImage lockFocus]; 

		// draw label
		[self drawLabelAndString:imageBounds
						withString:countStr
						fontSize:LABEL_DRAG_FSIZE
						fontColor:[NSColor whiteColor]
						labelColor:[NSColor redColor]
						align:CELL_ALIGN_RIGHT
						valign:CELL_VALIGN_TOP
						hPadding:LABEL_DRAG_HPADDING
						vPadding:LABEL_DRAG_VPADDING
						radius:LABEL_DRAG_RADIUS
		];

		[sourceImage dissolveToPoint:sourcePoint fraction:0.8];

		[dragImage unlockFocus]; 

		// draw drag image
		[self dragImage:dragImage
					at:dragPoint 
					offset:NSMakeSize(0,0) 
					event:event 
					pasteboard:pboard 
					source:self 
					slideBack:YES
		]; 
	}

}
//------------------------------------
// draggingEntered
//------------------------------------
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)info
{

	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard){
		return NSDragOperationNone;
	}

	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}

	// from file path
	if ([[pboard types] containsObject:NSFilenamesPboardType]){
		return NSDragOperationCopy;
	}

	return NSDragOperationNone;

}
//------------------------------------
// draggingExited
//------------------------------------
- (void)draggingExited:(id <NSDraggingInfo>)info
{
	// none
}
//------------------------------------
// draggingUpdated
//------------------------------------
- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)info
{

	// Get pboard
	NSPasteboard *pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard){
		return NSDragOperationNone;
	}

	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}

	// from file path
	if ([[pboard types] containsObject:NSFilenamesPboardType]){
		return NSDragOperationCopy;
	}

	return NSDragOperationNone;

}
//------------------------------------
// prepareForDragOperation
//------------------------------------
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)info
{
	return YES;
}
//------------------------------------
// performDragOperation
//------------------------------------
- (BOOL)performDragOperation:(id <NSDraggingInfo>)info
{

	NSPasteboard *pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard) {
        return NO;
	}

	// ignore self
	if (draggingSource == self) {
		return NO;
	}

	// Check pboard type
	// from filepath
	if ([[pboard types] containsObject:NSFilenamesPboardType]){
		NSArray *filePaths = [pboard propertyListForType:NSFilenamesPboardType];
		if([filePaths count] > 0){	
			SEL sel = NSSelectorFromString(@"importItemWithFilePaths:");
			[viewMain performSelector:sel withObject:filePaths afterDelay:0.1];
			return YES;
		}
	}

	return YES;

}
//------------------------------------
// draggingSourceOperationMaskForLocal
//------------------------------------
- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)isLocal
{
	if(isLocal == YES){
		return	NSDragOperationLink;
	}else{
		return	NSDragOperationLink;
	}
}
//------------------------------------
// acceptsFirstResponder
//------------------------------------
-(BOOL)acceptsFirstResponder
{
	return YES;
}
//------------------------------------
// maxCols
//------------------------------------
- (int)maxCols
{

	int cols;
	float cellWidth = [self defaultMainMatrixCellSize];
	NSSize contentSize = [[self enclosingScrollView] contentSize];
	cols = (contentSize.width + CELL_PADDING_WIDTH) / (cellWidth + CELL_PADDING_WIDTH);
	if(cols < 1){
		cols = 1;
	}

	return cols;
}
//------------------------------------
// startIndex
//------------------------------------
- (void)setStartIndex:(int)index
{
	startIndex_ = index;
}
- (int)startIndex
{
	return startIndex_;

}
//------------------------------------
// endIndex
//------------------------------------
- (void)setEndIndex:(int)index
{
	endIndex_ = index;
}
- (int)endIndex
{
	return endIndex_;

}
//------------------------------------
// selectedIndex
//------------------------------------
- (void)setSelectedIndex:(int)index
{
	selectedIndex_ = index;
}
- (int)selectedIndex
{
	return selectedIndex_;

}
//------------------------------------
// setPointedIndex
//------------------------------------
- (void)setPointedIndex:(int)index
{
	pointedIndex_ = index;
}
- (int)pointedIndex
{
	return pointedIndex_;
}
//------------------------------------
// lastIndex
//------------------------------------
- (int)lastIndex
{
	return [self totalResults] - 1;
}
//------------------------------------
// totalResults
//------------------------------------
- (int)totalResults
{
	return [[filelistArrayController arrangedObjects] count];
}
//------------------------------------
// maxResults
//------------------------------------
- (int)maxResults
{
	int maxResults = [self defaultMaxResults];

	// no limit
	if(maxResults <= 0){
		maxResults = [self totalResults];
	}

	return maxResults;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{

	[filelistArrayController removeObserver:self forKeyPath:@"arrangedObjects"];
	[filelistArrayController removeObserver:self forKeyPath:@"selection"];

	[[NSNotificationCenter defaultCenter] removeObserver:self];

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults removeObserver:self forKeyPath:@"optTabViewMainIndex"];
	[defaults removeObserver:self forKeyPath:@"optMaxResultsType"];
	
	[super dealloc];
}

@end
