#import "UserDefaultsExtension.h"

@implementation NSObject(userDefaultsExtension_)

//------------------------------------
// setWindowRect 
//------------------------------------
- (void)setWindowRect:(NSWindow*)aWindow key:(NSString*)key
{
	
	if(aWindow == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// set rect string
	NSString *recStrWindow = [defaults stringForKey:key];
	if(recStrWindow){
		NSRect rectWindow = NSRectFromString(recStrWindow);
		[aWindow setFrame:rectWindow display:YES animate:NO];
	}else{
		[aWindow center];
	}

}
//------------------------------------
// setWindowPosition 
//------------------------------------
- (void)setWindowPosition:(NSWindow*)aWindow key:(NSString*)key
{

	if(aWindow == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// set rect string
	NSString *recStrWindow = [defaults stringForKey:key];
	if(recStrWindow){
		NSRect rectWindow = NSRectFromString(recStrWindow);
		[aWindow setFrameOrigin:NSMakePoint(rectWindow.origin.x,rectWindow.origin.y)];
	}else{
		[aWindow center];
	}

}
//------------------------------------
// saveWindowRect 
//------------------------------------
- (void)saveWindowRect:(NSWindow*)aWindow key:(NSString*)key
{
	if(aWindow == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSString *rectStrWindow = NSStringFromRect([aWindow frame]);
	[defaults setObject:rectStrWindow forKey:key];
	[defaults synchronize];

}
//-------------------------
// setSplitViewRect
//-------------------------
- (void)setSplitViewRect:(NSSplitView *)aSplitView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSArray *array = [defaults objectForKey:key];
	if (array == nil || [array isKindOfClass:[NSArray class]] == NO){
		return;
	}

	NSArray *subviews = [aSplitView subviews];

	int i;
	for (i = 0; i < [subviews count] && i < [array count]; i++){
		NSRect rect = NSRectFromString([array objectAtIndex:i]);
		[[subviews objectAtIndex:i] setFrame:rect];
	}
}

//-------------------------
// saveSplitViewRect
//-------------------------
- (void)saveSplitViewRect:(NSSplitView *)aSplitView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSMutableArray *array= [[[NSMutableArray alloc] init] autorelease];
	int i;

	// create array
	NSArray *subviews = [aSplitView subviews];
	for (i = 0; i < [subviews count]; i++){
		[array addObject:NSStringFromRect([[subviews objectAtIndex:i] frame])];
	}

	// save array
	if([array count] > 0){
		[defaults setObject:array forKey:key];
	}else{
		if([defaults objectForKey:key]){
			[defaults removeObjectForKey:key];
		}
	}
	[defaults synchronize];
}
//-------------------------
// splitViewRectString
//-------------------------
- (NSString*)splitViewRectString:(NSString*)key index:(int)index
{

	NSString *rectString = @"";

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if([defaults objectForKey:key]){
		NSArray *array = [defaults objectForKey:key];
		if (array != nil && [array isKindOfClass:[NSArray class]] == YES){
			if (index >= 0 && index < [array count]){
				rectString = [array objectAtIndex:index];
			}
		}
	}

	return rectString;
}
//------------------------------------
// setTableColumnState
//------------------------------------
- (void)setTableColumnState:(NSTableView*)aTableView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSArray *defaultColumns = [defaults objectForKey:key];
	if (defaultColumns == nil || [defaultColumns isKindOfClass:[NSArray class]] == NO){
//		NSLog(@"defaults is not array");
		return;
	}

	NSTableColumn *aColumn;
	NSString *identifier;
	id record;
	int i;
	int columnIndex = 0;
	int currentIndex;
	int lastColumnIndex = [[aTableView tableColumns] count] - 1;

	for (i = 0; i < [defaultColumns count]; i++){

		record = [defaultColumns objectAtIndex:i];

		if(![record valueForKey:@"identifier"]){
			continue;
		}

		identifier =[record valueForKey:@"identifier"];

		aColumn = [aTableView tableColumnWithIdentifier:identifier];

		// column is exist
		if(aColumn != nil){

			columnIndex++;
			currentIndex = [aTableView columnWithIdentifier:identifier];

//			NSLog(@"identifier=%@", identifier);
//			NSLog(@"lastColumnIndex=%d", lastColumnIndex);
//			NSLog(@"currentIndex=%d -> columnIndex=%d", currentIndex, columnIndex);

			// move column
			if(columnIndex <= lastColumnIndex){
				[aTableView moveColumn:currentIndex toColumn:columnIndex];
			}

			// set column width
			if([record valueForKey:@"width"]){
				[aColumn setWidth:[[record valueForKey:@"width"] intValue]];
			}
		}
	}

}
//-------------------------
// saveTableColumnState
//-------------------------
- (void)saveTableColumnState:(NSTableView*)aTableView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSMutableArray *defaultColumns = [[[NSMutableArray alloc] init] autorelease];

	NSArray *tableColumns = [aTableView tableColumns];
	int i;

	// create array
	for (i = 0; i < [tableColumns count]; i++){
		[defaultColumns addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[[tableColumns objectAtIndex:i] identifier], @"identifier",
				[NSNumber numberWithInt:[[tableColumns objectAtIndex:i] width]], @"width",
				nil
			]
		];
	}

	// save array
	if([defaultColumns count] > 0){
		[defaults setObject:defaultColumns forKey:key];
	}else{
		if([defaults objectForKey:key]){
			[defaults removeObjectForKey:key];
		}
	}
	[defaults synchronize];
}
//------------------------------------
// setArrayControllerSortDescriptor
//------------------------------------
- (void)setArrayControllerSortDescriptor:(NSArrayController*)aController key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSString *column;
	BOOL isAsc = YES;

	NSDictionary *sortDict = [defaults objectForKey:key];
	if (sortDict == nil || [sortDict isKindOfClass:[NSDictionary class]] == NO){
		return;
	}

	// value not found
	if(![sortDict valueForKey:@"column"]){
		return;
	}

	// colmun not found
	column = [sortDict valueForKey:@"column"];
/*
	if([aTableView tableColumnWithIdentifier:column] == nil){
		return;
	}
*/
	if([sortDict valueForKey:@"isAsc"]){
		isAsc = [[sortDict valueForKey:@"isAsc"] boolValue];
	}

	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:column ascending:isAsc selector:@selector(compare:)];
	[sortDescriptor autorelease];
	[aController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];

}
//------------------------------------
// saveArrayControllerSortDescriptor
//------------------------------------
- (void)saveArrayControllerSortDescriptor:(NSArrayController*)aController key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if([[aController sortDescriptors] count] > 0){
		NSSortDescriptor *aDescriptor = [[aController sortDescriptors] objectAtIndex:0];

		[defaults setObject:
			[NSDictionary dictionaryWithObjectsAndKeys:
						[aDescriptor key], @"column",
						[NSNumber numberWithBool:[aDescriptor ascending]], @"isAsc",
						nil
			]
		forKey:key];
	}
	[defaults synchronize];
}
//------------------------------------
// setSearchFieldRecentSearches
//------------------------------------
- (void)setSearchFieldRecentSearches:(NSSearchField*)searchField key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSArray *array = [defaults objectForKey:key];
	if (array == nil || [array isKindOfClass:[NSArray class]] == NO){
		return;
	}

	[searchField setRecentSearches:array];

}
//------------------------------------
// saveSearchFieldRecentSearches
//------------------------------------
- (void)saveSearchFieldRecentSearches:(NSSearchField*)searchField key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[searchField recentSearches] forKey:key];

	[defaults synchronize];
}

//------------------------------------
// saveLastSelectedIndex 
//------------------------------------
- (void)saveLastSelectedIndex:(int)index key:(NSString*)key
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// last uniqueId
	[defaults setObject:[NSNumber numberWithInt:index] forKey:key];
}
//------------------------------------
// getLastSelectedIndex 
//------------------------------------
- (int)getLastSelectedIndex:(NSString*)key
{
	int index = -1;
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if([defaults objectForKey:key]){
		index = [[defaults objectForKey:key] intValue];
	}

	return index;

}

//------------------------------------
// defaultMaxResults 
//------------------------------------
- (int)defaultMaxResults
{

	int maxResultsType = [self defaultIntValue:@"optMaxResultsType"];
	int maxResults = 0;

	// no limit
	if(maxResultsType == 0){
		maxResults = 0;
	}
	else if(maxResultsType == 1){
		maxResults = 10;
	}
	else if(maxResultsType == 2){
		maxResults = 30;
	}
	else if(maxResultsType == 3){
		maxResults = 50;
	}
	else if(maxResultsType == 4){
		maxResults = 100;
	}


/*
	int maxResults = [self defaultIntValue:@"optMaxResults"];

//	if(maxResults <= 0){
//		maxResults = 10;
//	}
*/

	return maxResults;

}
//------------------------------------
// defaultMainMatrixCellSize
//------------------------------------
- (float)defaultMainMatrixCellSize
{
	float scale = [self defaultFloatValue:@"optMainMatrixCellScale"];
	return (100 + scale * scale);
}
//----------------------
// defaultPlayRepeat
//----------------------
- (int)defaultPlayRepeat
{
	return [self defaultIntValue:@"optPlayRepeat"];
}
//----------------------
// defaultPlayInterval
//----------------------
- (float)defaultPlayInterval
{
	return [self defaultFloatValue:@"optPlayInterval"];
}

//----------------------
// defaultPlayRate
//----------------------
- (float)defaultPlayRate
{
	return [self defaultFloatValue:@"optPlayRate"];
}
//----------------------
// defaultPlayVolume
//----------------------
- (float)defaultPlayVolume
{
	return [self defaultFloatValue:@"optPlayVolume"];
}
//----------------------
// defaultMovieKeepAspect
//----------------------
- (BOOL)defaultMovieKeepAspect
{
	return [self defaultBoolValue:@"optMovieKeepAspect"];
}
//----------------------
// defaultSkipInterval
//----------------------
- (float)defaultSkipInterval
{
	float interval = 5.0;

	switch([self defaultIntValue:@"optSkipIntervalNo"]){
		case 0:
			interval = 1.0;
			break;
		case 1:
			interval = 5.0;
			break;
		case 2:
			interval = 10.0;
			break;
		case 3:
			interval = 15.0;
			break;
		case 4:
			interval = 30.0;
			break;
		case 5:
			interval = 60.0;
			break;
	}

	return interval;
//	return [self defaultFloatValue:@"optSkipInterval"];
}

//------------------------------------
// defaultThumbsFrameType
//------------------------------------
- (int)defaultThumbsFrameType
{
	return [self defaultIntValue:@"optThumbsFrameType"];
}
//------------------------------------
// defaultThumbsFrameSec
//------------------------------------
- (float)defaultThumbsFrameSec
{

	float sec = 1.0;

	switch([self defaultIntValue:@"optThumbsFrameSecNo"]){
		case -1:
			sec = 0.0;
			break;
		case 0:
			sec = 1.0;
			break;
		case 1:
			sec = 5.0;
			break;
		case 2:
			sec = 10.0;
			break;
		case 3:
			sec = 30.0;
			break;
	}

	return sec;

//	return [self defaultFloatValue:@"optThumbsFrameSec"];
}
//------------------------------------
// defaultThumbsSize
//------------------------------------
- (NSSize)defaultThumbsSize
{
	NSSize size = NSMakeSize(150,150);

	switch([self defaultIntValue:@"optThumbsSize"]){
		case 0:
			size = NSMakeSize(100,100);
			break;
		case 1:
			size = NSMakeSize(150,150);
			break;
		case 2:
			size = NSMakeSize(200,200);
			break;
	}

	return size;
}
//------------------------------------
// defaultChapterSize
//------------------------------------
- (NSSize)defaultChapterSize;
{
	NSSize size = NSMakeSize(100,100);
	return size;
}
//------------------------------------
// setDefaultStringValue 
//------------------------------------
- (void)setDefaultStringValue:(NSString*)string key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:string forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultStringValue 
//------------------------------------
- (NSString*)defaultStringValue:(NSString*)key
{
	NSString *string = @"";

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// string value 
	if([defaults objectForKey:key]){
		string = [defaults objectForKey:key];
	}

	return string;
}
//------------------------------------
// setDefaultIntValue 
//------------------------------------
- (void)setDefaultIntValue:(int)value key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[NSNumber numberWithInt:value] forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultIntValue
//------------------------------------
- (int)defaultIntValue:(NSString*)key
{
	int value = 0;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// int value 
	if([defaults objectForKey:key]){
		value = [[defaults objectForKey:key] intValue];
	}

	return value;
}
//------------------------------------
// setDefaultFloatValue 
//------------------------------------
- (void)setDefaultFloatValue:(float)value key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[NSNumber numberWithFloat:value] forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultFloatValue
//------------------------------------
- (float)defaultFloatValue:(NSString*)key
{
	float value = 0.0;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// float value 
	if([defaults objectForKey:key]){
		value = [[defaults objectForKey:key] floatValue];
	}

	return value;
}
//------------------------------------
// setDefaultBoolValue 
//------------------------------------
- (void)setDefaultBoolValue:(BOOL)value key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[NSNumber numberWithBool:value] forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultBoolValue
//------------------------------------
- (BOOL)defaultBoolValue:(NSString*)key
{
	BOOL value = NO;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// BOOL value 
	if([defaults objectForKey:key]){
		value = [[defaults objectForKey:key] boolValue];
	}

	return value;
}

//-------------------------
// setDefaultArrayValue
//-------------------------
- (void)setDefaultArrayValue:(NSArray*)array key:(NSString*)key
{

	if(array == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:array forKey:key];
	[defaults synchronize];

}
//-------------------------
// defaultArrayValue
//-------------------------
- (NSArray*)defaultArrayValue:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if([defaults objectForKey:key]){
		NSArray *array = [defaults objectForKey:key];
		if (array != nil && [array isKindOfClass:[NSArray class]] == YES){
			return array;
		}
	}

	return nil;
}
//------------------------------------
// getAppSupportFolderPath
//------------------------------------
- (NSString*)getAppSupportFolderPath
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
    return [basePath stringByAppendingPathComponent:@"MacVideoTunes"];
}
//-------------------------
// defaultOSVersionNo
//-------------------------
- (NSString*)defaultOSVersionNo
{
	// versionString = Version 10.4.11 (Build 8S165)
	NSString *versionString = [[NSProcessInfo processInfo] operatingSystemVersionString];
	NSScanner *scan = [NSScanner scannerWithString:versionString];

	NSString *versionNo = nil;
	[scan scanUpToString:@" " intoString:nil];
	[scan scanUpToString:@" " intoString:&versionNo];

	if(versionNo == nil){
		versionNo = @"";
	}

	return versionNo;
}
//-------------------------
// defaultAppName
//-------------------------
- (NSString*)defaultAppName
{
	NSString *appName = [self defaultStringValue:@"optAppName"];
	if([appName isEqualToString:@""]){
		appName = @"MacVideoTunes";
	}
	return appName;
}

@end