#import "ViewApplist.h"
#import "TBArrayController.h"
#import "FileAttributeExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewApplist

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// addItem
//------------------------------------
- (IBAction)addItem:(id)sender
{

	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
														isDir:NO
														isMultiSel:NO
														isPackage:NO
														isAlias:YES
														canCreateDir:NO
														defaultPath:@""
							];

	if([filePaths count] <= 0){
		return;
	}

	NSFileManager *myFile = [NSFileManager defaultManager];

	NSString *filePath = [self resolveAliasFileWithPath:[filePaths objectAtIndex:0]];
	NSString *appName = [myFile displayNameAtPath:filePath];

	[self createItem:appName filePath:filePath];

	// reload
	[applistArrayController rearrangeObjects];


}
//------------------------------------
// removeItem
//------------------------------------
- (IBAction)removeItem:(id)sender
{

	NSArray *items = [applistArrayController selectedObjects];
	if([items count] <= 0){
		return;
	}

	NSEnumerator *enumItems = [items objectEnumerator];
	id record;
	// remove
	while (record = [enumItems nextObject]) {
		// remove record
		[[MacVideoTunes_AppDelegate managedObjectContext] deleteObject:record];
	}

//	[applistArrayController remove:nil];

	[applistArrayController rearrangeObjects];

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createItem
//------------------------------------	  
- (void)createItem:(NSString*)appName filePath:(NSString*)filePath
{

	NSManagedObjectContext *context = [MacVideoTunes_AppDelegate managedObjectContext];

	NSManagedObject *object = [NSEntityDescription
			insertNewObjectForEntityForName:@"applist"
			inManagedObjectContext:context];

	// get max index
	int index = [tbArrayController getMaxIntValueFromEntityWithColumn:@"applist" column:@"index"];

	index++;

	[object setValue:[NSNumber numberWithInt:index] forKey:@"index"];
	[object setValue:appName forKey:@"appName"];
	[object setValue:filePath forKey:@"filePath"];

}
//------------------------------------
// sortItem
//------------------------------------
- (void)sortItems:(NSArray*)items targetRow:(int)targetRow
{

	NSMutableArray *targetObjects = [[[NSMutableArray alloc] init] autorelease];
	NSManagedObjectContext *moc = [tbArrayController getManagedObjectContext:@"applist"];

	// create objects
	int i;
	for (i = 0; i < [items count]; i++) {						
		NSURL *objectURL = [NSURL URLWithString:[items objectAtIndex:i]];
		NSManagedObjectID *objectID = [[moc persistentStoreCoordinator] managedObjectIDForURIRepresentation:objectURL];
		if (objectID != nil) {
			id object = [moc objectRegisteredForID:objectID];
			[targetObjects addObject:object];
		}
	}
	// no targetObjects
	if([targetObjects count] <= 0){
		return;
	}

	NSArray *arrangedObjects = [applistArrayController arrangedObjects];

	int targetIndex = 0;
	id record;

	// get targetIndex
	if(targetRow > 0 && targetRow <= [arrangedObjects count]){
		record = [arrangedObjects objectAtIndex:(targetRow - 1)];
		targetIndex = [[record valueForKey:@"index"] intValue];
	}

	NSMutableArray *updatedObjects = [[NSMutableArray alloc] init];

	// update dropped items
	NSEnumerator *enumObjects = [targetObjects objectEnumerator];
	while (record = [enumObjects nextObject]) {

		targetIndex++;
		[record setValue:[NSNumber numberWithInt:targetIndex] forKey:@"index"];
 
		// add updated record
		[updatedObjects addObject:record];
	}

	// update other items
	for (i = targetRow; i < [arrangedObjects count]; i++) {
		record = [arrangedObjects objectAtIndex:i];

		// contains updated records
		if ([updatedObjects containsObject:record]) {
			continue;
		}
		targetIndex++;
		[record setValue:[NSNumber numberWithInt:targetIndex] forKey:@"index"];
	}

	[updatedObjects release];

	[applistArrayController rearrangeObjects];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
    [super dealloc];
}

@end
