#import "ViewChapter.h"
#import "ViewPlayer.h"
#import "ViewGallery.h"
#import "ViewProcStatus.h"
#import "TBArrayController.h"
#import "ChapterExtension.h"
#import "FileAttributeExtension.h"
#import "MovieAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

static int CHAPER_ALERT_MAX_COUNT = 30;

@implementation ViewChapter

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self setIsPlay:NO];
	[self setParams:nil];
	[self setItemObject:nil];
	[txtTitle setStringValue:@"Chapter"];

	// alloc chapterList
	chapterList_ = [[NSMutableArray alloc] init];

	// set roundedView attribute
//	[roundedView setViewAttr:0.5 radius:20.0];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleWindowChapterDidClosed:) name:WINDOW_NOTIF_CHAPTER_DID_CLOSED object:nil];
	[nc addObserver:self selector:@selector(handleWindowDialogDidOpened:) name:WINDOW_NOTIF_DIALOG_DID_OPENED object:nil];
	[nc addObserver:self selector:@selector(handleWindowFullScreenDidClosed:) name:WINDOW_NOTIF_FULLSCREEN_DID_CLOSED object:nil];
	[nc addObserver:self selector:@selector(handleWindowFullScreenDidClosed:) name:WINDOW_NOTIF_FULLSCREEN_DID_CHANGED object:nil];

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openChapterWindow
//------------------------------------
- (IBAction)openChapterWindow:(id)sender
{
	// open chapter
	[self openChapterView:YES];
}

//------------------------------------
// closeChapterWindow
//------------------------------------
- (IBAction)closeChapterWindow:(id)sender
{
	[chapterWindow close];
}
//------------------------------------
// changeChapterWindow
//------------------------------------
- (IBAction)changeChapterWindow:(id)sender
{
	if([chapterWindow isVisible] == NO){
		[self openChapterWindow:nil];
	}else{
		[self closeChapterWindow:nil];
	}
}
//------------------------------------
// playItemChapter
//------------------------------------
- (IBAction)playItemChapter:(id)sender
{

	id itemObject = [chapterMatrix selectedObject];
	if(itemObject == nil){
		return;
	}

	[self setItemObject:itemObject];
	[self setIsPlay:YES];

	if([self defaultBoolValue:@"optChapterAutoClose"] == YES){
		// post notification when window is closed
		[chapterWindow close];
	}
	else{
		[self playSelectedChapter:NO isFront:YES];
	}

}
//------------------------------------
// createItemChapter
//------------------------------------
- (IBAction)createItemChapter:(id)sender
{

	int tag = [sender tag];

	// not correct
	if([self canCreateChapter] == NO){
		[self displayMessage:@"alert"
					messageText:@"Can not create chapter."
					infoText:@"Current movie is not correct."
					btnList:@"Cancel"
		];
		return;
	}

	// get params
	NSDictionary *params = [self params];
	NSString *uniqueId = [params valueForKey:@"uniqueId"];
	
	if(!uniqueId || [uniqueId isEqualToString:@""]){
		[self displayMessage:@"alert"
					messageText:@"Can not create chapter."
					infoText:@"UniqueId is not defined."
					btnList:@"Cancel"
		];
		return;
	}

	// get filelist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", uniqueId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];
	if([fetchArray count] <= 0){
		[self displayMessage:@"alert"
					messageText:@"Can not create chapter."
					infoText:@"Item is not found."
					btnList:@"Cancel"
		];
		return;
	}
	id record = [fetchArray objectAtIndex:0];

	// start proc notification
	[self postVideoProcDidChangedNotification:YES];

	NSString *chapterId = @"";
	BOOL ret = YES;

	// current time
	if(tag == 0){

		int timeSec = [moviePlayerView movieCurrentTimeSec];
		chapterId = [NSString stringWithFormat:@"%d", timeSec];

		ret = [self createChapterAtCurrentTime:[moviePlayerView movie]
										uniqueId:uniqueId
										chapterId:chapterId
										timeSec:timeSec
										hasImage:[self hasImage]
										record:record
				];
	}
	// interval time
	else if(tag == 1){

		int intervalSec = [self defaultIntValue:@"optChapterAutoIntervalSec"];

		ret = [self createChapterAtIntervalTime:[moviePlayerView movie]
										uniqueId:uniqueId
										intervalSec:intervalSec
										hasImage:[self hasImage]
										record:record
				];

	}

	// success
	if(ret == YES){

		// create chapter list
		[self createChapterList:[self params]];

		// set updatedIndex
		int updatedIndex = [self getUpdatedIndex:chapterId];
		[chapterMatrix setSelectedIndex:updatedIndex];

		// post notification
		[self postItemChapterDidChangedNotification];

		// open chapter
		if([chapterWindow isVisible] == NO){
			[self openChapterView:NO];
		}

	}else{
		// open chapter
		if([chapterWindow isVisible] == NO){
			[self openChapterView:YES];
		}
	}

	// stop proc notification
	[self postVideoProcDidChangedNotification:NO];

}

//------------------------------------
// removeItemChapter
//------------------------------------
- (IBAction)removeItemChapter:(id)sender
{

	id itemObject = [chapterMatrix selectedObject];
	if(itemObject == nil){
		return;
	}

	NSString *chapterId = [itemObject valueForKey:@"chapterId"];

	// get params
	NSDictionary *params = [self params];
	NSString *uniqueId = [params valueForKey:@"uniqueId"];
//	NSString *title = [params valueForKey:@"title"];

	if(!uniqueId || [uniqueId isEqualToString:@""]){
		[self displayMessage:@"alert"
					messageText:@"Can not delete chapter."
					infoText:@"UniqueId is not defined."
					btnList:@"Cancel"
		];
		return;
	}

	// get filelist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", uniqueId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];
	if([fetchArray count] <= 0){
		[self displayMessage:@"alert"
					messageText:@"Can not delete chapter."
					infoText:@"Item is not found."
					btnList:@"Cancel"
		];
		return;
	}

	id record = [fetchArray objectAtIndex:0];

	//
	// remove chapter
	//
	//
	// update record
	//
	int chapterCount = 0;
	if([self removeChapterSet:chapterId record:record count:&chapterCount] == NO){

		[self displayMessage:@"alert"
					messageText:@"Can not delete chapter."
					infoText:@"Failed updating item."
					btnList:@"Cancel"
		];

		return;
	}

//	NSLog(@"chapterCount=%d", chapterCount);

	//
	// remove
	//
	BOOL ret = YES;
	NSString *filePath = @"";
	NSString *message = @"";

	// remove folder
	if(chapterCount <= 0){
		ret = [self removeChapterFolder:uniqueId];
		filePath = [self getChapterFolderPath:uniqueId];;
		message = @"Failed deleting chapter folder.";
	}
	// remove image
	else{
		ret = [self removeChapterImage:uniqueId chapterId:chapterId];
		filePath = [self getChapterImagePath:uniqueId chapterId:chapterId];
		message = @"Failed deleting chapter image.";
	}

	if(ret == NO){
		[self displayMessage:@"alert"
					messageText:message
					infoText:[NSString stringWithFormat:@"Please check \"%@\"", filePath]
					btnList:@"Cancel"
		];
//		return;
	}

	// create chapter list
	[self createChapterList:[self params]];

	// post notification
	[self postItemChapterDidChangedNotification];

	// open chapter
	if([chapterWindow isVisible] == NO){
		[self openChapterView:NO];
	}

}
//------------------------------------
// showImageInFinder
//------------------------------------
- (IBAction)showImageInFinder:(id)sender
{

	id itemObject = [chapterMatrix selectedObject];
	if(itemObject == nil){
		return;
	}

	NSString *uniqueId = [itemObject valueForKey:@"uniqueId"];
	NSString *chapterId = [itemObject valueForKey:@"chapterId"];

	[self showFileInFinder:[self getChapterImagePath:uniqueId chapterId:chapterId]];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// openChapterView
//------------------------------------
- (void)openChapterView:(BOOL)isCreate
{
	// close gallery window
	if([viewGallery isVisibleWindow] == YES){
		[viewGallery closeGalleryWindow:nil];
	}

	[self setIsPlay:NO];

	// set roundedView attribute
	[roundedView setViewAttr:0.5 radius:10.0];

	if(isCreate == YES){
		[chapterMatrix changeMatrix:nil];
	}

	[chapterWindow open];
}
//------------------------------------
// playSelectedChapter
//------------------------------------
- (void)playSelectedChapter:(BOOL)isOpen isFront:(BOOL)isFront
{

	id itemObject = [self itemObject];

	// no item
	if(itemObject == nil){
		return;
	}

	int timeSec = [[itemObject valueForKey:@"timeSec"] intValue];

	if([self hasMovie] == YES){

		// open
		[viewPlayer openPlayerView:isOpen isFront:isFront];

		// play movie at timeSec
		[moviePlayerView setMovieCurrentTimeWithSecond:timeSec];
		[moviePlayerView setMoviePlay:YES isAction:NO];
	}

/*
	if([viewPlayer setPlayerView:[self params] timeSec:timeSec isPlay:YES isOpen:isOpen] == YES){
		// none
	}
*/
}
//------------------------------------
// setChapter
//------------------------------------
- (void)setChapter:(NSDictionary*)params
{

	[self createChapterList:params];

	[self setParams:params];

	NSString *title = [params valueForKey:@"title"];
	[txtTitle setStringValue:title];

	// create matrix
	[chapterMatrix setSelectedIndex:0];
	if([self isVisibleWindow] == YES){
		[chapterMatrix changeMatrix:nil];
	}

}

//------------------------------------
// createChapterList
//------------------------------------
- (void)createChapterList:(NSDictionary*)params
{

	// clear matrix
	[chapterMatrix clearMatrix];

	// remove chapterList
	[self removeChapterList];

	NSArray *chapterSets = nil;

	// get values
	NSString *uniqueId = [params valueForKey:@"uniqueId"];
	NSString *chapterSet = [params valueForKey:@"chapterSet"];
	int playTime = [[params valueForKey: @"playTime"] intValue];
	BOOL hasImage = [[params valueForKey: @"hasImage"] boolValue];

	// create chapter values
	if(chapterSet && ![chapterSet isEqualToString:@""]){
		chapterSets = [self createChapterSets:chapterSet];
	}

	NSString *chapterId;
	NSString *title;
	int index = -1;
	int timeSec;

	int i;
	id record;

	// add chapterList
	for(i = 0; i < [chapterSets count]; i++){

		record = [chapterSets objectAtIndex:i];

		index++;
		timeSec = [[record valueForKey:@"timeSec"] intValue];

		chapterId = [NSString stringWithFormat:@"%d", timeSec];
		title = [self convertTimeToString:timeSec];

		[chapterList_ addObject:
				[NSMutableDictionary dictionaryWithObjectsAndKeys:
					[NSNumber numberWithInt:index], @"index",
					uniqueId, @"uniqueId",
					chapterId, @"chapterId",
					title, @"title",
					[NSNumber numberWithInt:timeSec], @"timeSec",
					[NSNumber numberWithInt:playTime], @"playTime",
					[NSNumber numberWithBool:hasImage], @"hasImage",
					nil
				]
		];
	}

}

//------------------------------------
// removeChapterList
//------------------------------------
- (void)removeChapterList
{

	[chapterList_ removeAllObjects];

}
//------------------------------------
// getUpdatedIndex
//------------------------------------
- (int)getUpdatedIndex:(NSString*)chapterId
{
	int updatedIndex = 0;

	int i;
	id record;

	// add chapterList
	for(i = 0; i < [chapterList_ count]; i++){

		record = [chapterList_ objectAtIndex:i];

		if([chapterId isEqualToString:[record valueForKey:@"chapterId"]]){
			updatedIndex = i;
			break;
		}
	
	}

	return updatedIndex;

}
//------------------------------------
// createChapterAtCurrentTime
//------------------------------------
- (BOOL)createChapterAtCurrentTime:(QTMovie*)movie
								uniqueId:(NSString*)uniqueId
								chapterId:(NSString*)chapterId
								timeSec:(int)timeSec
								hasImage:(BOOL)hasImage
								record:(id)record
{

	// no movie
	if(movie == nil){
		return NO;
	}

	// no uniqueId
	if(uniqueId == nil || [uniqueId isEqualToString:@""]){
		return NO;
	}

	// no chapterId
	if(chapterId == nil || [chapterId isEqualToString:@""]){
		return NO;
	}

	//
	// update record
	//
	int chapterCount = 0;
	if([self updateChapterSet:chapterId record:record count:&chapterCount] == YES){

		//
		// create chapter image
		//
		if(hasImage == YES){
			if([self createChapterImageAtTime:movie
										uniqueId:uniqueId
										chapterId:chapterId
										timeSec:timeSec
										isCurrent:YES
										isCheckFolder:YES
										showAlert:YES
										
				] == NO
			){
				NSString *filePath = [self getChapterImagePath:uniqueId chapterId:chapterId];
				[self displayMessage:@"alert"
							messageText:@"Failed creating chapter image."
							infoText:[NSString stringWithFormat:@"Please check \"%@\"", filePath]
							btnList:@"Cancel"
				];
//				return NO;
			}
		}

	}
	else{

		[self displayMessage:@"alert"
					messageText:@"Can not create chapter."
					infoText:@"Failed updating item."
					btnList:@"Cancel"
		];

		return NO;
	}

	return YES;

}
//------------------------------------
// createChapterAtIntervalTime
//------------------------------------
- (BOOL)createChapterAtIntervalTime:(QTMovie*)movie
							uniqueId:(NSString*)uniqueId
							intervalSec:(int)intervalSec
							hasImage:(BOOL)hasImage
							record:(id)record
{

	// check and create chapter folder
	if([self checkAndCreateChapterFolder:uniqueId showAlert:YES] == NO){
		return NO;
	}

	// no movie
	if(movie == nil){
		return NO;
	}

	if(intervalSec <= 0){
		return NO;
	}

	// no uniqueId
	if(uniqueId == nil || [uniqueId isEqualToString:@""]){
		return NO;
	}

	// duration
	QTTime duration = [movie duration];
	int durationSec = duration.timeValue / duration.timeScale;

	int totalCount = durationSec / intervalSec + 1;
	int procCount = 0;
	int result = 0;

	// alert
	if(totalCount > CHAPER_ALERT_MAX_COUNT){
		result = [self displayMessage:@"alert"
							messageText:[NSString stringWithFormat: @"Create over %d (%d) chapters.\nContinue create?",
																		CHAPER_ALERT_MAX_COUNT,
																		totalCount
										]
							infoText:@"It needs much time for creating."
							btnList:@"Create,Cancel"
				];
		if(result != NSAlertFirstButtonReturn){
			return NO;
		}
	}

	// init proc status
	[viewProcStatus initProcStatus:@"Creating Chapter"
						targetName:@""
						procCount:procCount
						totalCount:totalCount
	];

	// create chapter
	NSString *title = [record valueForKey:@"title"];
	int timeSec = 0;
	int chapterCount = 0;
	int i;

	for (i = 0; i < totalCount; i++){

		timeSec = i * intervalSec;

		// update proc status
		procCount++;
		NSString *targetName = [NSString stringWithFormat:@"%@ - %@", [self convertTimeToString:timeSec], title];
		[viewProcStatus updateProcStatus:procCount targetName:targetName];

		NSString *chapterId = [NSString stringWithFormat:@"%d", timeSec];

		// add chapter
		if([self updateChapterSet:chapterId record:record count:&chapterCount] == YES){

			// create chapter image
			if(hasImage == YES){
				[self createChapterImageAtTime:movie
										uniqueId:uniqueId
										chapterId:chapterId
										timeSec:timeSec
										isCurrent:NO
										isCheckFolder:NO
										showAlert:NO
				];
			}
										
		}

	}

	// close proc status
	[viewProcStatus closeProcStatus:YES];

	return YES;

}
//------------------------------------
// createChapterImageAtTime
//------------------------------------
- (BOOL)createChapterImageAtTime:(QTMovie*)movie
						uniqueId:(NSString*)uniqueId
						chapterId:(NSString*)chapterId
						timeSec:(int)timeSec
						isCurrent:(BOOL)isCurrent
						isCheckFolder:(BOOL)isCheckFolder
						showAlert:(BOOL)showAlert
{
	BOOL ret = NO;

	NSImage *image = nil;

	// current frame
	if(isCurrent == YES){
		image = [[movie currentFrameImage] copy];
	}
	// at time
	else{
		QTTime duration = [movie duration];
		QTTime chapterTime;
		chapterTime.timeScale = duration.timeScale;
		chapterTime.timeValue = duration.timeScale * timeSec;
		image = [[movie frameImageAtTime:chapterTime] copy];
	}

	if(image != nil){
		// set original size
		NSSize baseSize = [[movie attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 
		[image setSize:baseSize];
		[image setScalesWhenResized:YES];

		ret = [self createChapterImage:image
							uniqueId:uniqueId
							chapterId:chapterId
							isCheckFolder:isCheckFolder
							showAlert:showAlert
				];

	}
	[image release];

	return ret;
}
//------------------------------------
// createChapterSets
//------------------------------------
- (NSArray*)createChapterSets:(NSString*)chapterSet
{

	int i;
	int timeSec;
	NSMutableArray *chapters = [[[NSMutableArray alloc] init] autorelease];

	NSArray *chapterSets = [chapterSet componentsSeparatedByString:@":"];

	// create array
	for(i = 0; i < [chapterSets count]; i++){

		// null
		if([[chapterSets objectAtIndex:i] isEqualToString:@""]){
			continue;
		}

		timeSec = [[chapterSets objectAtIndex:i] intValue];
		[chapters addObject:
				[NSMutableDictionary dictionaryWithObjectsAndKeys:
					[NSNumber numberWithInt:timeSec], @"timeSec",
					nil
				]
		];
	}

	// sort array
	NSSortDescriptor *descriptor = [[[NSSortDescriptor alloc] initWithKey: @"timeSec" ascending:YES] autorelease];
	return [chapters sortedArrayUsingDescriptors:[NSArray arrayWithObject:descriptor]];

}
///------------------------------------
// updateChapterSet
//------------------------------------
- (BOOL)updateChapterSet:(NSString*)chapterId record:(id)record count:(int*)count
{

	// update chapterSet
	NSMutableArray *chapters = [[[NSMutableArray alloc] init] autorelease];

	NSString *chapterSet = [record valueForKey:@"chapterSet"];

	if(chapterSet && ![chapterSet isEqualToString:@""]){
		[chapters addObjectsFromArray:[chapterSet componentsSeparatedByString:@":"]];
	}

	// add chapterId
	if(![chapters containsObject:chapterId]){
		[chapters addObject:chapterId];
		[record setValue:[chapters componentsJoinedByString:@":"] forKey:@"chapterSet"];
	}

	*count = [chapters count];

	return YES;
}
///------------------------------------
// removeChapterSet
//------------------------------------
- (BOOL)removeChapterSet:(NSString*)chapterId record:(id)record count:(int*)count
{

	// update chapterSet
	NSMutableArray *chapters = [[[NSMutableArray alloc] init] autorelease];

	NSString *chapterSet = [record valueForKey:@"chapterSet"];

	if(chapterSet && ![chapterSet isEqualToString:@""]){
		[chapters addObjectsFromArray:[chapterSet componentsSeparatedByString:@":"]];
	}

	// remove chapterId
	if([chapters containsObject:chapterId]){
		[chapters removeObject:chapterId];
		[record setValue:[chapters componentsJoinedByString:@":"] forKey:@"chapterSet"];
	}

	*count = [chapters count];

	return YES;
}
//------------------------------------
// handleWindowChapterDidClosed
//------------------------------------
- (void)handleWindowChapterDidClosed:(NSNotification *)notification
{

	// clear matrix
	[chapterMatrix clearMatrix];

	if([self defaultBoolValue:@"optChapterAutoClose"] == YES){
		// play item
		if([self isPlay] == YES){
			[self playSelectedChapter:YES isFront:NO];
		}
	}
	[self setIsPlay:NO];

	// remove chapterList
//	[self removeChapterList];

}
//------------------------------------
// handleWindowDialogDidOpened
//------------------------------------
- (void)handleWindowDialogDidOpened:(NSNotification *)notification
{
	// close
	[self closeChapterWindow:nil];
}
//------------------------------------
// handleWindowFullScreenDidClosed
//------------------------------------
- (void)handleWindowFullScreenDidClosed:(NSNotification *)notification
{
	// close
	[self closeChapterWindow:nil];
}
//------------------------------------
// postItemChapterDidChangedNotification
//------------------------------------
- (void)postItemChapterDidChangedNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_CHAPTER_DID_CHANGED object:nil];
}
//------------------------------------
// chapterList
//------------------------------------
- (NSMutableArray*)chapterList
{
	return chapterList_;
}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
	[params_ release];
	params_ = params;
}
- (NSDictionary*)params
{
	return params_;
}
//------------------------------------
// itemObject
//------------------------------------
- (void)setItemObject:(id)itemObject
{
	itemObject_ = itemObject;
}
- (id)itemObject
{
	return itemObject_;
}
//------------------------------------
// isPlay
//------------------------------------
- (void)setIsPlay:(BOOL)isPlay
{
	isPlay_ = isPlay;
}
- (BOOL)isPlay
{
	return isPlay_;
}
//------------------------------------
// isVisibleWindow
//------------------------------------
- (BOOL)isVisibleWindow
{
	return [chapterWindow isVisible];
}
//------------------------------------
// isNotVisibleWindow
//------------------------------------
- (BOOL)isNotVisibleWindow
{
	return ![self isVisibleWindow];
}
//------------------------------------
// isKeyWindow
//------------------------------------
- (BOOL)isKeyWindow
{
	return [chapterWindow isKeyWindow];
}
//------------------------------------
// movieDurationSec
//------------------------------------
- (int)movieDurationSec
{
	int durationSec = 0;
	QTMovie *movie = [moviePlayerView movie];
	if(movie != nil){
		QTTime duration = [movie duration];
		durationSec = duration.timeValue / duration.timeScale;
	}

	return durationSec;
}

//------------------------------------
// isSelectedItem
//------------------------------------
- (BOOL)isSelectedItem
{
	BOOL ret = NO;
	if([chapterMatrix selectedObject] != nil){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// canCreateChapter
//------------------------------------
- (BOOL)canCreateChapter
{
	BOOL ret = NO;
	if( [self hasMovie] == YES &&
		[self hasDuration] == YES &&
		[self isAvailable] == YES
	){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// canShowImage
//------------------------------------
- (BOOL)canShowImage
{
	BOOL ret = NO;
	id itemObject = [chapterMatrix selectedObject];
	if(itemObject != nil){
		NSString *uniqueId = [itemObject valueForKey:@"uniqueId"];
		NSString *chapterId = [itemObject valueForKey:@"chapterId"];
		NSString *filePath = [self getChapterImagePath:uniqueId chapterId:chapterId];
		if([self checkFileExist:filePath] != FILE_EXIST_NOT_FOUND){
			ret = YES;
		}
	}
	return ret;
}

//------------------------------------
// hasDuration
//------------------------------------
- (BOOL)hasDuration
{
	return [moviePlayerView hasDuration];
}
//------------------------------------
// hasMovie
//------------------------------------
- (BOOL)hasMovie
{
	return [moviePlayerView hasMovie];
}
//------------------------------------
// hasImage
//------------------------------------
- (BOOL)hasImage
{
	return [moviePlayerView hasImage];
}
//------------------------------------
// isAvailable
//------------------------------------
- (BOOL)isAvailable
{
	return [moviePlayerView isAvailable];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[chapterList_ release];
	[params_ release];
	[super dealloc];
}

@end
