#import "ViewGallery.h"
#import "ViewPlayer.h"
#import "ViewChapter.h"
#import "TBArrayController.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewGallery

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self setIsPlay:NO];
	[self setItemObject:nil];

	// set roundedView attribute
//	[roundedView setViewAttr:0.5 radius:20.0];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleWindowGalleryDidClosed:) name:WINDOW_NOTIF_GALLERY_DID_CLOSED object:nil];
	[nc addObserver:self selector:@selector(handleWindowDialogDidOpened:) name:WINDOW_NOTIF_DIALOG_DID_OPENED object:nil];
	[nc addObserver:self selector:@selector(handleWindowFullScreenDidClosed:) name:WINDOW_NOTIF_FULLSCREEN_DID_CLOSED object:nil];
	[nc addObserver:self selector:@selector(handleWindowFullScreenDidClosed:) name:WINDOW_NOTIF_FULLSCREEN_DID_CHANGED object:nil];

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openGalleryWindow
//------------------------------------
- (IBAction)openGalleryWindow:(id)sender
{
	// close chapter window
	if([viewChapter isVisibleWindow] == YES){
		[viewChapter closeChapterWindow:nil];
	}

	[self setIsPlay:NO];

	// set roundedView attribute
	[roundedView setViewAttr:0.5 radius:20.0];


	[galleryMatrix initMatrix:nil];
	[galleryWindow open];
}

//------------------------------------
// closeGalleryWindow
//------------------------------------
- (IBAction)closeGalleryWindow:(id)sender
{
	[galleryWindow close];
}
//------------------------------------
// changeGalleryWindow
//------------------------------------
- (IBAction)changeGalleryWindow:(id)sender
{
	if([galleryWindow isVisible] == NO){
		[self openGalleryWindow:nil];
	}else{
		[self closeGalleryWindow:nil];
	}
}
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{

	id itemObject = [galleryMatrix selectedObject];
	if(itemObject == nil){
/*
		[self displayMessage:@"alert"
					messageText:@"Can not play item."
					infoText:@"Item is not selected."
					btnList:@"Cancel"
		];
*/
		return;
	}

	// get filelist
	NSString *uniqueId = [itemObject valueForKey:@"uniqueId"];
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", uniqueId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];
	if([fetchArray count] <= 0){
		[self displayMessage:@"alert"
					messageText:@"Can not play item."
					infoText:@"Item is not found."
					btnList:@"Cancel"
		];
		return;
	}
	id record = [fetchArray objectAtIndex:0];

	[self setIsPlay:YES];
	[self setItemObject:record];

	// select index
	int selectedIndex = [galleryMatrix selectedIndex];
	int totalResults = [[filelistArrayController arrangedObjects] count];
	if(selectedIndex >= 0 && selectedIndex < totalResults){
		[filelistArrayController setSelectionIndex:selectedIndex];
	}

	if([self defaultBoolValue:@"optGalleryAutoClose"] == YES){
		// post notification when window is closed
		[galleryWindow close];
	}else{
		[self playSelectedItem:NO isFront:YES];
	}

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// playSelectedItem
//------------------------------------
- (void)playSelectedItem:(BOOL)isOpen isFront:(BOOL)isFront
{
	id itemObject = [self itemObject];

	// no item
	if(itemObject == nil){
		return;
	}

	// set params
	NSDictionary *params = itemObject;

	if([viewPlayer setPlayerView:params isPlay:YES isOpen:isOpen isFront:isFront] == YES){
		// none
	}
}
//------------------------------------
// handleWindowGalleryDidClosed
//------------------------------------
- (void)handleWindowGalleryDidClosed:(NSNotification *)notification
{

	// claer matrix
	[galleryMatrix clearMatrix];

	if([self defaultBoolValue:@"optGalleryAutoClose"] == YES){
		// play item
		if([self isPlay] == YES){
			[self playSelectedItem:YES isFront:NO];
		}
	}
	[self setIsPlay:NO];

}
//------------------------------------
// handleWindowDialogDidOpened
//------------------------------------
- (void)handleWindowDialogDidOpened:(NSNotification *)notification
{
	// close
	[self closeGalleryWindow:nil];
}
//------------------------------------
// handleWindowFullScreenDidClosed
//------------------------------------
- (void)handleWindowFullScreenDidClosed:(NSNotification *)notification
{
	// close
	[self closeGalleryWindow:nil];
}
//------------------------------------
// itemObject
//------------------------------------
- (void)setItemObject:(id)itemObject
{
	itemObject_ = itemObject;
}
- (id)itemObject
{
	return itemObject_;
}
//------------------------------------
// isPlay
//------------------------------------
- (void)setIsPlay:(BOOL)isPlay
{
	isPlay_ = isPlay;
}
- (BOOL)isPlay
{
	return isPlay_;
}
//------------------------------------
// isVisibleWindow
//------------------------------------
- (BOOL)isVisibleWindow
{
	return [galleryWindow isVisible];
}
//------------------------------------
// isNotVisibleWindow
//------------------------------------
- (BOOL)isNotVisibleWindow
{
	return ![self isVisibleWindow];
}
//------------------------------------
// isKeyWindow
//------------------------------------
- (BOOL)isKeyWindow
{
	return [galleryWindow isKeyWindow];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

@end
