#import "ViewItemInfo.h"
#import "TBArrayController.h"
#import "FileItemManager.h"
#import "FileAttributeExtension.h"
#import "ThumbsExtension.h"
#import "UserDefaultsExtension.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"

@implementation ViewItemInfo

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rext
	[self setWindowRect:infoWindow key:@"rectWindowInfo"];
	[self setWindowPosition:thumbsSettingWindow key:@"rectWindowThumbsSetting"];

	[self setTitle:@""];
	[self setFilePath:@""];
	[self setLabelColorNo:0];
	[self setUpdateKeys:nil];

	[self setCanEditTitle:NO];
	[self setCanEditFilePath:NO];
	[self setCanEditLabelColorNo:NO];

	[self setIsSingleSelected:YES];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillCloseInfo:) name:NSWindowWillCloseNotification object:infoWindow];
	[nc addObserver:self selector:@selector(windowWillCloseThumbsSetting:) name:NSWindowWillCloseNotification object:thumbsSettingWindow];
}

//------------------------------------
// openInfoWindow
//------------------------------------
- (IBAction)openInfoWindow:(id)sender
{
	if([self createItemInfo] == YES){
		[infoWindow makeKeyAndOrderFront:self];
	}
}
//------------------------------------
// openThumbsSettingWindow
//------------------------------------
- (IBAction)openThumbsSettingWindow:(id)sender
{
	if([self createThumbsSetting] == YES){
		[thumbsSettingWindow makeKeyAndOrderFront:self];
	}
}
//------------------------------------
// closeThumbsSettingWindow
//------------------------------------
- (IBAction)closeThumbsSettingWindow:(id)sender
{
	[thumbsSettingWindow close];
}
//------------------------------------
// changeItemInfo
//------------------------------------
- (IBAction)changeItemInfo:(id)sender
{

	// update
	if([self updateItemInfo] == YES){
		[infoWindow performClose:nil];
	}

}
//------------------------------------
// selectItemInfo
//------------------------------------
- (IBAction)selectItemInfo:(id)sender
{

	// update
	if([self updateItemInfo] == NO){
		return;
	}

	BOOL canSelect = NO;
	int tag = [sender tag];

	// previous
	if(tag == 0){
		if([filelistArrayController canSelectPrevious] == YES){
			[filelistArrayController selectPrevious:nil];
			canSelect = YES;
		}
	}
	// next
	else if(tag == 1){
		if([filelistArrayController canSelectNext] == YES){
			[filelistArrayController selectNext:nil];
			canSelect = YES;
		}
	}

	// create info
	if(canSelect == YES){
		SEL sel = NSSelectorFromString(@"createItemInfo");
		[self performSelector:sel withObject:nil afterDelay:0.1];
	}
}

//------------------------------------
// setItemFilePath
//------------------------------------
- (IBAction)setItemFilePath:(id)sender
{

	BOOL isFile = NO;
	BOOL isDir = NO;
	NSString *defaultPath = [txtFilePath stringValue];

	if([self isSingleSelected] == YES){
		isFile = YES;
		defaultPath = [defaultPath stringByDeletingLastPathComponent];
	}else{
		isDir = YES;
	}

	NSArray *filePaths = [self selectFilePathWithOpenPanel:isFile
														isDir:isDir
														isMultiSel:NO
														isPackage:NO
														isAlias:YES
														canCreateDir:NO
														defaultPath:defaultPath
							];

	if([filePaths count] <= 0){
		return;
	}

	[txtFilePath setStringValue:[filePaths objectAtIndex:0]];

}
//------------------------------------
// showItemInFinder
//------------------------------------
- (IBAction)showItemInFinder:(id)sender
{

	[self showFileInFinder:[self filePath]];

}
//------------------------------------
// showImageInFinder
//------------------------------------
- (IBAction)showImageInFinder:(id)sender
{

	NSArray *updateKeys = [self updateKeys];

	// no select
	if([updateKeys count] <= 0){
		return;
	}

	[self showFileInFinder:[self getThumbsImagePath:[updateKeys objectAtIndex:0]]];

}
//------------------------------------
// createItemThumbsImage
//------------------------------------
- (IBAction)createItemThumbsImage:(id)sender
{

	// close
	[self closeThumbsSettingWindow:nil];

	// get filelist
	NSPredicate *pred = [NSPredicate predicateWithFormat:@"uniqueId IN %@", [self updateKeys]];
	NSArray *items = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];

	int count = [items count];
	int result;

	// no select
	if(count <= 0){
		[self displayMessage:@"alert"
							messageText:@"Item is not found."
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	// alert
	if(count > THUMBS_ALERT_MAX_COUNT){
		result = [self displayMessage:@"alert"
							messageText:[NSString stringWithFormat: @"Creating over %d (%d) thumbnails.\nContinue create?",
																		THUMBS_ALERT_MAX_COUNT,
																		count
										]
							infoText:@"It needs much time for creating."
							btnList:@"Create,Cancel"
				];
		if(result != NSAlertFirstButtonReturn){
			return;
		}
	}

	[fileItemManager createItemThumbsImageWithItems:items];

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED object:nil];

	// create preview image
	if([items count] == 1){
		[self setPreviewImage:[items objectAtIndex:0]];
	}

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createItemInfo
//------------------------------------
- (BOOL)createItemInfo
{

	BOOL isSingleSelected = NO;
	BOOL isSameFolder = NO;
	BOOL isSameLabelColor = NO;
	
	BOOL canEditTitle = NO;
	BOOL canEditFilePath = NO;
	BOOL canEditLabelColorNo = NO;
	BOOL canEditItem = NO;

	BOOL canCreateImage = NO;
	BOOL canSearchImage = NO;
	BOOL canSearchItem = NO;

	NSString *fileInfoStr = @"";
	int fileInfoMaxCount = 50;
	id record;

	// get selected record
	NSArray *fetchedArray = [filelistArrayController selectedObjects];
	int count = [fetchedArray count];

	// no select
	if(count <= 0){
		[self displayMessage:@"alert"
									messageText:@"Item is not selected."
									infoText:@""
									btnList:@"Cancel"
									];
		return NO;
	}

	// selected multi files
	if(count == 1){
		isSingleSelected = YES;
	}else{
		isSingleSelected = NO;
	}

	// get file paths
	NSMutableArray *folderPaths = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *labelColorNos = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *coloredNos = [[[NSMutableArray alloc] init] autorelease];
	NSString *folderPath;
	NSString *fileName;
	NSNumber *colorNo;
	int fileCount = 0;

	NSEnumerator *enumArray = [fetchedArray objectEnumerator];
	while (record = [enumArray nextObject]) {

		fileCount++;

		// check folder path
		folderPath = [[record valueForKey:@"filePath"] stringByDeletingLastPathComponent];
 		if(folderPath && ![folderPath isEqualToString:@""]){
 			if(![folderPaths containsObject:folderPath]){
				[folderPaths addObject:folderPath];
			}
		}

		// check labelColorNo
		colorNo = [record valueForKey:@"labelColorNo"];
 		if(!colorNo){
			colorNo = [NSNumber numberWithInt:0];
		}
// 		if(colorNo && [colorNo intValue] > 0){
// 		if(colorNo){
 			if(![labelColorNos containsObject:colorNo]){
				[labelColorNos addObject:colorNo];
			}
//		}
		// colored
		if([colorNo intValue] > 0){
 			if(![coloredNos containsObject:colorNo]){
				[coloredNos addObject:colorNo];
			}
		}

		// append fileInfoStr
		if(fileCount <= fileInfoMaxCount){
			fileName = [record valueForKey:@"title"];
//			fileName = [[record valueForKey:@"filePath"] lastPathComponent];
			if(fileName){
				fileInfoStr = [fileInfoStr stringByAppendingFormat:@"%@\n", fileName];
			}
		}
	}

//	NSLog(@"labelColorNos=%@", [labelColorNos description]);
//	NSLog(@"coloredNos=%@", [coloredNos description]);

	// selected same folder
	if([folderPaths count] <= 1){
		isSameFolder = YES;
	}

	if([labelColorNos count] <= 1){
		isSameLabelColor = YES;
	}

	// set status
	record = [fetchedArray objectAtIndex:0];

	NSString *uniqueId = @"";
	NSString *title = @"No Selection";
	NSString *filePath = @"";
	NSString *fileInfo = @"";
	
	int labelColorNo = 0;
	int imageWidth = 0;
	int imageHeight = 0;
	
	NSImage *image = nil;
	NSImage *imageIcon = nil;

	NSString *windowTitle = @"Information";
	NSString *pathTitle = @"File Path :";
	NSString *infoTitle = @"";
//	NSString *infoThumbsSize = @"";
	NSString *infoLabelColor = @"";

	//
	// single selected
	//
	if(isSingleSelected == YES){
		uniqueId = [record valueForKey:@"uniqueId"];
		title = [record valueForKey:@"title"];
		filePath = [record valueForKey:@"filePath"];
		if([record valueForKey: @"labelColorNo"]){
			labelColorNo = [[record valueForKey: @"labelColorNo"] intValue];
		}
		int playTime = [[record valueForKey: @"playTime"] intValue];
		int fileSize = [[record valueForKey: @"fileSize"] intValue];
		if([record valueForKey: @"imageWidth"] && [record valueForKey: @"imageHeight"]){
			imageWidth = [[record valueForKey: @"imageWidth"] intValue];
			imageHeight = [[record valueForKey: @"imageHeight"] intValue];
		}
		NSDate *fileDate = [record valueForKey: @"fileDate"];
		BOOL hasImage = [[record valueForKey: @"hasImage"] boolValue];
		BOOL isSound = NO;

		NSString *playTimeStr = @"-";
		// convert play time
		if(playTime > 0){
			playTimeStr = [self convertTimeToString:playTime];
		}
		// fileSize
		NSString *fileSizeStr = @"-";
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}
		// imageSize
		NSString *imageSizeStr = @"-";
		if(imageWidth > 0 && imageHeight > 0){
			imageSizeStr = [NSString stringWithFormat:@"%d x %d", imageWidth, imageHeight];
		}
		// sound
		if(playTime > 0 && hasImage == NO){
			isSound = YES;
		}
		NSString *fileDateStr = @"-";
		if(fileDate != nil){
			fileDateStr = [self convertDateToString:fileDate formatString:@"%Y/%m/%d"];
		}
		
		// file info
		fileInfo = [NSString stringWithFormat:@"Time: %@\nVideo : %@\nSize: %@\nDate : %@\nFile Path:\n%@",
												playTimeStr,
												imageSizeStr,
												fileSizeStr,
												fileDateStr,
												filePath
					];

		// preview image
		image = [self getThumbsImage:filePath
									thumbsPath:[self getThumbsImagePath:uniqueId]
									isSound:isSound
				];

		// image size
//		NSSize imageSize = [image size];
//		if(!NSEqualSizes(imageSize, NSZeroSize)){
//			infoThumbsSize = [NSString stringWithFormat:@"%.0f x %.0f", imageSize.width, imageSize.height]; 
//		}

		// file icon
		if([self checkFileExist:filePath] == FILE_EXIST_FILE){
			imageIcon = [[NSWorkspace sharedWorkspace] iconForFile:filePath];
		}else{
			imageIcon =[NSImage imageNamed:@"icon_file_not_found"];
			[imageIcon setScalesWhenResized:YES];
			[imageIcon setSize:NSMakeSize(30,30)];
		}
//		windowTitle = title;
		pathTitle = @"File Path :";

		canEditTitle = YES;
		canEditFilePath = YES;
		canEditLabelColorNo = YES;
		canSearchItem = YES;

		if(hasImage == YES){
			canSearchImage = YES;
		}
	}
	//
	// multi selected
	//
	else{
//		windowTitle = [NSString stringWithFormat:@"Selected %d items", count];
		title = @"Multiple Selection";
		pathTitle = @"Folder Path :";
		infoTitle = [NSString stringWithFormat:@"Selected %d items", count];
		if(count <= fileInfoMaxCount){
			fileInfo = fileInfoStr;
		}else{
			fileInfo = @"Multiple Selection";
		}
		image = [NSImage imageNamed:@"image_multi_select"];
		imageIcon = [NSImage imageNamed:@"icon_folder"];

		// same folder path
		if(isSameFolder == YES && [folderPaths count] > 0){
			filePath = [folderPaths objectAtIndex:0];
			canEditFilePath = YES;
			canSearchItem = YES;
		}else{
			filePath = @"Multiple Selection";
		}

		// colored no
//		if(isSameLabelColor == YES && [labelColorNos count] > 0){
		if([coloredNos count] > 0){
			labelColorNo = [[coloredNos objectAtIndex:0] intValue];
		}

		// label colors
		if([labelColorNos count] > 1){
			infoLabelColor = @"Multiple Selection";
		}
		canEditLabelColorNo = YES;
	}

	canCreateImage = [self canCreateImage:fetchedArray];
	canEditItem = YES;

	// set values
	[self setUpdateKeys:[fetchedArray valueForKey:@"uniqueId"]];
	[self setTitle:title];
	[self setFilePath:filePath];
	[self setLabelColorNo:labelColorNo];

	// can edit
	[self setCanEditTitle:canEditTitle];
	[self setCanEditFilePath:canEditFilePath];
	[self setCanEditLabelColorNo:canEditLabelColorNo];

	[self setIsSingleSelected:isSingleSelected];

	// set status
	[infoWindow setTitle:windowTitle];

	[txtTitle setStringValue:title];
	[txtFilePath setStringValue:filePath];
	[txtFileInfo setString:fileInfo];

	[txtPathTitle setStringValue:pathTitle];
	[txtInfoTitle setStringValue:infoTitle];
	[txtInfoLabelColor setStringValue:infoLabelColor];

	[pbtnLabelColor selectItemWithTag:labelColorNo];
	
	[imagePreview setImage:image];
//	[imagePreview setToolTip:infoThumbsSize];
	[imageFileIcon setImage:imageIcon];

	// enable/disable
	[txtTitle setEnabled:canEditTitle];
	[txtFilePath setEnabled:canEditFilePath];
	[pbtnLabelColor setEnabled:canEditLabelColorNo];

	[btnPathSelect setEnabled:canEditFilePath];
	[btnUpdate setEnabled:canEditItem];

	[btnNext setEnabled:(isSingleSelected && [filelistArrayController canSelectNext])];
	[btnPrev setEnabled:(isSingleSelected && [filelistArrayController canSelectPrevious])];

	[btnCreateImage setEnabled:canCreateImage];
	[btnSearchImage setEnabled:canSearchImage];
	[btnSearchItem setEnabled:canSearchItem];

/*
	NSLog(@"canEditTitle=%d", canEditTitle);
	NSLog(@"canEditFilePath=%d", canEditFilePath);
	NSLog(@"canEditLabelColorNo=%d", canEditLabelColorNo);
	NSLog(@"canEditItem=%d", canEditItem);
*/

	// close thumbsInfo
	if([thumbsSettingWindow isVisible] == YES){
		[self closeThumbsSettingWindow:nil];
	}

	return YES;

}
//------------------------------------
// updateItemInfo
//------------------------------------
- (BOOL)updateItemInfo
{

	NSArray *updateKeys = [self updateKeys];

	BOOL canEditTitle = [self canEditTitle];
	BOOL canEditFilePath = [self canEditFilePath];
	BOOL canEditLabelColorNo = [self canEditLabelColorNo];

	BOOL isSingleSelected = [self isSingleSelected];

	int count = [updateKeys count];
	int result;

	// no select
	if(count <= 0){
		[self displayMessage:@"alert"
									messageText:@"Item is not selected."
									infoText:@""
									btnList:@"Cancel"
									];
		return NO;
	}

	// get values
	NSString *filePath = [txtFilePath stringValue];
	NSString *title = [txtTitle stringValue];
	int labelColorNo = [[pbtnLabelColor selectedItem] tag];
	BOOL isChangedTitle = NO;
	BOOL isChangedFilePath = NO;
	BOOL isChangedLabelColorNo = NO;

	if(canEditTitle == YES){
		// no title
		if([title isEqualToString:@""]){
			[self displayMessage:@"alert"
										messageText:@"Title is not entered."
										infoText:@"Enter title."
										btnList:@"Cancel"
										];
			return NO;
		}

	}

	// delete "/" at last
	if(canEditFilePath == YES){
		BOOL isDel = YES;
		while (isDel == YES && ![filePath isEqualToString:@""]){
			filePath = [self deleteLastString:filePath delStr:@"/" isDel:&isDel];
		}

		[txtFilePath setStringValue:filePath];
	}

	// check changed value
	if(canEditTitle == YES && ![title isEqualToString:[self title]]){
		isChangedTitle = YES;
	}
	if(canEditFilePath == YES && ![filePath isEqualToString:[self filePath]]){
		isChangedFilePath = YES;
	}
	if(canEditLabelColorNo == YES && labelColorNo != [self labelColorNo]){
		isChangedLabelColorNo = YES;
	}

	// no change
	if( isChangedTitle == NO &&
		isChangedFilePath == NO &&
		isChangedLabelColorNo == NO
	){
//		NSLog(@"no changed value");
		return YES;
	}

	//
	// check file path
	//
	if(isChangedFilePath == YES){
		NSString *pathTitle;
		if(isSingleSelected == YES){
			pathTitle = @"file";
		}else{
			pathTitle = @"folder";
		}
		NSString *message = @"";
		BOOL isErr = NO;

		// no file path
		if([filePath isEqualToString:@""]){
			message = [NSString stringWithFormat:@"%@ path is not entered.", [pathTitle capitalizedString]];
			isErr = YES;
		}

		// check file or folder
		int isExist = [self checkFileExist:filePath];
		// not found
		if(isExist == FILE_EXIST_NOT_FOUND){
			message = [NSString stringWithFormat:@"%@ path is not found.", [pathTitle capitalizedString]];
			isErr = YES;
		}
		// single select && not a file
		// multi select && not a folder
		else if((isSingleSelected == YES && isExist != FILE_EXIST_FILE) ||
				(isSingleSelected == NO && isExist != FILE_EXIST_DIR)
		){
			message = [NSString stringWithFormat:@"%@ path is not a %@.", [pathTitle capitalizedString], pathTitle];
			isErr = YES;
		}
		// not correct
		else if([[filePath substringToIndex:1] isEqualToString:@"."]){
			message = [NSString stringWithFormat:@"%@ path is not correct.", [pathTitle capitalizedString]];
			isErr = YES;
		}

		// error
		if(isErr == YES){
			[self displayMessage:@"alert"
										messageText:message
										infoText:[NSString stringWithFormat:@"Please enter %@ path correctly.", pathTitle]
										btnList:@"Cancel"
										];
			return NO;
		}
	}

	// confirm
	if([self defaultBoolValue:@"optShowAlertBeforeChange"] == YES){
		result = [self displayMessage:@"alert"
										messageText:@"Change item info?"
										infoText:@""
										btnList:@"Change,Cancel"
										];
		if(result != NSAlertFirstButtonReturn){
			return NO;
		}
	}

/*
	NSLog(@"isChangedTitle=%d", isChangedTitle);
	NSLog(@"isChangedFilePath=%d", isChangedFilePath);
	NSLog(@"isChangedLabelColorNo=%d", isChangedLabelColorNo);
*/
	// updateFilelist
	BOOL ret = [self updateFilelist:updateKeys
					title:(NSString*)title 
					isChangedTitle:(BOOL)isChangedTitle
					filePath:(NSString*)filePath
					isChangedFilePath:(BOOL)isChangedFilePath
					labelColorNo:(int)labelColorNo
					isChangedLabelColorNo:(BOOL)isChangedLabelColorNo
				];

	if(ret == YES){
		// post notification
		NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
		[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED object:nil];
	}

	return ret;
}
//------------------------------------
// updateFilelist
//------------------------------------
- (BOOL)updateFilelist:(NSArray*)updateKeys
					title:(NSString*)title 
					isChangedTitle:(BOOL)isChangedTitle
					filePath:(NSString*)filePath
					isChangedFilePath:(BOOL)isChangedFilePath
					labelColorNo:(int)labelColorNo
					isChangedLabelColorNo:(BOOL)isChangedLabelColorNo
{

	// fetch update records
	NSPredicate* pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"uniqueId IN %@", updateKeys];

	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];

	// no item
	if([fetchArray count] <= 0){
		[self displayMessage:@"alert"
									messageText:@"Item is not found."
									infoText:@""
									btnList:@"Cancel"
									];
		return NO;
	}

	id record;
	NSString *fileName;

	NSEnumerator *enumFetchArray = [fetchArray objectEnumerator];
    while (record = [enumFetchArray nextObject]) {

		// title
		if(isChangedTitle == YES){
			[record setValue:title forKey:@"title"];
		}
		// filePath
		if(isChangedFilePath == YES){
			if([self isSingleSelected] == YES){
				[record setValue:filePath forKey:@"filePath"];
			}else{
				// get file name
				fileName = [[record valueForKey:@"filePath"] lastPathComponent];
				// create filePath + fileName
				[record setValue:[NSString stringWithFormat:@"%@/%@", filePath, fileName] forKey:@"filePath"];
			}
		}
		// labelColorNo
		if(isChangedLabelColorNo == YES){
			[record setValue:[NSNumber numberWithInt:labelColorNo] forKey:@"labelColorNo"];
		}
	}

	return YES;

}
//------------------------------------
// createThumbsSetting
//------------------------------------
- (BOOL)createThumbsSetting
{

	NSPredicate *pred = [NSPredicate predicateWithFormat:@"uniqueId IN %@", [self updateKeys]];
	NSArray *items = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];

	int count = [items count];

	// no select
	if(count <= 0){
		[self displayMessage:@"alert"
							messageText:@"Item is not found."
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	NSString *infoThumbs = @"";
	if(count == 1){
		NSString *title = [[items objectAtIndex:0] valueForKey:@"title"];
		infoThumbs = [NSString stringWithFormat:@"Item : %@", title];
	}else{
		infoThumbs = [NSString stringWithFormat:@"Selected %d items", count];
	}

	[txtInfoThumbs setStringValue:infoThumbs];

	return YES;
}
//------------------------------------
// setPreviewImage
//------------------------------------
- (void)setPreviewImage:(id)record
{

	NSString *uniqueId = [record valueForKey: @"uniqueId"];
	NSString *filePath = [record valueForKey:@"filePath"];
	int playTime = [[record valueForKey: @"playTime"] intValue];
	BOOL hasImage = [[record valueForKey: @"hasImage"] boolValue];
	BOOL isSound = NO;

	// sound
	if(playTime > 0 && hasImage == NO){
		isSound = YES;
	}

	NSImage *image = [self getThumbsImage:filePath
									thumbsPath:[self getThumbsImagePath:uniqueId]
									isSound:isSound
					];

	[imagePreview setImage:image];

}
//------------------------------------
// canCreateImage
//------------------------------------
- (BOOL)canCreateImage:(NSArray*)items
{
	BOOL ret = NO;

	NSEnumerator *enumItems = [items objectEnumerator];
	id record;

	while (record = [enumItems nextObject]) {

		// file is exist && hasImage
		if( [self checkFileExist:[record valueForKey: @"filePath"]] == FILE_EXIST_FILE &&
			[[record valueForKey: @"hasImage"] boolValue] == YES
		){
			ret = YES;
			break;
		}
	}

	return ret;
}
//------------------------------------
// updateKeys
//------------------------------------
- (void)setUpdateKeys:(NSArray*)updateKeys
{
	[updateKeys retain];
    [updateKeys_ release];
    updateKeys_ = updateKeys;
}
- (NSArray*)updateKeys
{
    return updateKeys_;
}
//------------------------------------
// title
//------------------------------------
- (void)setTitle:(NSString*)title
{
	[title retain];
    [title_ release];
    title_ = title;
}
- (NSString*)title
{
    return title_;
}
//------------------------------------
// filePath
//------------------------------------
- (void)setFilePath:(NSString*)filePath
{
	[filePath retain];
    [filePath_ release];
    filePath_ = filePath;
}
- (NSString*)filePath
{
    return filePath_;
}
//------------------------------------
// labelColorNo
//------------------------------------
- (void)setLabelColorNo:(int)labelColorNo
{
    labelColorNo_ = labelColorNo;
}
- (int)labelColorNo
{
    return labelColorNo_;
}

//------------------------------------
// canEditTitle
//------------------------------------
- (void)setCanEditTitle:(BOOL)canEditTitle
{
    canEditTitle_ = canEditTitle;
}
- (BOOL)canEditTitle
{
    return canEditTitle_;
}
//------------------------------------
// canEditFilePath
//------------------------------------
- (void)setCanEditFilePath:(BOOL)canEditFilePath
{
    canEditFilePath_ = canEditFilePath;
}
- (BOOL)canEditFilePath
{
    return canEditFilePath_;
}
//------------------------------------
// canEditLabelColorNo
//------------------------------------
- (void)setCanEditLabelColorNo:(BOOL)canEditLabelColorNo
{
    canEditLabelColorNo_ = canEditLabelColorNo;
}
- (BOOL)canEditLabelColorNo
{
    return canEditLabelColorNo_;
}

//------------------------------------
// isSingleSelected
//------------------------------------
- (void)setIsSingleSelected:(BOOL)isSingleSelected
{
    isSingleSelected_ = isSingleSelected;
}
- (BOOL)isSingleSelected
{
    return isSingleSelected_;
}


//------------------------------------
// windowWillCloseInfo
//------------------------------------
-(void)windowWillCloseInfo:(NSNotification *)notification
{
	// close thumbsInfo
	if([thumbsSettingWindow isVisible] == YES){
		[self closeThumbsSettingWindow:nil];
	}

	// save window rect
	[self saveWindowRect:infoWindow key:@"rectWindowInfo"];

}
//------------------------------------
// windowWillCloseThumbsSetting
//------------------------------------
-(void)windowWillCloseThumbsSetting:(NSNotification *)notification
{
	// save window rect
	[self saveWindowRect:thumbsSettingWindow key:@"rectWindowThumbsSetting"];

}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[updateKeys_ release];
	[title_ release];
	[filePath_ release];

	[super dealloc];
}
@end
