#import "ViewMain.h"
#import "ViewPlayer.h"
#import "ViewPlaylist.h"
#import "TBArrayController.h"
#import "FileItemManager.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "ThumbsExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewMain

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:mainWindow key:@"rectWindowMain"];
	// split view rect
	[self setSplitViewRect:spvMain key:@"rectSplitViewMain"];
	[self setSplitViewRect:spvNavi key:@"rectSplitViewNavi"];
	[self setSplitViewRect:spvHead key:@"rectSplitViewHead"];

	[self setSearchString:@""];
	[self setSearchPred:nil];
	[self setPlistUniqueId:@""];
	[self setPlistIsFolder:NO];
	[self setPlistPred:nil];
	[self setItemsPred:nil];
	[self setItemsTitle:@""];
	[self setWindowTitle:[self defaultAppName]];

	// set first responder
	[mainWindow makeFirstResponder:txtSearchField];

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];

	[self changeTabViewMainResultIndex];
	[self changeTabViewMainSliderIndex];
	[self changeTabViewMainStatusIndex];
	[self changeTabViewMainButtonEnable];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(mainWindowWillClose:) name:NSWindowWillCloseNotification object:mainWindow];

	// set observer
	[filelistArrayController addObserver:self forKeyPath:@"selection" options:0 context:nil];
}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	[self playSelectedItem:YES isFront:NO];

}
//------------------------------------
// playItemWithTimer
//------------------------------------
- (IBAction)playItemWithTimer:(id)sender
{
	// no open player
	[self playSelectedItem:NO isFront:NO];
}
//------------------------------------
// searchMissingItems
//------------------------------------
- (IBAction)searchMissingItems:(id)sender
{
	int tag = [sender tag];

	NSMutableArray *items = [[[NSMutableArray alloc] init] autorelease];

	// show
	if(tag == TAG_SHOW_MISSING_ITEMS_YES){
		NSArray *fetchedArray = [tbArrayController getAllObjects:@"filelist"];

		NSEnumerator *enumArray = [fetchedArray objectEnumerator];
		id record;

		while (record = [enumArray nextObject]) {
			// not found
			if([self checkFileExist:[record valueForKey: @"filePath"]] == FILE_EXIST_NOT_FOUND){
				[items addObject:[record valueForKey: @"uniqueId"]];
			}
		}
	}

	[self searchWithItems:items title:@"Missing Items"];

}
//------------------------------------
// importItemWithSelect
//------------------------------------
- (IBAction)importItemWithSelect:(id)sender
{

	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
														isDir:YES
														isMultiSel:YES
														isPackage:NO
														isAlias:YES
														canCreateDir:NO
														defaultPath:@""
							];

	if([filePaths count] <= 0){
		return;
	}

	[self importItemWithFilePaths:filePaths];
}
//------------------------------------
// showItemInFinder
//------------------------------------
- (IBAction)showItemInFinder:(id)sender
{

	NSArray *items = [[filelistArrayController selectedObjects] valueForKey:@"filePath"];

	// no select
	if([items count] <= 0){
		return;
	}

	[self showFileInFinder:[items objectAtIndex:0]];

}
//------------------------------------
// showImageInFinder
//------------------------------------
- (IBAction)showImageInFinder:(id)sender
{

	NSString *uniqueId = [imagePreview uniqueId];

	if(!uniqueId){
		return;
	}

	[self showFileInFinder:[self getThumbsImagePath:uniqueId]];

}
//------------------------------------
// openItemWithApp
//------------------------------------
- (IBAction)openItemWithApp:(id)sender
{

	id record = [sender representedObject];

	NSArray *filePaths = [[filelistArrayController selectedObjects] valueForKey:@"filePath"];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	NSString *appPath = @"";
	if([record valueForKey:@"filePath"]){
		appPath = [record valueForKey:@"filePath"];
	}

	[self openFileWithApp:appPath filePaths:filePaths];

}
//------------------------------------
// createItemThumbsImage
//------------------------------------
- (IBAction)createItemThumbsImage:(id)sender
{

	NSArray *items = [filelistArrayController selectedObjects];
	int count = [items count];

	// no select
	if(count <= 0){
		return;
	}

	// alert
	if(count > THUMBS_ALERT_MAX_COUNT){
		int result = [self displayMessage:@"alert"
							messageText:[NSString stringWithFormat: @"Creating over %d (%d) thumbnails.\nContinue create?",
																		THUMBS_ALERT_MAX_COUNT,
																		count
										]
							infoText:@"It needs much time for creating."
							btnList:@"Create,Cancel"
				];
		if(result != NSAlertFirstButtonReturn){
			return;
		}

	}

	[fileItemManager createItemThumbsImageWithItems:items];

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED object:nil];
}
//------------------------------------
// addItemToPlaylist
//------------------------------------
- (IBAction)addItemToPlaylist:(id)sender
{

	NSManagedObject *targetItem = [sender representedObject];

	NSArray *items = [filelistArrayController selectedObjects];
 
	[fileItemManager addItemsToPlaylist:targetItem items:items];

}
//------------------------------------
// removeItem
//------------------------------------
- (IBAction)removeItem:(id)sender
{

	// remove from playlist
	if([self isSearchWithPlaylist] == YES){
		[self removeItemFromPlaylist:nil];
	}
	// remove from app
	else{
		[self removeItemFromLibrary:nil];
	}

}

//------------------------------------
// removeItemFromLibrary
//------------------------------------
- (IBAction)removeItemFromLibrary:(id)sender
{

	NSArray *items = [filelistArrayController selectedObjects];

	int count = [items count];
	// no select
	if(count <= 0){
		return;
	}

	NSString *message = @"";
	if(count == 1){
		message = [NSString stringWithFormat:@"Delete %d item from library?", count];
	}else{
		message = [NSString stringWithFormat:@"Delete %d items from library?", count];
	}

	int result = [self displayMessageWithIcon:@"icon_alert_item_delete"
										messageText:message
										infoText:@"Original file is not deleted."
										btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove itemlist from app
	[fileItemManager removeFilelistWithItems:items];

	// reload
	[filelistArrayController rearrangeObjects];	
//	[self searchWithPredicate:YES withPlist:YES withItems:YES];

}
//------------------------------------
// removeItemFromPlaylist
//------------------------------------
- (IBAction)removeItemFromPlaylist:(id)sender
{

	NSArray *items = [filelistArrayController selectedObjects];

	int count = [items count];
	// no select
	if(count <= 0){
		return;
	}

	// not result from playlist
	if([self isSearchWithPlaylist] == NO){
		return;
	}

	// get playllist
	NSPredicate *pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", [self plistUniqueId]];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not found
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Playlist is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	id selectedItem = [fetchedArray objectAtIndex:0];
	NSString *title = [selectedItem valueForKey:@"title"];

	NSString *message = @"";
	if(count == 1){
//		message = [NSString stringWithFormat:@"Delete %d item from Playlist?", count];
		message = [NSString stringWithFormat:@"Delete %d item from \"%@\"?", count, title];
	}else{
		message = [NSString stringWithFormat:@"Delete %d items from \"%@\"?", count, title];
	}

	// confirm
	int result = [self displayMessageWithIcon:@"icon_alert_playlist"
							messageText:message
							infoText:@"Delete from only playlist."
							btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove relation from playlist
	[fileItemManager removeItemsFromPlaylist:selectedItem items:items];

	// reload
	[filelistArrayController rearrangeObjects];	
//	[self searchWithPredicate:YES withPlist:YES withItems:YES];

}
//------------------------------------
// changePreviewImage
//------------------------------------
- (IBAction)changePreviewImage:(id)sender
{
	NSImage *image = nil;
	BOOL isExist = NO;
	NSString *uniqueId = @"";

	if([[filelistArrayController selectedObjects] count] > 0){
		id record = [[filelistArrayController selectedObjects] objectAtIndex:0];
		uniqueId = [record valueForKey:@"uniqueId"];
		NSString *filePath = [record valueForKey:@"filePath"];
		int playTime = [[record valueForKey: @"playTime"] intValue];
		BOOL hasImage = [[record valueForKey: @"hasImage"] boolValue];
		BOOL isSound = NO;
		// sound
		if(playTime > 0 && hasImage == NO){
			isSound = YES;
		}
		// isExist
		if([self checkFileExist:filePath] == FILE_EXIST_FILE){
			isExist = YES;
		}
		// get thumbs image
		image = [self getThumbsImage:filePath
							thumbsPath:[self getThumbsImagePath:uniqueId]
							isSound:isSound
				];
	}

	[imagePreview setImage:image];
	[imagePreview setIsExist:isExist];
	[imagePreview setUniqueId:uniqueId];
	
}
//------------------------------------
// changeTabViewMainIndex
//------------------------------------
- (IBAction)changeTabViewMainIndex:(id)sender
{
	int index = [sender tag];
	[self setDefaultIntValue:index key:@"optTabViewMainIndex"];
	[self changeTabViewMainResultIndex];
	[self changeTabViewMainSliderIndex];
	[self changeTabViewMainStatusIndex];
	[self changeTabViewMainButtonEnable];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// playSelectedItem
//------------------------------------
- (void)playSelectedItem:(BOOL)isOpen isFront:(BOOL)isFront
{
	// no select
	if([[filelistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[filelistArrayController selectedObjects] objectAtIndex:0];

	// set params
	NSDictionary *params = record;

	// open player
//	NSLog(@"isOpen=%d", isOpen);
//	if(isOpen == YES){
//		[viewPlayer openPlayerWindow:nil];
//	}

	if([viewPlayer setPlayerView:params isPlay:YES isOpen:isOpen isFront:isFront] == YES){
		// none
	}
}
//------------------------------------
// searchWithString
//------------------------------------
- (void)searchWithString:(NSString*)searchString pred:(NSPredicate*)pred
{

	[self setSearchString:searchString];
	[self setSearchPred:pred];

	// set pred
	[self searchWithPredicate:YES withPlist:YES withItems:YES];

}
//------------------------------------
// searchWithPlaylist
//------------------------------------
- (void)searchWithPlaylist:(NSString*)uniqueId title:(NSString*)title isFolder:(BOOL)isFolder pred:(NSPredicate*)pred
{

	[self setPlistUniqueId:uniqueId];
	[self setPlistIsFolder:isFolder];
	[self setPlistPred:pred];

	// set window title
	if([uniqueId isEqualToString:@""]){
		[self setWindowTitle:[self defaultAppName]];
	}else{
		[self setWindowTitle:title];
	}

	// set pred
	[self searchWithPredicate:YES withPlist:YES withItems:YES];

}
//------------------------------------
// searchWithItems
//------------------------------------
- (void)searchWithItems:(NSArray*)items title:(NSString*)title
{

	NSPredicate *pred = nil;

	if([items count] > 0){
		pred = [NSPredicate predicateWithFormat:@"uniqueId IN %@", items];
	}

	// set pred
	[self setItemsPred:pred];
	[self setItemsTitle:title];
	[self searchWithPredicate:YES withPlist:YES withItems:YES];

}
//------------------------------------
// searchWithPredicate
//------------------------------------
- (void)searchWithPredicate:(BOOL)withString withPlist:(BOOL)withPlist withItems:(BOOL)withItems
{

	NSMutableArray *predItems = [[[NSMutableArray alloc] init] autorelease];
	NSString *windowTitle = [self windowTitle];

	// clear pred
	if(withString == NO){
		[txtSearchField setStringValue:@""];
		[self setSearchString:@""];
		[self setSearchPred:nil];
	}
	if(withPlist == NO){
		[viewPlaylist deselectItem:nil];
		[self setPlistUniqueId:@""];
		[self setPlistPred:nil];
	}
	if(withItems == NO){
		[self setItemsPred:nil];
	}

	NSPredicate *pred = nil;
	NSPredicate *searchPred = [self searchPred];
	NSPredicate *plistPred = [self plistPred];
	NSPredicate *itemsPred = [self itemsPred];

	// add preds 
	if(searchPred != nil){
		[predItems addObject:searchPred];
//		[self setItemsPred:nil];
	}
	if(plistPred != nil){
		[predItems addObject:plistPred];
//		[self setItemsPred:nil];
	}
	// items pred
	if(itemsPred != nil){
		[predItems addObject:itemsPred];
		windowTitle = [windowTitle stringByAppendingFormat:@" [%@]", [self itemsTitle]];
	}

	// create pred
	if([predItems count] > 0){
		pred = [NSCompoundPredicate andPredicateWithSubpredicates:predItems];
	}

//	NSLog(@"pred=%@", [pred description]);

	// window title
	[mainWindow setTitle:windowTitle];

	// filter
	[self handleSearchStatusChanged:PROC_IND_STATUS_START];

	[filelistArrayController setFilterPredicate:pred];

	[self handleSearchStatusChanged:PROC_IND_STATUS_STOP];

}
//------------------------------------
// importItemWithFilePaths
//------------------------------------
- (void)importItemWithFilePaths:(NSArray*)filePaths
{

	NSString *plistUniqueId = [self plistUniqueId];
	id targetItem = nil;

	// get playlist
	NSPredicate *pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", plistUniqueId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];
	if([fetchArray count] > 0){
		targetItem = [fetchArray objectAtIndex:0];
	}

	// impor
	if([fileItemManager importItemWithFilePaths:filePaths
									targetItem:targetItem
									targetType:IMPORT_TARGET_FILELIST
		] == YES
	){
		// reload
		[self searchWithPredicate:YES withPlist:YES withItems:YES];
	}
}

//------------------------------------
// changeTabViewMainResultIndex
//------------------------------------
- (void)changeTabViewMainResultIndex
{
	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	[tabViewMainResult selectTabViewItemAtIndex:index];

}
//------------------------------------
// changeTabViewMainSliderIndex
//------------------------------------
- (void)changeTabViewMainSliderIndex
{
	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	[tabViewMainSlider selectTabViewItemAtIndex:index];

}
//------------------------------------
// changeTabViewMainStatusIndex
//------------------------------------
- (void)changeTabViewMainStatusIndex
{
	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	[tabViewMainStatus selectTabViewItemAtIndex:index];

}
//------------------------------------
// changeTabViewMainButtonEnable
//------------------------------------
- (void)changeTabViewMainButtonEnable
{

	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	NSString *btnTableName = @"btn_view_table_off";
	NSString *btnGrigName = @"btn_view_grid_off";
	BOOL tableEnabled = NO;
	BOOL gridEnabled = NO;

	if(index == 0){
		gridEnabled = YES;
		btnTableName = @"btn_view_table_on";
	}
	else if(index == 1){
		tableEnabled = YES;
		btnGrigName = @"btn_view_grid_on";
	}

	[btnTabViewMainTable setImage:[NSImage imageNamed:btnTableName]];
	[btnTabViewMainGrid setImage:[NSImage imageNamed:btnGrigName]];

}
//------------------------------------
// handleSearchStatusChanged
//------------------------------------
- (void)handleSearchStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// change review image
	if([keyPath isEqualToString:@"selection"]){
		[self changePreviewImage:nil];
	}
}

//------------------------------------
// mainWindowWillClose
//------------------------------------
-(void)mainWindowWillClose:(NSNotification *)notification
{

	// save state from user default
	// window rect
	[self saveWindowRect:mainWindow key:@"rectWindowMain"];

	// split view rect
	[self saveSplitViewRect:spvMain key:@"rectSplitViewMain"];
	[self saveSplitViewRect:spvNavi key:@"rectSplitViewNavi"];
	[self saveSplitViewRect:spvHead key:@"rectSplitViewHead"];

}
//----------------------------------------
// windowTitle
//----------------------------------------
- (void)setWindowTitle:(NSString*)windowTitle
{
	[windowTitle retain];
    [windowTitle_ release];
    windowTitle_ = windowTitle;
}
- (NSString*)windowTitle
{
	return windowTitle_;
}
//------------------------------------
// searchString
//------------------------------------
- (void)setSearchString:(NSString*)searchString
{
	[searchString retain];
    [searchString_ release];
    searchString_ = searchString;

}
- (NSString*)searchString
{
	return searchString_;
}
//------------------------------------
// searchPred
//------------------------------------
- (void)setSearchPred:(NSPredicate*)searchPred
{
	[searchPred retain];
    [searchPred_ release];
    searchPred_ = searchPred;

}
- (NSPredicate*)searchPred
{
	return searchPred_;
}
//------------------------------------
// plistUniqueId
//------------------------------------
- (void)setPlistUniqueId:(NSString*)plistUniqueId
{
	[plistUniqueId retain];
    [plistUniqueId_ release];
    plistUniqueId_ = plistUniqueId;

}
- (NSString*)plistUniqueId
{
	return plistUniqueId_;
}
//------------------------------------
// plistIsFolder
//------------------------------------
- (void)setPlistIsFolder:(BOOL)plistIsFolder
{
    plistIsFolder_ = plistIsFolder;

}
- (BOOL)plistIsFolder
{
	return plistIsFolder_;
}
//------------------------------------
// plistPred
//------------------------------------
- (void)setPlistPred:(NSPredicate*)plistPred
{
	[plistPred retain];
    [plistPred_ release];
    plistPred_ = plistPred;

}
- (NSPredicate*)plistPred
{
	return plistPred_;
}
//------------------------------------
// itemsPred
//------------------------------------
- (void)setItemsPred:(NSPredicate*)itemsPred
{
	[itemsPred retain];
    [itemsPred_ release];
    itemsPred_ = itemsPred;

}
- (NSPredicate*)itemsPred
{
	return itemsPred_;
}
//------------------------------------
// itemsTitle
//------------------------------------
- (void)setItemsTitle:(NSString*)itemsTitle
{
	[itemsTitle retain];
    [itemsTitle_ release];
    itemsTitle_ = itemsTitle;

}
- (NSString*)itemsTitle
{
	return itemsTitle_;
}
//------------------------------------
// canRemoveItemFromApp
//------------------------------------
- (BOOL)canRemoveItemFromApp
{
	if([self isKeyMainWindow] == YES){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// canRemoveItemFromPlaylist
//------------------------------------
- (BOOL)canRemoveItemFromPlaylist
{
	if([self isSearchWithPlaylist] == YES){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// isKeyMainWindow
//------------------------------------
- (BOOL)isKeyMainWindow
{
	return [mainWindow isKeyWindow];
}
//------------------------------------
//------------------------------------
// isSearchWithPlaylist
//------------------------------------
- (BOOL)isSearchWithPlaylist
{
	if(![[self plistUniqueId] isEqualToString:@""] &&
		[self plistIsFolder] == NO){
		return YES;
	}else{
		return NO;
	}

}

//------------------------------------
// hasImagePreview
//------------------------------------
- (BOOL)hasImagePreview
{
	if( [imagePreview image] != nil &&
		![[imagePreview uniqueId] isEqualToString:@""]
	){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[filelistArrayController removeObserver:self forKeyPath:@"selection"];

	[searchString_ release];
	[searchPred_ release];
	[plistUniqueId_ release];
	[plistPred_ release];
	[itemsPred_ release];
	[itemsTitle_ release];
	[windowTitle_ release];

	[super dealloc];
}
@end

