#import "ViewPlayer.h"
#import "ViewMain.h"
#import "ViewPrefs.h"
#import "ViewChapter.h"
#import "TBArrayController.h"
#import "ViewProcStatus.h"
#import "MovieAttributeExtension.h"
#import "FileAttributeExtension.h"
#import "ThumbsExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewPlayer

//=======================================================================
// awakeFromNib
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:playerWindow key:@"rectWindowPlayer"];

	// set binding array no
	[self setParams:nil];
	[self setSelectTimer:nil];
	[self setPlayTimer:nil];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handlePlayerWindowWillClose:) name:NSWindowWillCloseNotification object:playerWindow];
	[nc addObserver:self selector:@selector(handleVideoDidEnded:) name:VIDEO_NOTIF_PLAY_DID_ENDED object:nil];
	[nc addObserver:self selector:@selector(handleVideoPlayDidChanged:) name:VIDEO_NOTIF_PLAY_DID_CHANGED object:nil];

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openPlayerWindow
//------------------------------------
- (IBAction)openPlayerWindow:(id)sender
{
	[playerWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// orderFrontPlayerWindow
//------------------------------------
- (IBAction)orderFrontPlayerWindow:(id)sender
{
	[playerWindow orderFront:self];

}
//------------------------------------
// controlMoviePlay
//------------------------------------
- (IBAction)controlMoviePlay:(id)sender
{

	BOOL isPlaying = [moviePlayerView isPlaying];

	// clear timer
	[self clearPlayTimer];
	[self clearSelectTimer];

	[moviePlayerView setMoviePlay:!isPlaying isAction:YES];

	// open player
	if([playerWindow isMainWindow] == NO){
		if([moviePlayerView isPlaying] == YES){
			[self openPlayerWindow:nil];
		}
	}

}

//------------------------------------
// changePlayItem
//------------------------------------
- (IBAction)changePlayItem:(id)sender
{
	[self selectPlayItem:[sender tag] interval:0.7 withTimer:NO isLoop:NO];
}
//------------------------------------
// seekMovieTime
//------------------------------------
- (IBAction)seekMovieTime:(id)sender;
{

	NSTimeInterval value = [sender floatValue];

	[moviePlayerView setMovieCurrentTime:value];

}
//------------------------------------
// skipMovieTime
//------------------------------------
- (IBAction)skipMovieTime:(id)sender
{

	int tag = [sender tag];

	if(tag == VIDEO_SKIP_BACK){
		[moviePlayerView skipMovieTime:([self defaultSkipInterval] * -1)];
	}
	else if(tag == VIDEO_SKIP_NEXT){
		[moviePlayerView skipMovieTime:[self defaultSkipInterval]];
	}

}
//------------------------------------
// gotoMovieTime
//------------------------------------
- (IBAction)gotoMovieTime:(id)sender
{
	int tag = [sender tag];

	if(tag == VIDEO_GO_TO_START){
		[moviePlayerView gotoMovieTime:VIDEO_GO_TO_START];
	}
	else if(tag == VIDEO_GO_TO_END){
		[moviePlayerView gotoMovieTime:VIDEO_GO_TO_END];
	}

}
//------------------------------------
// createItemThumbsImage
//------------------------------------
- (IBAction)createItemThumbsImage:(id)sender
{

	if([self hasImage] == NO){
		return;
	}

	NSDictionary *params = [self params];
	NSString *uniqueId = [params valueForKey:@"uniqueId"];
	NSString *title = [params valueForKey:@"title"];

	if(!uniqueId || [uniqueId isEqualToString:@""]){
		[self displayMessage:@"alert"
					messageText:@"Can not create thumbnail image."
					infoText:@"UniqueId is not defined."
					btnList:@"Cancel"
		];
		return;
	}

	// get filelist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", uniqueId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"filelist" pred:pred];
	if([fetchArray count] <= 0){
		[self displayMessage:@"alert"
					messageText:@"Can not create thumbnail image."
					infoText:@"Item is not found."
					btnList:@"Cancel"
		];
		return;
	}

	// check and create thumbs folder
	if([self checkAndCreateThumbsFolder:YES] == NO){
		return;
	}

	QTMovie *movie = [moviePlayerView movie];
	if(movie == nil){
		return;
	}

	// start proc notification
	[self postVideoProcDidChangedNotification:YES];

	// process status
	[viewProcStatus initProcStatus:@"Creating Thumbnail"
						targetName:title
						procCount:0
						totalCount:1
	];

	// get image
	BOOL ret = YES;
	NSString *message = @"";
	NSString *info = @"";
	NSImage *image = [[movie currentFrameImage] copy];

	if(image != nil){

		// set original size
		NSSize baseSize = [[movie attributeForKey:QTMovieNaturalSizeAttribute] sizeValue]; 
		[image setSize:baseSize];
		[image setScalesWhenResized:YES];

		ret = [self createThumbsImage:image
								uniqueId:uniqueId
								isCheckFolder:NO
								showAlert:NO
				];
		if(ret == NO){
			message = @"Failed creating thumbnail image.";
		}
	}else{
		message = @"Can not create movie frame image.";
		ret = NO;
	}

	[image release];

	// update process status
	[viewProcStatus updateProcStatus:1 targetName:title];
	// close process window
	[viewProcStatus closeProcStatus:YES];

	// stop proc notification
	[self postVideoProcDidChangedNotification:NO];

	// success
	if(ret == YES){
		// post notification
		NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
		[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED object:nil];
	}
	// error
	else{
		[self displayMessage:@"alert"
					messageText:message
					infoText:info
					btnList:@"Cancel"
		];
	}

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// setPlayerView
//------------------------------------
- (BOOL)setPlayerView:(NSDictionary*)params
						isPlay:(BOOL)isPlay
						isOpen:(BOOL)isOpen
						isFront:(BOOL)isFront
{

	// open with default app
	if([self defaultBoolValue:@"optDefaultAppPlayer"] == YES){
		NSString *filePath = [params valueForKey:@"filePath"];
		[self openFileWithDefaultPlayer:filePath];
		return YES;
	}

	// open
	[self openPlayerView:isOpen isFront:isFront];

	// same movie
	NSDictionary *playerParams = [self params];
	if(playerParams != nil){
		NSString *uniqueId1 = [params valueForKey:@"uniqueId"];
		NSString *uniqueId2 = [playerParams valueForKey:@"uniqueId"];
		if([uniqueId1 isEqualToString:uniqueId2]){
			[moviePlayerView setMoviePlay:isPlay isAction:NO];
			return YES;
		}
	}

	BOOL ret = NO;

	// set movie view
	if([moviePlayerView setPlayerView:params isPlay:isPlay] == YES){
		[self setParams:params];
		[viewChapter setChapter:params];
		ret = YES;
	}
	// error
	else{
//		NSString *errorDescription = [moviePlayerView errorDescription];
//		[self displayMessageAlertOpenMovie:errorDescription];
		ret = NO;
	}

	return ret;

}
//------------------------------------
// openPlayerView
//------------------------------------
- (void)openPlayerView:(BOOL)isOpen isFront:(BOOL)isFront
{

	if(isOpen == YES){
		[self openPlayerWindow:nil];
	}else{
		if(isFront == YES){
			[self orderFrontPlayerWindow:nil];
		}else{
//			if([self isVisiblePlayerWindow] == NO){
//				[self openPlayerWindow:nil];
//			}
		}
	}

}
//------------------------------------
// openFileWithDefaultPlayer
//------------------------------------
- (BOOL)openFileWithDefaultPlayer:(NSString*)filePath
{

	// check file
	if([self checkFileExist:filePath] == FILE_EXIST_NOT_FOUND){
		[self displayMessageAlertFileIsNotFound:filePath];
		return NO;
	}

	// check app
	BOOL isError = NO;
	int result;
	NSString *appPath = [self defaultStringValue:@"optDefaultAppPlayerPath"];
	if([appPath isEqualToString:@""]){
		result = [self displayMessage:@"alert"
							messageText:@"Default Player is not selected."
							infoText:@"Please select player in Preferences."
							btnList:@"Cancel,Open Preferences"
				];
		
		isError = YES;
	}

	// not found
	if(isError == NO){
		if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
			result = [self displayMessage:@"alert"
								messageText:@"Default Player is not found."
								infoText:[NSString stringWithFormat:@"Check file \"%@\"", appPath]
								btnList:@"Cancel,Open Preferences"
					];
			isError = YES;
		}
	}

	// not app
	if(isError == NO){
		if(![[[appPath pathExtension] lowercaseString] isEqualToString:@"app"]){
			result = [self displayMessage:@"alert"
								messageText:@"Default Player is not application."
								infoText:[NSString stringWithFormat:@"Check file \"%@\"", appPath]
								btnList:@"Cancel,Open Preferences"
					];
			isError = YES;
		}
	}
	// error 
	if(isError == YES){
		// open prefs
		if(result == NSAlertSecondButtonReturn){
			[viewPrefs openTabViewWithIdentifier:@"helper"];
		}
		return NO;
	}else{
		// open file
		[self openFileWithApp:appPath filePaths:[NSArray arrayWithObject:filePath]];
		return YES;
	}

}
//------------------------------------
// playItemWithTimer
//------------------------------------
- (void)playItemWithTimer:(id)sender
{

	[self selectPlayItem:1 interval:0.0 withTimer:YES isLoop:YES];

}
//------------------------------------
// playRepeatItemWithTimer
//------------------------------------
- (void)playRepeatItemWithTimer:(id)sender
{

	[moviePlayerView replayMovie];

}
//------------------------------------
// selectPlayItem
//------------------------------------
- (void)selectPlayItem:(int)tag
				interval:(float)interval
				withTimer:(BOOL)withTimer
				isLoop:(BOOL)isLoop
{
	// no select
//	if([[filelistArrayController selectedObjects] count] <= 0){
//		return;
//	}

	// clear timer
	[self clearPlayTimer];
	[self clearSelectTimer];

	BOOL canSelect = NO;

	int count = [[filelistArrayController arrangedObjects] count];

	// previous
	if(tag == 0){
		if([filelistArrayController canSelectPrevious] == YES){
			[filelistArrayController selectPrevious:nil];
			canSelect = YES;
		}else{
			// go to last
			if(isLoop == YES && count > 0){
				[filelistArrayController setSelectionIndex:count - 1];
				canSelect = YES;
			}
		}

	}
	// next
	else if(tag == 1){
		if([filelistArrayController canSelectNext] == YES){
			[filelistArrayController selectNext:nil];
			canSelect = YES;
		}else{
			// go to top
			if(isLoop == YES && count > 0){
				[filelistArrayController setSelectionIndex:0];
				canSelect = YES;
			}
		}
	}

	// select with timer
	if(canSelect == YES){
		// params
		NSDictionary *params =[NSDictionary dictionaryWithObjectsAndKeys:
								[NSNumber numberWithBool:withTimer], @"withTimer",
								nil
							];
		[self setSelectTimer:[NSTimer scheduledTimerWithTimeInterval:interval
								target:self 
								selector:@selector(selectItemWithTimer:)
								userInfo:params
								repeats: NO]
		];
		[[NSRunLoop currentRunLoop] addTimer:[self selectTimer] forMode:(NSString*)kCFRunLoopCommonModes];
	}

}
//------------------------------------
// selectItemWithTimer
//------------------------------------
- (void)selectItemWithTimer:(NSTimer*)aTimer
{

	NSDictionary *params = [aTimer userInfo];
	BOOL withTimer = [[params valueForKey:@"withTimer"] boolValue];

	if(withTimer == YES){
		[viewMain playItemWithTimer:nil];
	}else{
		[viewMain playItem:nil];
	}

}
//------------------------------------
// handleVideoDidEnded
//------------------------------------
- (void)handleVideoDidEnded:(NSNotification *)notification
{
	// current is no playing
	if([moviePlayerView isCurrentPlaying] == NO){
		return;
	}

	// clear timer
	[self clearPlayTimer];
	[self clearSelectTimer];

	int playRepeat = [self defaultPlayRepeat];
	float interval = [self defaultPlayInterval];

	// next item
	if(playRepeat == VIDEO_PLAY_REPEAT_ALL){
		// set timer
		[self setPlayTimer:[NSTimer scheduledTimerWithTimeInterval:interval
								target:self 
								selector:@selector(playItemWithTimer:)
								userInfo:nil
								repeats: NO]
		];
		[[NSRunLoop currentRunLoop] addTimer:[self playTimer] forMode:(NSString*)kCFRunLoopCommonModes];
	}
	// replay
	else if(playRepeat == VIDEO_PLAY_REPEAT_ONE){
		[self setPlayTimer:[NSTimer scheduledTimerWithTimeInterval:interval
								target:self 
								selector:@selector(playRepeatItemWithTimer:)
								userInfo:nil
								repeats: NO]
		];
		[[NSRunLoop currentRunLoop] addTimer:[self playTimer] forMode:(NSString*)kCFRunLoopCommonModes];
	}

}
//------------------------------------
// handleVideoPlayDidChanged
//------------------------------------
- (void)handleVideoPlayDidChanged:(NSNotification *)notification
{
	// clear timer
	[self clearPlayTimer];
	[self clearSelectTimer];

}
//------------------------------------
// handlePlayerWindowWillClose
//------------------------------------
-(void)handlePlayerWindowWillClose:(NSNotification *)notification
{

	[moviePlayerView deletePlayerView];

	// window rect
	[self saveWindowRect:playerWindow key:@"rectWindowPlayer"];

}
//------------------------------------
// playTimer
//------------------------------------
- (void)setPlayTimer:(NSTimer*)playTimer
{
	[playTimer retain];
	[playTimer_ release];
	playTimer_ = playTimer;
}
- (void)clearPlayTimer
{
	if([[self playTimer] isValid] == YES){
		[[self playTimer] invalidate];
	}
}
- (NSTimer*)playTimer
{
	return playTimer_;
}
//------------------------------------
// selectTimer
//------------------------------------
- (void)setSelectTimer:(NSTimer*)selectTimer
{
	[selectTimer retain];
	[selectTimer_ release];
	selectTimer_ = selectTimer;
}
- (void)clearSelectTimer
{
	if([[self selectTimer] isValid] == YES){
		[[self selectTimer] invalidate];
	}
}
- (NSTimer*)selectTimer
{
	return selectTimer_;
}
//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
	[params_ release];
	params_ = params;
}
- (NSDictionary*)params
{
	return params_;
}
//------------------------------------
// isMainPlayerWindow
//------------------------------------
- (BOOL)isMainPlayerWindow
{
	BOOL ret = NO;

	// main window
	if([playerWindow isMainWindow] == YES){
		ret = YES;
	}
	// visible && topScreen
	else{
		if( [self isVisiblePlayerWindow] == YES &&
			[self isTopScreenPlayerWindow] == YES){
			ret = YES;
		}
	}

	return ret;
}
//------------------------------------
// isVisiblePlayerWindow
//------------------------------------
- (BOOL)isVisiblePlayerWindow
{
	return [playerWindow isVisible];
}
//------------------------------------
// isFullScreenPlayerWindow
//------------------------------------
- (BOOL)isFullScreenPlayerWindow
{
	return [playerWindow isFullScreen];
}
//------------------------------------
// isTopScreenPlayerWindow
//------------------------------------
- (BOOL)isTopScreenPlayerWindow
{
	return [playerWindow isTopScreen];
}
//------------------------------------
// canCreateImage
//------------------------------------
- (BOOL)canCreateImage
{
	BOOL ret = NO;
	if( [self hasMovie] == YES &&
		[self hasImage] == YES &&
		[self isAvailable] == YES
	){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// canChangeImageScale
//------------------------------------
- (BOOL)canChangeImageScale
{
	BOOL ret = NO;
	if( [self hasMovie] == YES &&
		[self isMainPlayerWindow] == YES
	){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// canChangeScaleToFit
//------------------------------------
- (BOOL)canChangeScaleToFit
{
	return [self canCreateImage];
}
//------------------------------------
// isAvailable
//------------------------------------
- (BOOL)isAvailable
{
	return [moviePlayerView isAvailable];
}
//------------------------------------
// isPlaying
//------------------------------------
- (BOOL)isPlaying
{
	return [moviePlayerView isPlaying];
}
//------------------------------------
// isContinuePlaying
//------------------------------------
- (BOOL)isContinuePlaying
{
	return [moviePlayerView isContinuePlaying];
}
//------------------------------------
// hasDuration
//------------------------------------
- (BOOL)hasDuration
{
	return [moviePlayerView hasDuration];
}
//------------------------------------
// hasMovie
//------------------------------------
- (BOOL)hasMovie
{
	return [moviePlayerView hasMovie];
}
//------------------------------------
// hasImage
//------------------------------------
- (BOOL)hasImage
{
	return [moviePlayerView hasImage];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[self clearSelectTimer];
	[selectTimer_ release];
	[self clearPlayTimer];
	[playTimer_ release];
	[params_ release];
    [super dealloc];
}

@end
