#import "ViewPlaylist.h"
#import "ViewMain.h"
#import "ViewPlaylist.h"
#import "FileItemManager.h"
#import "TBArrayController.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation ViewPlaylist

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	previousUniqueId_ = @"";

	// set display setting
	[self setControlButtonEnable];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(frameDidResize:) name:NSViewFrameDidChangeNotification object:[olvPlaylist enclosingScrollView]];


}
//=======================================================================
// action methods
//=======================================================================
//------------------------------------
// addPlaylist
//------------------------------------
- (IBAction)addPlaylist:(id)sender
{

	// add item
	[self addItem:@"New Playlist"
				isFolder:NO
				isSelect:YES
				isEdit:YES
	];

}

//------------------------------------
// addFolder
//------------------------------------	  
- (IBAction)addFolder:(id)sender
{

	// add folder item
	[self addItem:@"New Folder"
			isFolder:YES
			isSelect:YES
			isEdit:YES
	];

}
//------------------------------------
// searchItem
//------------------------------------
- (IBAction)searchItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];
	NSString *uniqueId = @"";

	// selected
	if(selectedItem != nil){
		uniqueId = [selectedItem valueForKey:@"uniqueId"];
		// same playlist
		if([uniqueId isEqualToString:previousUniqueId_]){
			return;
		}
	}

	// search
	[self searchItemWithItem:selectedItem];

}
//------------------------------------
// searchWithDeselectItem
//------------------------------------
- (IBAction)searchWithDeselectItem:(id)sender
{
	// deselect
	[self deselectItem:nil];
	// search
	[self searchItem:nil];
}
//------------------------------------
// removeItem
//------------------------------------	  
- (IBAction)removeItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];
	// no select
	if(!selectedItem){
		return;
	}

	BOOL isFolder = [[selectedItem valueForKey:@"isFolder"] boolValue];
	int result;

	// Alert before remove
	// folder
	if(isFolder == YES){
		result = [self displayMessageWithIcon:@"icon_alert_folder"
											messageText:@"Delete Folder?"
											infoText:@"Delete all playlists in folder."
											btnList:@"Delete,Cancel"
											];
	}
	// playlist
	else{
		result = [self displayMessageWithIcon:@"icon_alert_playlist"
											messageText:@"Delete Playlist?"
											infoText:@""
											btnList:@"Delete,Cancel"
											];
	}

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove tree of selectedItem
	[tbArrayController removeTreeItems:selectedItem];

	// reload
	[playlistTreeController rearrangeObjects];

	// searchWithDeselectItem
	[self searchWithDeselectItem:nil];

	// deselect
//	[self deselectItem:nil];
	// search
//	[self searchItem:nil];

}
//------------------------------------
// showItem
//------------------------------------
- (IBAction)showItem:(id)sender
{
	// get object from representedObject
	NSManagedObject *targetItem = [sender representedObject];
	// deselect
	if(targetItem == nil){
		// searchWithDeselectItem
		[self searchWithDeselectItem:nil];
//		[self deselectItem:nil];
//		[self searchItem:nil];
		return;
	}

	id parentItem = [targetItem valueForKey:@"parent"];
	NSMutableArray *parentItems = [[NSMutableArray alloc] init];

	// add parentItems until parent is null
	while(parentItem){
		[parentItems addObject:parentItem];
		parentItem = [parentItem valueForKey:@"parent"];
	}

	int i;
	int numberOfRows = [olvPlaylist numberOfRows];

	for(i = 0; i < numberOfRows; i++){
		id item = [olvPlaylist itemAtRow:i];
		id columnItem = [item observedObject];
		// folder
		if([[columnItem valueForKey:@"isFolder"] boolValue] == YES){
			// expand
			if([parentItems containsObject:columnItem]){
				[olvPlaylist expandItem:item expandChildren:NO];
				numberOfRows = [olvPlaylist numberOfRows];
			}
		}
		// select row
		if(targetItem == columnItem){
			[olvPlaylist selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
			// search
			[self searchItemWithItem:columnItem];
			break;
		}

	}

	[parentItems release];

}
//------------------------------------
// editItem
//------------------------------------
- (IBAction)editItem:(id)sender
{

	[olvPlaylist editSelectedItem];

}
//------------------------------------
// deselectItem
//------------------------------------
- (IBAction)deselectItem:(id)sender
{
	[olvPlaylist deselectItem];
	previousUniqueId_ = @"";
}

//------------------------------------
// nullAction
//------------------------------------
- (IBAction)nullAction:(id)sender
{
	// null action
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// addItem
//------------------------------------
- (void)addItem:(NSString*)title
		isFolder:(BOOL)isFolder
		isSelect:(BOOL)isSelect
		isEdit:(BOOL)isEdit
{


	// get selected item
	id selectedRowItem = [olvPlaylist selectedRowObject];
	NSManagedObject *selectedItem = nil;
	if(selectedRowItem){
		selectedItem = [olvPlaylist selectedObservedObject];
	}

	// create item
	id item = [tbArrayController insertPlaylist:0
									title:title
									parentItem:nil
									isFolder:isFolder
				];

	// add item to patent of selected item
	[tbArrayController addItemToParent:selectedItem item:item maxNum:YES];			

	// reload
	[playlistTreeController rearrangeObjects];

	// expand selectedRowItem
	if(selectedItem){
		if([[selectedItem valueForKey:@"isFolder"] boolValue] ==YES){
			[olvPlaylist expandItem:selectedRowItem expandChildren:YES];
		}
	}

	// select add item
	if(isSelect == YES){
		[olvPlaylist selectItem:item];
	}

	// edit selected row
	if(isEdit == YES){
		[olvPlaylist editSelectedItem];
	}

}
//------------------------------------
// searchItemWithItem
//------------------------------------
- (void)searchItemWithItem:(NSManagedObject*)item
{

	NSPredicate *pred = nil;
	NSString *uniqueId = @"";
	NSString *title = @"";
	BOOL isFolder = NO;

	if(item != nil){
		uniqueId = [item valueForKey:@"uniqueId"];
		title = [item valueForKey:@"title"];
		isFolder = [[item valueForKey:@"isFolder"] boolValue];

		NSSet *uniqueIdSet = [NSSet setWithArray:[[tbArrayController getTreeItems:item] valueForKey:@"uniqueId"]];
		pred = [NSPredicate predicateWithFormat:@"ANY rlt_playlist.uniqueId IN %@", uniqueIdSet];
	}

	previousUniqueId_ = uniqueId;

	[viewMain searchWithPlaylist:uniqueId title:title isFolder:isFolder pred:pred];

}
//------------------------------------
// addItemToPlaylistWithParams
//------------------------------------
- (void)addItemToPlaylistWithParams:(NSDictionary*)params
{

	id targetItem = [params valueForKey:@"targetItem"];
	NSArray *items = [params valueForKey:@"items"];

	[fileItemManager addItemsToPlaylist:targetItem items:items];

}
//------------------------------------
// importItemWithParams
//------------------------------------
- (void)importItemWithParams:(NSDictionary*)params
{

	id targetItem = [params valueForKey:@"targetItem"];
	NSArray *filePaths = [params valueForKey:@"filePaths"];
	NSString *uniqueId = [targetItem valueForKey:@"uniqueId"];

	// import
	if([fileItemManager importItemWithFilePaths:filePaths
									targetItem:targetItem
									targetType:IMPORT_TARGET_PLAYLIST
		] == YES
	){
		// same target
		if([uniqueId isEqualToString:previousUniqueId_]){
			// reload
			[self searchItemWithItem:targetItem];
		}
	}
}
//------------------------------------
// setControlButton
//------------------------------------
- (void)setControlButtonEnable
{
	BOOL canAdd = [playlistTreeController canAdd];
	int selectedRow = [olvPlaylist selectedRow];
	BOOL isDisplay = [olvPlaylist isDisplay];

	if(canAdd == YES && isDisplay == YES){
		[btnAdd setEnabled:YES];
	}else{
		[btnAdd setEnabled:NO];
	}

	if(selectedRow >= 0 && isDisplay == YES){
		[btnRemove setEnabled:YES];
	}else{
		[btnRemove setEnabled:NO];
	}

}
//------------------------------------
// frameDidResize
//------------------------------------
-(void)frameDidResize:(NSNotification *)notification
{
	[self setControlButtonEnable];
}
//--------------------------------------
// isSelectItem
//--------------------------------------
- (BOOL)isSelectItem
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if([[item valueForKey:@"isFolder"] boolValue] == NO){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectFolder
//--------------------------------------
- (BOOL)isSelectFolder
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if([[item valueForKey:@"isFolder"] boolValue] == YES){
			ret = YES;
		}
	}

	return ret;
}

//------------------------------------
// isKeyMainWindow
//------------------------------------
- (BOOL)isKeyMainWindow
{
	return [mainWindow isKeyWindow];
}
//------------------------------------
// canAddPlaylist
//------------------------------------
- (BOOL)canAddPlaylist
{
	if( [self isKeyMainWindow] == YES &&
		[self isDisplayPlaylist] == YES){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// isDisplayPlaylist
//------------------------------------
- (BOOL)isDisplayPlaylist
{
	return [olvPlaylist isDisplay];
}

//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[previousUniqueId_ release];
	[super dealloc];
}
@end
