#import "ViewPrefs.h"
#import "DialogExtension.h"
#import "FileAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewPrefs

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	//
	// set state from user default
	//
	// window position
	[self setWindowPosition:prefsWindow key:@"rectWindowPrefs"];
	[self setWindowPosition:helpWindow key:@"rectWindowHelp"];

	// set default player
	NSString *filePath = [self defaultStringValue:@"optDefaultAppPlayerPath"];
	if(![filePath isEqualToString:@""]){
		// set app name
		[txtDefaultAppName setStringValue:[[NSFileManager defaultManager] displayNameAtPath:filePath]];
		// set image
		[imageDefaultApp setImage:[[NSWorkspace sharedWorkspace] iconForFile:filePath]];
	}

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:prefsWindow];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:helpWindow];

}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// openPrefsWindow
//------------------------------------
- (IBAction)openPrefsWindow:(id)sender
{
	[prefsWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// openPrefsWindowWithTabIdentifier
//------------------------------------
- (IBAction)openPrefsWindowWithTabIdentifier:(id)sender
{
	if([sender representedObject]){
		NSString *identifier = [sender representedObject];
		[self openTabViewWithIdentifier:identifier];
	}
}

//------------------------------------
// openHelpWindow
//------------------------------------
- (IBAction)openHelpWindow:(id)sender
{
	[helpWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// selectDefaultAppPlayer
//------------------------------------
- (IBAction)selectDefaultAppPlayer:(id)sender
{

	NSString *defaultPath = [self defaultStringValue:@"optDefaultAppPlayerPath"];
	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
														isDir:NO
														isMultiSel:NO
														isPackage:NO
														isAlias:YES
														canCreateDir:NO
														defaultPath:defaultPath
							];

	// set default player
	if([filePaths count] > 0){
		[self setDefaultAppPlayer:filePaths];
	}
}
//------------------------------------
// changeWindowTheme
//------------------------------------
- (IBAction)changeWindowTheme:(id)sender
{
	[mainWindow changeWindowTheme:nil];
//	[playerWindow changeWindowTheme:nil];
	[logWindow changeWindowTheme:nil];
	
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// setDefaultAppPlayer
//------------------------------------
- (BOOL)setDefaultAppPlayer:(NSArray*)filePaths
{

	if([filePaths count] <= 0){
		return NO;
	}

	NSString *filePath = [self resolveAliasFileWithPath:[filePaths objectAtIndex:0]];
	NSString *appName = [[NSFileManager defaultManager] displayNameAtPath:filePath];

	// not app
	if(![[[filePath pathExtension] lowercaseString] isEqualToString:@"app"]){
		[self displayMessage:@"alert"
			messageText:@"Selected file is not application."
			infoText:[NSString stringWithFormat:@"File \"%@\".", appName]
			btnList:@"Cancel"
		];
		return NO;
	}

	// set default
	[self setDefaultStringValue:filePath key:@"optDefaultAppPlayerPath"];

	// set app name
	[txtDefaultAppName setStringValue:appName];
	// set image
	[imageDefaultApp setImage:[[NSWorkspace sharedWorkspace] iconForFile:filePath]];

	return YES;
}
//------------------------------------
// openTabViewWithIdentifier
//------------------------------------
- (void)openTabViewWithIdentifier:(NSString*)identifier
{
	if([tabViewPrefs indexOfTabViewItemWithIdentifier:identifier] != NSNotFound){
		[tabViewPrefs selectTabViewItemWithIdentifier:identifier];
	}
	[prefsWindow makeKeyAndOrderFront:self];
}

//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	if([notification object] == prefsWindow){
		[self saveWindowRect:prefsWindow key:@"rectWindowPrefs"];
	}
	else if([notification object] == helpWindow){
		[self saveWindowRect:helpWindow key:@"rectWindowHelp"];
	}
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}

@end
